@@ -132,7 +132,8 @@
 
 (defun mew-addrbook-clean-up ()
   (remove-hook 'kill-emacs-hook 'mew-addrbook-clean-up)
-  (mew-lisp-save mew-alias-auto-file mew-alias-auto-alist)
+  (when mew-alias-auto-alist
+    (mew-lisp-save mew-alias-auto-file mew-alias-auto-alist))
   (setq mew-alias-auto-alist nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -279,7 +280,7 @@ returned."
 	    (mew-insert-file-contents chr)
 	    (setq files (cons chr files))))
 	(goto-char (point-max))))
-    ;; remove commets
+    ;; remove comments
     (goto-char (point-min))
     (while (re-search-forward cregex nil t)
       (setq beg (match-beginning 0))
@@ -301,7 +302,7 @@ returned."
       (end-of-line)
       (forward-char 1)
       (delete-backward-char 1))
-    ;; replace ", " to "\0" inside/ouside quote.
+    ;; replace ", " to "\0" inside/outside quote.
     (goto-char (point-min))
     (while (re-search-forward ",[ \t]+" nil t)
       (replace-match ",\0" nil t))
@@ -332,7 +333,7 @@ returned."
 (defun mew-addrbook-strsafe (var)
   (if (or (string= "" var) (string= "*" var))
       nil
-    (mew-replace-character var 0 32)))
+    (mew-replace-character var 0 mew-sp)))
 
 (defun mew-addrbook-make-alist ()
   (let (alias colon addrs nick name alist)
@@ -366,39 +367,34 @@ returned."
 executed with '\\[universal-argument]', it will add personal information.  Otherwise,
 it will add an alias."
   (interactive "P")
-  (mew-summary-display nil)
   (let ((fld (mew-summary-folder-name))
 	(msg (mew-summary-message-number2))
-	(buf (mew-buffer-message))
 	from shortname addrs name)
     (save-excursion
-      (set-buffer buf)
-      (unless (mew-msghder-p)
-	(setq buf (mew-cache-hit fld msg 'must-hit))
-	(set-buffer buf))
-      (setq from (mew-header-get-value mew-from:))
-      (if (null from)
+      (mew-summary-set-message-buffer fld msg)
+      (setq from (mew-header-get-value mew-from:)))
+    (if (null from)
+	(message "No address to be registered")
+      ;; assuming From: contains just one address
+      (setq addrs (mew-addrstr-parse-address from))
+      (if (mew-is-my-address mew-regex-my-address-list addrs)
+	  (if personalinfo
+	      (setq addrs (car (mew-header-parse-address-list
+				(list mew-to:))))
+	    (setq addrs (mapconcat 'identity
+				   (mew-header-parse-address-list
+				    (list mew-to: mew-cc:))
+				   ",")))
+	(when (string-match "\\(.*\\)<.*>" from)
+	  (setq name (match-string 1 from))
+	  (setq name (mew-chop name)))
+	(if mew-addrbook-strip-domainpart
+	    (setq shortname (downcase (mew-addrstr-extract-user addrs)))
+	  (setq shortname (downcase addrs))))
+      (if (not addrs)
 	  (message "No address to be registered")
-	;; assuming From: contains just one address
-	(setq addrs (mew-addrstr-parse-address from))
- 	(if (mew-is-my-address mew-regex-my-address-list addrs)
- 	    (if personalinfo
- 		(setq addrs (car (mew-header-parse-address-list
- 				  (list mew-to:))))
-	      (setq addrs (mapconcat 'identity
-				     (mew-header-parse-address-list
-				      (list mew-to: mew-cc:))
-				     ",")))
- 	  (when (string-match "\\(.*\\)<.*>" from)
-	    (setq name (match-string 1 from))
-	    (setq name (mew-chop name)))
- 	  (if mew-addrbook-strip-domainpart
- 	      (setq shortname (downcase (mew-addrstr-extract-user addrs)))
- 	    (setq shortname (downcase addrs))))
- 	(if (not addrs)
- 	    (message "No address to be registered")
- 	  (mew-addrbook-prepare-template
-	   personalinfo shortname addrs nil name))))))
+	(mew-addrbook-prepare-template
+	 personalinfo shortname addrs nil name)))))
 
 (defun mew-addrbook-prepare-template (personalinfop shortname addrs &optional nickname name)
   (delete-other-windows)
@@ -567,7 +563,7 @@ reflected."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2003 Mew developing team.
+;; Copyright (C) 1999-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
