@@ -119,9 +119,9 @@
     (while (< i len)
       (setq char (aref str i))
       (cond
-       ((char-equal char 32)
+       ((char-equal char mew-sp)
 	(aset ret j ?_))
-       ((and (> char 32)
+       ((and (> char mew-sp)
 	     (< char 126)
 	     (not (char-equal char ?=))
 	     (not (char-equal char ??))
@@ -180,7 +180,7 @@
 	  (setq char (aref qpstr i))
 	  (cond
 	   ((char-equal char ?_)
-	    (aset ret j 32))
+	    (aset ret j mew-sp))
 	   ((char-equal char key)
 	    (aset ret j (+ (* (mew-hexchar-to-int (aref qpstr (1+ i))) 16)
 			   (mew-hexchar-to-int (aref qpstr (+ i 2)))))
@@ -243,7 +243,7 @@
            (mew-header-encode-string1
 	    charset b-or-q fun hcs (substring str 0 med) key-len)
            (mew-header-encode-string1
-	    charset b-or-q fun hcs (substring str med nil))))
+	    charset b-or-q fun hcs (substring str med))))
       (list encoded-word))))
 
 (defun mew-header-encode-split-string (str)
@@ -258,7 +258,7 @@
 			(cons (substring str start beg) ret))))
       (setq start end))
     (if (/= start (length str))
-	(setq ret (cons (substring str start nil) ret)))
+	(setq ret (cons (substring str start) ret)))
     (nreverse ret)))
 
 (defun mew-header-encode-comma-text (str)
@@ -377,7 +377,7 @@
        ;; route
        ((char-equal c ?<)
 	(mew-header-encode-cond2 nil)
-	(if (or (char-equal (char-before (point)) 32)
+	(if (or (char-equal (char-before (point)) mew-sp)
 		(char-equal (char-before (point)) ?\t))
 	    (insert "<")
 	  (insert " <"))
@@ -431,7 +431,7 @@
        ;; end of group
 
        ;; space
-       ((or (char-equal c 32) (char-equal c ?\t))
+       ((or (char-equal c mew-sp) (char-equal c ?\t))
 	(cond
 	 ((or (eq status 'ascii) (eq status 'space))
 	  (insert (substring str bound i)) ;; 'ascii
@@ -449,13 +449,13 @@
 	(catch 'comma
 	  (while (< i len)
 	    (setq c (aref str i))
-	    (if (or (char-equal c 32) (char-equal c ?\t) (char-equal c ?\n))
+	    (if (or (char-equal c mew-sp) (char-equal c ?\t) (char-equal c ?\n))
 		() ;; loop
 	      (throw 'comma nil))
 	    (setq i (1+ i))))
 	;; get back to the end of white spaces
 	(setq bound i)
-	(setq c 32)
+	(setq c mew-sp)
 	(setq i (1- i))
 	(setq status 'space))
        ;; end of comma
@@ -508,7 +508,7 @@
 		      (delete-char 1)
 		      (insert "\t"))))
 	    ;; Ugh!
-	    (if (re-search-forward "[ \t]" limit2 t) ;; hold on ayway
+	    (if (re-search-forward "[ \t]" limit2 t) ;; hold on anyway
 		(progn
 		  (backward-char)
 		  (insert "\n")
@@ -729,7 +729,7 @@ That is, each line may be more than 75."
 	  ;;
 	  (goto-char (point-min))
 	  ;; Perfect regular expression for quoted strings.
-	  ;; See "Marstering Regular Expressions", O'REILLY
+	  ;; See "Mastering Regular Expressions", O'REILLY
 	  (while (re-search-forward "\"\\([^\"\\\\]\\|\\\\.\\)*\"" nil t)
 	    (goto-char (match-beginning 0))
 	    (insert "\n")
@@ -830,9 +830,7 @@ That is, each line may be more than 75."
       (setq max (1+ max))
       (while ext
 	(setq ea (car ext))
-	(setq paramname (nth 0 ea))
-	(setq paramvalue (nth 1 ea))
-	(setq num (nth 2 ea))
+	(mew-set '(paramname paramvalue num) ea)
 	(if (setq entry (assoc paramname ext-sort))
 	    (progn
 	      (aset (nth 1 entry) num paramvalue)
@@ -848,9 +846,7 @@ That is, each line may be more than 75."
       (while ext-sort
 	(setq ea (car ext-sort))
 	(setq paramvalue nil)
-	(setq paramname (nth 0 ea))
-	(setq entry (nth 1 ea))
-	(setq charset (nth 2 ea))
+	(mew-set '(paramname entry charset) ea)
 	(setq num 0)
 	(catch 'concat-loop
 	  (while (< num max)
@@ -891,7 +887,7 @@ That is, each line may be more than 75."
 	  (if asterisk
 	      (setq section 0)
 	    (setq section nil))
-	(setq section (string-to-int (substring section 1 nil))))
+	(setq section (string-to-number (substring section 1))))
       (if (null asterisk)
 	  ;; delete quote
 	  (progn
@@ -936,9 +932,9 @@ That is, each line may be more than 75."
   value)
 
 (defun mew-param-sanity-check (ent)
-  (let* ((param (nth 0 ent))
-	 (value (nth 1 ent))
-	 (new (mew-header-sanity-check-string value)))
+  (let (param value new)
+    (mew-set '(param value) ent)
+    (setq new (mew-header-sanity-check-string value))
     (unless (string= value new) (mew-decode-warning-params param 'ctl))
     (list param new)))
 
@@ -946,7 +942,7 @@ That is, each line may be more than 75."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2003 Mew developing team.
+;; Copyright (C) 1997-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
