@@ -133,6 +133,14 @@
 
 ;;
 
+(defun mew-ssl-cert-directory (&optional case)
+  (mew-cfent-value case "ssl-cert-directory" mew-ssl-cert-directory))
+
+(defun mew-ssl-verify-level (&optional case)
+  (mew-cfent-value case "ssl-verify-level" mew-ssl-verify-level))
+
+;;
+
 (defun mew-smtp-server (&optional case)
   (mew-cfent-value case "smtp-server" mew-smtp-server))
 
@@ -345,15 +353,18 @@
 (defun mew-content-type (&optional case)
   (mew-capitalize (mew-cfent-value case "content-type" mew-content-type)))
 
-(defun mew-use-retrieve (&optional case)
-  (mew-cfent-value case "use-retrieve" mew-use-retrieve))
-
 (defun mew-imap-spam-field (&optional case)
   (mew-cfent-value case "imap-spam-field" mew-imap-spam-field))
 
 (defun mew-imap-spam-word (&optional case)
   (mew-cfent-value case "imap-spam-word" mew-imap-spam-word))
 
+(defun mew-imap-spam-pattern (&optional case)
+  (mew-cfent-value case "imap-spam-pattern" mew-imap-spam-pattern))
+
+(defun mew-imap-spam-folder (&optional case)
+  (mew-cfent-value case "imap-spam-folder" mew-imap-spam-folder))
+
 ;;
 
 (defun mew-proto (&optional case)
@@ -392,61 +403,27 @@
 (defun mew-use-old-pgp (&optional case)
   (mew-cfent-value case "use-old-pgp" mew-use-old-pgp))
 
+;;
+
+(defun mew-inbox-action-alist (&optional case)
+  (mew-cfent-value case "inbox-action-alist" mew-inbox-action-alist))
+
 ;;;
 ;;; Setting case
 ;;;
 
-(defvar mew-case-input nil
-  "\\<mew-summary-mode-map>This is used for
-1. '\\[mew-summary-retrieve]' to select a POP server (or something)
-2. Biff to select a server (or somthing)
-3. If 'mew-use-case-input-completion' is non-nil, this value 
-   is automatically inserted when inputing a folder>")
+(defvar mew-case nil)
 
-(defvar mew-case-output nil
-  "This is used to
-1. a draft. 
-   It is used to complete Newsgroups, 
-                 choose default Content-Type: for a file to be attached,
-                 choose a signature file, 
-                 select a SMTP server (or something).
-2. flushing a message queue.")
-
-(defun mew-case-set-both ()
+(defun mew-case-set ()
   "Set the case both for input and output."
   (interactive)
-  (let ((case (mew-input-case mew-case-input "Input/Output")))
-    (setq mew-case-input case)
-    (setq mew-case-output case)))
+  (setq mew-case (mew-input-case mew-case)))
 
-(defun mew-case-set-input ()
-  "Set the case for input. If 'mew-case-synchronize' is
-non-nil, set the case for output, too."
+(defun mew-summary-set-case ()
+  "Set the case."
   (interactive)
-  (if mew-case-synchronize
-      (mew-case-set-both)
-    (setq mew-case-input (mew-input-case mew-case-input "Input"))))
-
-(defun mew-case-set-output ()
-  "Set the case for output. If 'mew-case-synchronize' is
-non-nil, set the case for input, too."
-  (interactive)
-  (if mew-case-synchronize
-      (mew-case-set-both)
-    (setq mew-case-output (mew-input-case mew-case-output "Output"))))
-
-(defun mew-summary-set-case (&optional output)
-  "Set the case. 
- (1) If 'mew-case-synchronize' is non-nil, set the case both for
- input and output.
- (2) If called with '\\[universal-argument]', set the case for output.
- (3) Otherwise, set the case for input."
-  (interactive "P")
   (let (case inbox)
-    (mew-summary-or-thread
-     (if output
-	 (setq case (mew-case-set-output))
-       (setq case (mew-case-set-input))))
+    (setq case (mew-case-set))
     (save-excursion
       (let ((bufs mew-buffers))
 	(while bufs
@@ -473,12 +450,11 @@ fields in the header according to the new value."
   (interactive "P")
   (let ((old-case (mew-tinfo-get-case)) new-case)
     (if arg
-	(setq new-case (mew-input-case old-case "This draft" 'edit))
+	(setq new-case (mew-input-case old-case 'edit))
       (setq new-case (mew-draft-get-case-by-guess))
       (if mew-case-guess-addition
 	  (setq new-case (mew-draft-add-case old-case new-case)))
-      (setq new-case
-	    (mew-input-case (or new-case mew-case-output) "This draft")))
+      (setq new-case (mew-input-case (or new-case mew-case))))
     (mew-tinfo-set-case new-case)
     (mew-draft-mode-name (mew-tinfo-get-hdr-file))
     (mew-draft-replace-fields old-case)
@@ -546,7 +522,7 @@ fields in the header according to the new value."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2003 Mew developing team.
+;; Copyright (C) 1999-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
