@@ -56,8 +56,9 @@
   `((,mew-ct-pge mew-pgp-decrypt mew-pgp-ver mew-prog-pgp)))
 
 (defvar mew-decode-multipart-signed-switch
-  `((,mew-ct-pgs mew-pgp-verify   mew-pgp-ver   mew-prog-pgp)
-    (,mew-ct-sms mew-smime-verify mew-smime-ver mew-prog-smime)))
+  `((,mew-ct-pgs mew-pgp-verify    mew-pgp-ver   mew-prog-pgp)
+    (,mew-ct-sms mew-smime-verify  mew-smime-ver mew-prog-smime)
+    (,mew-ct-xsms mew-smime-verify mew-smime-ver mew-prog-smime)))
 
 ;;
 
@@ -105,11 +106,11 @@
 over 'mew-header-veil-count' are covered with invisible veils.
 This commands toggles visibility of these lines."
   (interactive)
-  (when mew-use-header-veil
+  (when (and mew-use-header-veil (get-buffer (mew-buffer-message)))
     (mew-summary-msg-or-part
      (save-excursion
        (set-buffer (mew-buffer-message))
-       (when (mew-msghder-p)
+       (when (mew-msghdr-p)
 	 (let* ((win (get-buffer-window (current-buffer)))
 		(pos (window-start win)))
 	   (mew-toggle-header-veil (mew-minfo-get-veil-to))
@@ -235,7 +236,7 @@ Return the existence MIME-Version: and the value of Subject:."
 	 (visibles (make-list (length mew-field-spec) nil))
 	 (prop mew-use-highlight-header)
 	 key beg med subj from contents others
-	 key-face val-face N nspec visiblep mimep fld)
+	 key-face val-face N nspec visiblep mimep fld ext ext-str ext-face)
     (mew-decode-narrow-to-header
      (while (not (eobp))
        (if (not (looking-at mew-keyval))
@@ -296,7 +297,19 @@ Return the existence MIME-Version: and the value of Subject:."
 	     (while (looking-at mew-lwsp+)
 	       (forward-line)
 	       (put-text-property (match-end 0) (1- (point))
-				  'face val-face)))
+				  'face val-face))
+	     (when (setq ext (mew-nspec-extraface nspec))
+	       (save-restriction
+		 (narrow-to-region med (point))
+		 (while ext
+		   (goto-char (point-min))
+		   (setq ext-str (car ext))
+		   (setq ext (cdr ext))
+		   (setq ext-face (car ext))
+		   (setq ext (cdr ext))
+		   (while (re-search-forward ext-str nil t)
+		     (put-text-property (match-beginning 0) (match-end 0) 'face ext-face)))
+		 (goto-char (point-max)))))
 	   (cond
 	    ((null nspec) ;; others
 	     ;; This must be "buffer-substring".
@@ -351,8 +364,7 @@ Return a part syntax after moving the beginning of the content body."
 	 (mew-header-goto-next)
 	 (setq attr (assoc key mew-mime-fields))
 	 (when attr
-	   (setq n (nth 1 attr))
-	   (setq act (nth 2 attr))
+	   (mew-set '(nil n act) attr)
 	   (cond
 	    ((eq act 'analyze)
 	     (setq value (mew-param-decode
@@ -484,7 +496,8 @@ Return a part syntax after moving the beginning of the content body."
 	(or (mew-syntax-get-cd syntax) (mew-syntax-set-cd syntax subj))
 	;; (point-min), (point) - 1, (point), (point-max)
 	(mew-highlight-body-region (point) (point-max))
-	(mew-decode-syntax-rfc822 syntax))))))) ;; return value
+	(mew-decode-syntax-rfc822
+	 syntax (not (mew-xinfo-get-text-body))))))))) ;; return value
 
 ;;;
 ;;; Decoding singlepart
@@ -602,7 +615,7 @@ Return a part syntax after moving the beginning of the content body."
 	  ;; do nothing
 	  )
 	 (t
-	  ;; xxx how about message/partinal?
+	  ;; xxx how about message/partial?
 	  (mew-syntax-set-ct syntax mew-type-apo)
 	  ))))
      ;; Multipart, decoding is not required
@@ -625,9 +638,6 @@ Return a part syntax after moving the beginning of the content body."
 	    (setq syntax (mew-decode-multipart-encrypted syntax cnt))))
 	 (t
 	  (setq syntax (mew-decode-multipart syntax cnt nil))))))
-     ;; S/MIME encrypted or signed
-     ((string-match mew-ct-sme ct)
-      (setq syntax (mew-smime-decrypt-or-verify syntax cnt ctl cte)))
      ;; Others
      ((string= mew-ct-trh ct)
       (save-restriction
@@ -716,7 +726,7 @@ Return a part syntax after moving the beginning of the content body."
 	(save-excursion
 	  (unless break (forward-line -1))
 	  (beginning-of-line)
-	  (forward-char -1) ;; skip the preceeding CRLF
+	  (forward-char -1) ;; skip the preceding CRLF
 	  ;; the end of the part
 	  (save-restriction
 	    (narrow-to-region start (point))
@@ -827,7 +837,7 @@ Return a part syntax after moving the beginning of the content body."
 			   " does not exist")))
      (t
       (setq file3result (funcall func file1 file2))
-      (setq file3 (nth 0 file3result) result (nth 1 file3result))))
+      (mew-set '(file3 result) file3result)))
     ;;
     (if (and func existp (file-exists-p file3))
 	(mew-flet 
@@ -902,7 +912,7 @@ Return a part syntax after moving the beginning of the content body."
     ;;
     (delete-region end1 (point-max))
     ;; Now the signed part only
-    ;; Call protocl function
+    ;; Call protocol function
     (cond
      (unknown
       (setq result (concat "unknown protocol " proto)))
@@ -966,7 +976,7 @@ Return a part syntax after moving the beginning of the content body."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
