@@ -41,7 +41,9 @@
   (make-local-variable 'mail-header-separator)
   (setq mail-header-separator mew-header-separator)
   (make-local-variable 'comment-start)
-  (setq comment-start ">")
+  (setq comment-start mew-comment-start)
+  (make-local-variable 'comment-start-skip)
+  (setq comment-start-skip mew-comment-start-skip)
   (make-local-hook 'after-change-functions)
   (add-hook 'after-change-functions 'mew-draft-dynamic-highlight nil 'local)
   (if (boundp 'write-file-functions)
@@ -50,7 +52,16 @@
   (make-local-variable 'after-save-hook)
   (when mew-require-final-newline
     (make-local-variable 'require-final-newline)
-    (setq require-final-newline t)))
+    (setq require-final-newline t))
+  (when (featurep 'dnd)
+    (make-variable-buffer-local 'x-dnd-test-function)
+    (make-variable-buffer-local 'dnd-protocol-alist)
+    (setq x-dnd-test-function 'mew-draft-dnd-test-function)
+    (setq dnd-protocol-alist
+	  (append '(("^file:///" . mew-draft-dnd-handle-local-file)
+		    ("^file://"  . mew-draft-dnd-handle-file)
+		    ("^file:"    . mew-draft-dnd-handle-local-file))
+		  dnd-protocol-alist))))
 
 (defun mew-draft-mode (&optional encrypted)
   "\\<mew-draft-mode-map>
@@ -126,7 +137,7 @@ To send a draft, type \\[mew-draft-make-message] and \\[mew-draft-send-message].
 \\[mew-attach-find-new-file]	Open a new file into a buffer on '.'.
 \\[mew-attach-external-body]	Input external-body on '.'.
 \\[mew-attach-audio]	Sampling voice and insert as audio file on '.'.
-\\[mew-attach-pgp-public-key]	Extract the PGP key for the inputed user on '.'.
+\\[mew-attach-pgp-public-key]	Extract the PGP key for the specified user on '.'.
 \\[mew-attach-description]	Input a description(Content-Description:).
 \\[mew-attach-disposition]	Change the file name(Content-Disposition:).
 \\[mew-attach-type]	Change the data type(Content-Type:).
@@ -269,7 +280,7 @@ Moreover, '~/.mew-fib' supports aliases like;
     ;; draft buffer
     (mew-set-buffer-cs mew-cs-m17n)
     ;; copy config, first
-    (mew-tinfo-set-case mew-case-output)
+    (mew-tinfo-set-case mew-case)
     (when mew-use-other-frame-for-draft
       (mew-tinfo-set-other-frame t)
       ;; to ensure to cite a message from summary frame.
@@ -354,7 +365,7 @@ the Body: field."
       ;; All addresses inserted on To: are appended to del.
       (cond
        ((null to) (mew-draft-header-insert mew-to: ""))
-       ((stringp to) ;; To: inputed from the mini-buffer.
+       ((stringp to) ;; To: specified from the mini-buffer.
 	;; do not check to is mine. Cc: is also string
 	;; We believe that user never specifies the same address of To: to Cc:.
 	(mew-draft-header-insert mew-to: to))
@@ -363,7 +374,7 @@ the Body: field."
 	(setq del (mew-insert-address-list mew-to: to del t))))
       (cond
        ((null cc) ()) ;; do nothing 
-       ((stringp cc) ;; Cc: inputed from the mini-buffer.
+       ((stringp cc) ;; Cc: specified from the mini-buffer.
 	(mew-draft-header-insert mew-cc: cc))
        ((listp cc) ;; Cc: collected by reply.
 	(mew-insert-address-list mew-cc: cc del nil))))
@@ -455,7 +466,7 @@ citation prefix and label.
 		   (not (and mew-cite-ignore-mouse-region
 			     (mew-mouse-region-p))))
 	      (setq beg (region-beginning) end (region-end)))
-	     ((mew-msghder-p)
+	     ((mew-msghdr-p)
 	      ;; header exists in Message mode
 	      (mew-header-goto-body)
 	      (setq beg (point) end (point-max)))
@@ -514,7 +525,7 @@ citation prefix and label.
 		   (not (and mew-cite-ignore-mouse-region
 			     (mew-mouse-region-p))))
 	      (setq beg (region-beginning) end (region-end)))
-	     ((mew-msghder-p)
+	     ((mew-msghdr-p)
 	      ;; header exists in Message mode. Skip the header
 	      ;; because we will concatenate it to cite later.
 	      (mew-header-goto-body)
@@ -522,12 +533,14 @@ citation prefix and label.
 	     (t
 	      (setq beg (point-min) end (point-max)))))
 	  (setq cite (mew-buffer-substring beg end)))
+	;;
 	;; concat the header
-	;; see also mew-summary-reply
+	;;
 	(setq tbuf (or nonmewbuf
 		       (save-excursion
 			 (when (get-buffer msg-buf)
-			   (if (mew-msghder-p) (current-buffer))))
+			   (set-buffer msg-buf)
+			   (if (mew-msghdr-p) (current-buffer))))
 		       ;; header exists only in cache if multipart
 		       (mew-cache-hit fld msg)))
 	(if (get-buffer tbuf)
@@ -545,7 +558,7 @@ citation prefix and label.
       ;;
 
       ;; Append message-id to In-Reply-To:
-      (if (and irt-msgid (mew-msghder-p))
+      (if (and irt-msgid (mew-msghdr-p))
           (save-excursion
             (let* ((mew-references-max-count nil)
 		   (irt (mew-header-get-value mew-in-reply-to:))
@@ -708,7 +721,7 @@ If attachments exist and mew-signature-as-lastpart is *non-nil*,
 the file is attached to the last part. Otherwise, the file is 
 inserted into the body. If mew-signature-insert-last is *non-nil*,
 the file is inserted to the end of the body. Otherwise, inserted
-the cursor position. If excuted with '\\[universal-argument]',
+the cursor position. If executed with '\\[universal-argument]',
 you can set the case."
   (interactive "P")
   (let (case sigfile)
@@ -804,12 +817,20 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 (defun mew-draft-make-message (&optional privacy signer)
   "Compose a MIME message then put it into a queue folder."
   (interactive)
-  (mew-draft-process-message 'queue privacy signer))
+  (if (and (interactive-p) ;; prevent the loop
+	   mew-use-old-pgp
+	   mew-protect-privacy-with-old-pgp-signature)
+      (mew-pgp-sign-message)
+    (mew-draft-process-message 'queue privacy signer)))
 
 (defun mew-draft-send-message ()
   "Compose a MIME message then send it."
   (interactive)
-  (mew-draft-process-message 'send))
+  (if (and (interactive-p) ;; just in case
+	   mew-use-old-pgp
+	   mew-protect-privacy-with-old-pgp-signature)
+      (mew-pgp-sign-message)
+    (mew-draft-process-message 'send)))
 
 (defun mew-draft-process-message (action &optional privacy signer)
   (run-hooks 'mew-make-message-hook)
@@ -874,6 +895,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
 		  (mew-delete-file mdi)
 		  (setq msg (mew-smtp-queue case "from Draft mode"))
+		  (mew-remove-buffer buf)
 		  (if (eq action 'send)
 		      (mew-smtp-send-message case queue (list msg))))
 	      (setq err t))))
@@ -918,6 +940,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
 		  (mew-delete-file mdi)
 		  (setq msg (mew-nntp2-queue case "from Draft mode"))
+		  (mew-remove-buffer buf)
 		  (if (eq action 'send)
 		      (mew-nntp2-send-message case postq (list msg))))
 	      (setq err t))))
@@ -962,11 +985,63 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 ;; backward-compatibility
 (defalias 'mew-draft-send-letter 'mew-draft-send-message)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Privacy
+;;;
+
+(defvar mew-draft-privacy-method 'pgp)
+
+(defun mew-draft-sign-message (&optional arg)
+  "Sign the entire draft. Input your passphrase."
+  (interactive "P")
+  (cond
+   ((eq mew-draft-privacy-method 'pgp)
+    (mew-pgp-sign-message arg))
+   ((eq mew-draft-privacy-method 'smime)
+    (mew-smime-sign-message arg))
+   (t
+    (message "'%s' is not supported" mew-draft-privacy-method))))
+
+(defun mew-draft-encrypt-message ()
+  "Encrypt the entire draft with PGP."
+  (interactive)
+  (cond
+   ((eq mew-draft-privacy-method 'pgp)
+    (mew-pgp-encrypt-message))
+;;   ((eq mew-draft-privacy-method 'smime)
+;;    (mew-smime-encrypt-message))
+   (t
+    (message "'%s' is not supported" mew-draft-privacy-method))))
+
+(defun mew-draft-sign-encrypt-message (&optional arg)
+  "Sign then encrypt the entire draft. Input your passphrase."
+  (interactive "P")
+  (cond
+   ((eq mew-draft-privacy-method 'pgp)
+    (mew-pgp-sign-encrypt-message arg))
+;;   ((eq mew-draft-privacy-method 'smime)
+;;    (mew-smime-sign-encrypt-message arg))
+   (t
+    (message "'%s' is not supported" mew-draft-privacy-method))))
+
+
+(defun mew-draft-encrypt-sign-message (&optional arg)
+  "Encrypt then sign the entire draft. Input your passphrase."
+  (interactive "P")
+  (cond
+   ((eq mew-draft-privacy-method 'pgp)
+    (mew-pgp-encrypt-sign-message arg))
+;;   ((eq mew-draft-privacy-method 'smime)
+;;    (mew-smime-encrypt-sign-message arg))
+   (t
+    (message "'%s' is not supported" mew-draft-privacy-method))))
+
 (provide 'mew-draft)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
