@@ -21,18 +21,20 @@
   (cdr (mew-assoc-case-equal cte switch 0)))
 
 (defvar mew-encode-multipart-encrypted-switch
-  '(("application/pgp-encrypted" mew-pgp-encrypt)
-    ("application/x-pkcs7-mime" mew-smime-encrypt)))
+  `((,mew-ct-pge mew-pgp-encrypt)))
 
 (defvar mew-encode-multipart-signed-switch
-  '(("application/pgp-signature" mew-pgp-sign)
-    ("application/x-pkcs7-signature" mew-smime-sign)))
+  `((,mew-ct-pgs mew-pgp-sign mew-pgp-canonicalize)
+    (,mew-ct-sms mew-smime-sign)))
 
 ;;
 
 (defsubst mew-encode-get-security-func (proto switch)
   (nth 1 (mew-assoc-case-equal proto switch 0)))
 
+(defsubst mew-encode-get-canonicalize-func (proto switch)
+  (nth 2 (mew-assoc-case-equal proto switch 0)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; MIME encoder
@@ -523,6 +525,12 @@
   ;; Only parts to be signed FIRST are limited to 7bit.
   (and privacy (mew-case-equal mew-ct-mls (car (car privacy)))))
 
+(defvar mew-encode-max-line-length 990)
+;; RFC 2822 defines 1000 but Sendmail uses 990
+
+(defvar mew-ask-encoding t)
+(defvar mew-default-encoding mew-b64)
+
 (defun mew-encode-mime-body (ctl cte file)
   ;; If file is 't', target is buffered.
   ;; text should be buffered
@@ -537,7 +545,7 @@
                      mew-prog-mime-encode-text-switch
                    mew-prog-mime-encode-switch))
          (beg (point))
-	 buffer cs opt)
+	 buffer cs opt last)
     (if (and (stringp file)
 	     (setq buffer (get-file-buffer file))
 	     (buffer-modified-p buffer))
@@ -555,7 +563,7 @@
 	(mew-frwlet
 	 (or cs mew-cs-autoconv) mew-cs-dummy
 	 (mew-insert-file-contents file)))
-      ;; A user may specify charset to convery JIS X 0201 Katakana.
+      ;; A user may specify charset to convey JIS X 0201 Katakana.
       ;; So, we need to avoid the sanity check.
       (unless charset
 	(if mew-use-charset-sanity-check
@@ -579,14 +587,29 @@
 	    (setq cte (mew-charset-to-cte charset)))
 	(if (or (null cte) (mew-case-equal cte mew-bin))
 	    ;; unknown charset
-	    (setq cte mew-b64))
+	    (setq cte mew-default-encoding))
 	(cond
 	 (mew-inherit-7bit
 	  (if (mew-case-equal cte mew-8bit)
-	      (setq cte mew-b64)))
+	      (setq cte mew-default-encoding)))
 	 (mew-use-8bit
 	  (if (mew-encode-charset-8bitp charset)
-	      (setq cte mew-8bit)))))))
+	      (setq cte mew-8bit))))
+	(when (or (mew-case-equal cte mew-7bit) (mew-case-equal cte mew-8bit))
+	  (save-excursion
+	    (goto-char beg)
+	    (mew-set-buffer-multibyte nil)
+	    (catch 'long-line
+	      (while t
+		(setq last (point))
+		(if (/= (forward-line) 0) (end-of-line))
+		(when (>= (- (point) last) mew-encode-max-line-length) ;; including LF
+		  (if (not mew-ask-encoding)
+		      (setq cte mew-default-encoding)
+		    (setq cte (mew-input-encoding)))
+		  (throw 'long-line nil))
+		(if (eobp) (throw 'long-line nil))))
+	    (mew-set-buffer-multibyte t))))))
      ((string= ct mew-ct-msg)
       (if (mew-encode-file-8bitp file)
 	  (setq cte mew-8bit)
@@ -658,8 +681,7 @@
 	 charset-cte charset bodybeg cst ask-cst)
     (setq charset-cte (mew-encode-mime-body ctl cte (or buffered file)))
     (goto-char beg)
-    (setq charset (nth 0 charset-cte))
-    (setq cte (nth 1 charset-cte))
+    (mew-set '(charset cte) charset-cte)
     (setq ctl (mew-delete "icharset" ctl))
     (when charset
       (setq ctl (mew-syntax-get-params ctl))
@@ -755,13 +777,13 @@
   (format mew-default-boundary
 	  string
 	  (mew-replace-character
-	   (mew-replace-character (current-time-string) 32 ?_) ?: ?_)
+	   (mew-replace-character (current-time-string) mew-sp ?_) ?: ?_)
 	  (mew-random-string 3 t)))
 
 (defun mew-encode-multipart (syntax path depth &optional buffered)
   (let* ((boundary
 	  (mew-boundary-get ;; 0 is nil for Next_Part
-	   (if (> depth 0) (format "BOUNDARY%s" (int-to-string depth)))))
+	   (if (> depth 0) (format "BOUNDARY%s" (number-to-string depth)))))
 	 (fullname (expand-file-name (mew-syntax-get-file syntax) path))
 	 (ctl (mew-syntax-get-ct syntax))
 	 (ct (mew-syntax-get-value ctl 'cap))
@@ -815,8 +837,8 @@
     (let (proto ct)
       (while privacy
 	(goto-char (point-min)) 
-	(setq ct (mew-capitalize (nth 0 (car privacy))))
-	(setq proto (nth 1 (car privacy)))
+	(mew-set '(ct proto) (car privacy))
+	(setq ct (mew-capitalize ct))
 	(setq privacy (cdr privacy))
 	(cond 
 	 ((string= mew-ct-mle ct)
@@ -826,7 +848,7 @@
 
 (defun mew-security-multipart-boundary (depth)
    (if depth
-       (mew-boundary-get (format "Security_Multipart%s" (int-to-string depth)))
+       (mew-boundary-get (format "Security_Multipart%s" (number-to-string depth)))
      (mew-boundary-get "Security_Multipart")))
 
 (defun mew-save-transfer-form (beg end retain)
@@ -872,8 +894,7 @@
        (mew-encode-error
 	(format "unknown error for %s. Check %s, anyway" 
 		mew-ct-mle mew-temp-dir))))
-    (setq file2 (nth 0 fc) cte2 (nth 1 fc) file3 (nth 2 fc) cte3 (nth 3 fc))
-    (setq errmsg (nth 4 fc))
+    (mew-set '(file2 cte2 file3 cte3 errmsg) fc)
     (if errmsg
 	(progn
 	  (mew-delete-file file1)
@@ -893,7 +914,7 @@
       (mew-encode-singlepart 
        (mew-encode-syntax-single file2 (list proto) cte2))
       (insert (format "\n--%s\n" boundary))
-      ;; Insert encrpted body
+      ;; Insert encrypted body
       (mew-encode-singlepart 
        (mew-encode-syntax-single file3 mew-type-apo cte3))
       (insert (format "\n--%s--\n" boundary))
@@ -907,7 +928,9 @@
   (let* ((boundary (mew-security-multipart-boundary depth))
 	 (switch mew-encode-multipart-signed-switch) ;; save length
 	 (func (mew-encode-get-security-func proto switch))
+	 (canon-func (mew-encode-get-canonicalize-func proto switch))
 	 file1 file2 micalg cte2 fmc errmsg ct2 cdp2)
+    (if (fboundp canon-func) (funcall canon-func))
     (setq file1 (mew-save-transfer-form (point-min) (point-max) 'retain))
     ;; The narrowed region still the ORIGINAL part (i.e. line breaks are LF)
     ;; Call the protocol function
@@ -918,9 +941,7 @@
        (mew-encode-error
 	(format "unknown error for %s. Check %s, anyway" 
 		mew-ct-mls mew-temp-dir))))
-    (setq file2 (nth 0 fmc) cte2 (nth 1 fmc) micalg (nth 2 fmc))
-    (setq errmsg (nth 3 fmc))
-    (setq ct2 (nth 4 fmc) cdp2 (nth 5 fmc))
+    (mew-set '(file2 cte2 micalg errmsg ct2 cdp2) fmc)
     (if errmsg
 	(progn
 	  (mew-delete-file file1)
@@ -939,7 +960,7 @@
       (mew-header-insert mew-cte: "7bit")
       (insert (format "\n--%s\n" boundary))
       (goto-char (point-max))
-      ;; After the sigend part
+      ;; After the signed part
       (insert (format "\n--%s\n" boundary))
       (mew-encode-singlepart 
        (mew-encode-syntax-single file2 (or ct2 (list proto)) cte2 nil nil cdp2))
@@ -1111,7 +1132,7 @@
 	(save-excursion
 	  (forward-line -1)
 	  (beginning-of-line) ;; just in case
-	  (forward-char -1) ;; skip the preceeding CRLF
+	  (forward-char -1) ;; skip the preceding CRLF
 	  ;; the end of the part
 	  (save-restriction
 	    (narrow-to-region start (point))
@@ -1184,7 +1205,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
