@@ -31,13 +31,18 @@
   (memq major-mode '(mew-draft-mode mew-header-mode)))
 
 (defsubst mew-thread-p ()
-  (mew-vinfo-get-thread-p))
+  (mew-vinfo-get-parent-folder))
 
 (defsubst mew-summary-or-virtual-p ()
   (or (mew-summary-p) (mew-virtual-p)))
 
-(defsubst mew-summary-or-thread-p ()
-  (or (mew-summary-p) (mew-thread-p)))
+(defsubst mew-virtual-for-one-summary ()
+  (and (mew-virtual-p) (= (length (mew-vinfo-get-flds)) 1)))
+
+(defsubst mew-pickable-p ()
+  (or (mew-summary-p)
+      (mew-thread-p)
+      (mew-virtual-for-one-summary)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -50,9 +55,9 @@
 (defun mew-folder-recursive-match (key folder)
   "Initial substring match for folders.
 If FOLDER is a sub-folder of KEY or KEY itself, t is returned."
-  ;; file-name-as-directory should be first because
-  ;; the path separator may be regex-non-safe.
-  (let ((regex (if (string-match "^[+-=%]+$" key)
+  ;; file-name-as-directory should be first
+  ;; because the path separator may be regex-non-safe.
+  (let ((regex (if (string-match "^[-+%$]$" key)
 		   (mew-folder-regex key)
 		 (mew-folder-regex (file-name-as-directory key)))))
     (string-match regex (file-name-as-directory folder))))
@@ -425,21 +430,21 @@ also ignored."
     (catch 'loop
       (while (< i j)
 	(setq c (aref str i))
-	(if (or (char-equal c 32) (char-equal c ?\t))
+	(if (or (char-equal c mew-sp) (char-equal c ?\t))
 	    (setq i (1+ i))
 	  (throw 'loop nil))))
     (setq j (1- j))
     (catch 'loop
       (while (< i j)
 	(setq c (aref str j))
-	(if (or (char-equal c 32) (char-equal c ?\t))
+	(if (or (char-equal c mew-sp) (char-equal c ?\t))
 	    (setq j (1- j))
 	  (throw 'loop nil))))
     (substring str i (1+ j))))
 
 (defun mew-quote-string (str qchar targets)
   (let* ((len (length str))
-	(ret (make-string (* len 2) 32))
+	(ret (make-string (* len 2) mew-sp))
 	(i 0) (j 0) c)
     (while (< i len)
       (setq c (aref str i))
@@ -531,14 +536,31 @@ If case is \"default\", it is not prepended."
 (defsubst mew-folder-to-thread (folder)
   (concat mew-folder-virtual folder))
 
-(defsubst mew-thread-to-folder (folder)
-  (mew-folder-string folder))
+(defsubst mew-physical-folder (folder)
+  ;; Upper functions may use string-match.
+  ;; So, string-match must not be used here.
+  (if (eq (aref folder 0) ?*)
+      (setq folder (substring folder 1)))
+  (if (eq (aref folder (1- (length folder))) ?*)
+      (setq folder (substring folder 0 -1)))
+  folder)
+
+;;
+
+(defun mew-summary-physical-folder ()
+  (cond
+   ((mew-thread-p)
+    (mew-vinfo-get-parent-folder))
+   ((mew-virtual-for-one-summary)
+    (car (mew-vinfo-get-flds)))
+   (t
+    (mew-summary-folder-name 'ext))))
 
 ;;
 
 (defsubst mew-local-to-dir (folder)
   (if (string-match "^[+]" folder)
-      (substring folder 1 nil)
+      (substring folder 1)
     folder))
 
 ;;
@@ -623,7 +645,9 @@ If case is \"default\", it is not prepended."
       (mew-concat-folder (mew-nntp-folder case) subdir))
      ((mew-folder-imapp folder)
       (mew-concat-folder (mew-imap-folder case)
-			 (mew-imap-expand-folder case subdir)))
+			 (mew-imap-expand-folder
+			  case
+			  (mew-imap-utf-7-encode-string subdir))))
      (t folder))))
 
 (defun mew-expand-folder (folder &optional message)
@@ -673,8 +697,8 @@ If case is \"default\", it is not prepended."
 	 msgs nums)
     (when (file-directory-p dir)
       (setq msgs (mew-dir-messages dir))
-      (setq nums (sort (mapcar 'string-to-int msgs) '<))
-      (mapcar 'int-to-string nums))))
+      (setq nums (sort (mapcar 'string-to-number msgs) '<))
+      (mapcar 'number-to-string nums))))
 
 (defun mew-folder-new-message (folder &optional num-only cache)
   (let* ((dir (mew-expand-folder folder))
@@ -685,11 +709,11 @@ If case is \"default\", it is not prepended."
     (when (file-directory-p dir)
       (setq msgs (mew-dir-messages dir regex))
       (while msgs
-	(setq cur (string-to-int (car msgs)))
+	(setq cur (string-to-number (car msgs)))
 	(if (> cur max) (setq max cur))
 	(setq msgs (cdr msgs)))
       (setq max (1+ max))
-      (setq maxfile (int-to-string max))
+      (setq maxfile (number-to-string max))
       (setq maxpath (mew-expand-folder folder maxfile))
       (while (file-exists-p maxpath) ;; xxx
 	;; If NFS is used, readdir() may fail. Emacs 20 does not
@@ -699,16 +723,16 @@ If case is \"default\", it is not prepended."
 	(setq msgs (mew-dir-messages dir regex))
 	(setq max 0)
 	(while msgs
-	  (setq cur (string-to-int (car msgs)))
+	  (setq cur (string-to-number (car msgs)))
 	  (if (> cur max) (setq max cur))
 	  (setq msgs (cdr msgs)))
 	(setq max (1+ max))
-	(setq maxfile (int-to-string max))
+	(setq maxfile (number-to-string max))
 	(setq maxpath (mew-expand-folder folder maxfile)))
       (while (get-file-buffer maxpath) ;; xxx
 	;; file not exist but there is a buffer.
 	(setq max (1+ max))
-	(setq maxfile (int-to-string max))
+	(setq maxfile (number-to-string max))
 	(setq maxpath (mew-expand-folder folder maxfile)))
       (while (file-exists-p maxpath)
 	(setq maxfile (read-string (format "%s/%s exists. Input a message number: " max folder)))
@@ -852,9 +876,16 @@ If case is \"default\", it is not prepended."
   (or (mew-which (concat elfile ".el") load-path)
       (mew-which (concat elfile ".elc") load-path)))
 
+(defvar mew-which-exec-suffixes '(""))
+
 (defun mew-which-exec (execfile)
-  (or (mew-which execfile exec-path)
-      (mew-which (concat execfile ".exe") exec-path)))
+  (let ((suffixes mew-which-exec-suffixes)
+	file)
+    (catch 'detect
+      (while suffixes
+	(when (setq file (mew-which (concat execfile (car suffixes)) exec-path))
+	  (throw 'detect file))
+	(setq suffixes (cdr suffixes))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -924,11 +955,13 @@ Does not examine containing directories for links."
       (substring file 2)
     file))
 
-(defun mew-set-file-modes (path)
+(defun mew-get-file-modes (path)
   (let* ((dir (file-name-directory path))
-	 (dirmode (mew-file-get-mode dir))
-	 (mode (logand dirmode mew-file-mode-mask)))
-    (set-file-modes path mode)))
+	 (dirmode (mew-file-get-mode dir)))
+    (logand dirmode mew-file-mode-mask)))
+
+(defun mew-set-file-modes (path)
+  (set-file-modes path (mew-get-file-modes path)))
 
 (defun mew-delete-file (file)
   (if (and (stringp file) (file-exists-p file)) (delete-file file)))
@@ -1133,7 +1166,8 @@ and sets buffer-file-coding-system."
 
 (defmacro mew-filter (&rest body)
   `(let ((pbuf (process-buffer process)) ;; MUST use 'process'
-	 (obuf (buffer-name)))
+	 (obuf (buffer-name))
+	 (inhibit-eol-conversion nil)) ;; \r\n as is
      (if (and (bufferp pbuf)
 	      (buffer-name pbuf)) ;; check a killed buffer
 	 ;; must use buffer-name instead of current-buffer
@@ -1372,23 +1406,23 @@ by side-effect."
   "\\([0-9]+\\)[ \t]+\\([a-zA-Z]+\\)[ \t]+\\([0-9]+\\)[ \t]+\\([0-9]+\\):\\([0-9]+\\)\\(:\\([0-9]+\\)\\)?\\([ \t]+\\([-+0-9a-zA-Z]+\\)\\)?")
 
 (defmacro mew-time-rfc-day  ()
-  '(string-to-int (substring s (match-beginning 1) (match-end 1))))
+  '(string-to-number (substring s (match-beginning 1) (match-end 1))))
 
 (defmacro mew-time-rfc-mon  ()
   '(substring s (match-beginning 2) (match-end 2)))
 
 (defmacro mew-time-rfc-year ()
-  '(string-to-int (substring s (match-beginning 3) (match-end 3))))
+  '(string-to-number (substring s (match-beginning 3) (match-end 3))))
 
 (defmacro mew-time-rfc-hour ()
-  '(string-to-int (substring s (match-beginning 4) (match-end 4))))
+  '(string-to-number (substring s (match-beginning 4) (match-end 4))))
 
 (defmacro mew-time-rfc-min  ()
-  '(string-to-int (substring s (match-beginning 5) (match-end 5))))
+  '(string-to-number (substring s (match-beginning 5) (match-end 5))))
 
 (defmacro mew-time-rfc-sec  ()
   '(if (match-beginning 7)
-       (string-to-int (substring s (match-beginning 7) (match-end 7)))
+       (string-to-number (substring s (match-beginning 7) (match-end 7)))
      0))
 
 ;; returns seconds
@@ -1398,7 +1432,7 @@ by side-effect."
 	     int)
 	 (cond
 	  ((string-match "^[-+][0-9]+$" tmzn)
-	   (setq int (string-to-int tmzn))
+	   (setq int (string-to-number tmzn))
 	   (+ (* (/ int 100) 3600) (* (% int 100) 60)))
 	  ((setq int (mew-time-tmzn-str-to-int tmzn))
 	   (* int 3600))
@@ -1508,7 +1542,7 @@ by side-effect."
 
 (defsubst mew-port-sanity-check (port)
   (if (string-match "^[0-9]+$" port)
-      (string-to-int port)
+      (string-to-number port)
     port))
 
 (defsubst mew-dot-insert ()
@@ -1562,11 +1596,20 @@ by side-effect."
 (defsubst mew-timing ()
   (sit-for 0.01))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun mew-set (vars vals)
+  (while vars
+    (when (car vars)
+      (set (car vars) (car vals)))
+    (setq vars (cdr vars))
+    (setq vals (cdr vals))))
+
 (provide 'mew-func)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2003 Mew developing team.
+;; Copyright (C) 1997-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
