@@ -36,8 +36,9 @@
     "delete"
     "case" "mailbox" "msgid" "max" "tag"
     "wrk" "jobs" "namespace" "authl"
-    "spam" "spam-field" "spam-word" "new-mailbox"
-    "my-prefix" "specials"))
+    "spam" "spam-field" "spam-word" "spam-pattern" 
+    "new-mailbox" "my-prefix" "specials" "bytes"
+    "thread-info" "status-buf"))
 
 (mew-info-defun "mew-imap-" mew-imap-info-list)
 
@@ -92,7 +93,7 @@
   (or (mew-imap-get-ssh-process pnm) (mew-imap-get-ssl-process pnm)))
  
 (defun mew-imap-command-capability (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm)
+  (mew-net-status (mew-imap-get-status-buf pnm)
 		  "Auth'ing"
 		  nil
 		  (mew-imap-secure-p pnm))
@@ -159,6 +160,7 @@
 	(setq info (mew-lisp-load data))
 	(rename-file data data-work 'override)
 	(setq mailbox (car info))
+	(setq mailbox (mew-imap-utf-7-encode-string mailbox))
 	(setq mailbox (mew-imap-expand-mailbox case mailbox))
 	(mew-imap-set-mailbox pnm mailbox)
 	(setq info (cdr info))
@@ -204,7 +206,8 @@
     (if (and mew-imap-skip-uidl
 	     (or (eq directive 'exec) (eq directive 'jobs) (eq directive 'get)))
 	(mew-imap-command-dispatch pro pnm directive refs rmvs nil)
-      (mew-net-status bnm "Checking" nil (mew-imap-secure-p pnm))
+      (mew-net-status (mew-imap-get-status-buf pnm)
+		      "Checking" nil (mew-imap-secure-p pnm))
       (cond
        ((eq directive 'biff)
 	(setq max (mew-lisp-load (mew-expand-folder bnm mew-imap-msgid-file))))
@@ -213,7 +216,7 @@
 	    (setq max (mew-lisp-load
 		       (mew-expand-folder bnm mew-imap-msgid-file))))))
       (if max
-	  (setq max (string-to-int max))
+	  (setq max (string-to-number max))
 	(setq max 0))
       (mew-imap-set-max pnm max)
       (mew-imap-process-send-string
@@ -242,11 +245,11 @@
 	 ((member uid rmvs)
 	  (setq dels (cons uid dels)))))
        ((eq directive 'biff)
-	(if (and uid (> (string-to-int uid) max))
+	(if (and uid (> (string-to-number uid) max))
 	    (setq rtrs (cons (list uid siz) rtrs))))
        ((eq directive 'scan)
 	(if (and uid (or range ;; all, last:n
-			 (> (string-to-int uid) max))) ;; update
+			 (> (string-to-number uid) max))) ;; update
 	    (setq rtrs (cons (list uid siz) rtrs))))
        ((eq directive 'sync)
 	(if uid (setq hlds (cons uid hlds))))
@@ -263,7 +266,7 @@
 	      (setq dels (cons uid dels)))
 	  (setq rtr (list uid siz))
 	  (setq rtrs (cons rtr rtrs)))))))
-    (mew-imap-set-msgid pnm (nth 0 (car rtrs))) ;; 'scan
+    (mew-imap-set-msgid pnm (or (nth 0 (car rtrs)) max)) ;; 'scan
     (mew-imap-set-uidl pnm uidl) ;; 'inc
     ;; last:n
     (when (and (eq directive 'scan) (numberp range))
@@ -297,8 +300,7 @@
       (while ents
 	(setq ent  (car ents))
 	(setq ents (cdr ents))
-	(setq uid  (nth 0 ent))
-	(setq del  (nth 2 ent))
+	(mew-set '(uid nil del) ent)
 	(setq flds (nthcdr 3 ent))
 	(if del (setq dels2 (cons uid dels2)))
 	(while flds
@@ -329,7 +331,10 @@
     (mew-imap-set-rttl pnm (length rtrs))
     (mew-imap-set-rtrs pnm rtrs)
     (mew-imap-set-dttl pnm (length dels))
-    (setq dels (mew-net-msg-group dels))))
+    ;; We can't use mew-net-msg-group since a member will be
+    ;; deleted from dels.
+    ;; (setq dels (mew-net-msg-group dels))
+    ))
   ;;
   (mew-imap-set-dels pnm dels)
   (mew-imap-set-dgttl pnm (length dels))
@@ -378,7 +383,7 @@
 
 (defun mew-imap-command-copy (pro pnm)
   (mew-net-status1
-   (mew-imap-get-bnm pnm) "Refiling"
+   (mew-imap-get-status-buf pnm) "Refiling"
    (mew-imap-get-rgttl pnm) (mew-imap-get-rgcnt pnm) (mew-imap-secure-p pnm))
   (let* ((case (mew-imap-get-case pnm))
 	 (spam (mew-imap-get-spam pnm))
@@ -396,7 +401,8 @@
 	  (mew-imap-command-logout pro pnm)))
       (mew-imap-set-rgcnt pnm (1+ (mew-imap-get-rgcnt pnm)))
       (mew-imap-set-rtrs pnm (cdr rtrs))
-      (setq dst (mew-imap-expand-mailbox case dst))
+      (setq dst (mew-imap-expand-mailbox
+		 case (mew-imap-utf-7-encode-string dst)))
       (mew-imap-set-rfl pnm rtr)
       (if spam
 	  (mew-imap-process-send-string pro pnm "COPY %s \"%s\"" uid dst)
@@ -409,7 +415,8 @@
 	 (dst (nth 0 rtr))) ;; 'exec or 'jobs
     (mew-imap-set-rgcnt pnm (1- (mew-imap-get-rgcnt pnm)))
     (mew-imap-set-rtrs pnm (cons rtr rtrs))
-    (setq dst (mew-imap-expand-mailbox case dst))
+    (setq dst (mew-imap-expand-mailbox
+	       case (mew-imap-utf-7-encode-string dst)))
     (mew-imap-process-send-string pro pnm "CREATE \"%s\"" dst)))
 
 (defun mew-imap-command-wmbx (pro pnm)
@@ -444,7 +451,7 @@
 
 (defun mew-imap-command-dels (pro pnm)
   (mew-net-status1
-   (mew-imap-get-bnm pnm) "Deleting"
+   (mew-imap-get-status-buf pnm) "Deleting"
    (mew-imap-get-dgttl pnm) (mew-imap-get-dgcnt pnm) (mew-imap-secure-p pnm))
   (let ((dels (mew-imap-get-dels pnm))
 	(spam (mew-imap-get-spam pnm))
@@ -467,22 +474,28 @@
 
 (defun mew-imap-command-search (pro pnm)
   (mew-net-status
-   (mew-imap-get-bnm pnm) "Filtering" nil (mew-imap-secure-p pnm))
-  (mew-imap-process-send-string
-   pro pnm "SEARCH HEADER %s %s"
-   (mew-imap-get-spam-field pnm)
-   (mew-imap-get-spam-word pnm)))
+   (mew-imap-get-status-buf pnm) "Filtering" nil (mew-imap-secure-p pnm))
+  (let ((pattern (mew-imap-get-spam-pattern pnm)))
+    (if pattern
+	(mew-imap-process-send-string pro pnm "SEARCH %s" pattern)
+      (mew-imap-process-send-string
+       pro pnm "SEARCH HEADER %s %s"
+       (mew-imap-get-spam-field pnm)
+       (mew-imap-get-spam-word pnm)))))
 
 (defun mew-imap-command-post-search (pro pnm)
   (let* ((case (mew-imap-get-case pnm))
-	 (trash (mew-imap-trash-folder case))
+	 (trash (or (mew-imap-spam-folder case) (mew-imap-trash-folder case)))
 	 dels rtrs beg)
-    (when (looking-at "\\* SEARCH ")
+    (goto-char (point-min))
+    (mew-crlf-to-lf)
+    (goto-char (point-min))
+    (when (looking-at "^\\* SEARCH ")
       (setq beg (match-end 0))
       (goto-char (point-max))
       (forward-line -1)
       (if (bolp) (forward-char -1))
-      (setq dels (mew-split (mew-buffer-substring beg (point)) 32))
+      (setq dels (mew-split (mew-buffer-substring beg (point)) mew-sp))
       (when trash
 	(setq rtrs (copy-sequence dels))
 	(mew-imap-set-rttl pnm (length rtrs))
@@ -522,7 +535,7 @@
       (mew-imap-command-fetch pro pnm)))))
 
 (defun mew-imap-command-fetch (pro pnm)
-  (mew-net-status2 (mew-imap-get-bnm pnm)
+  (mew-net-status2 (mew-imap-get-status-buf pnm)
 		   (mew-imap-get-rttl pnm)
 		   (mew-imap-get-rcnt pnm)
 		   (nth 1 (car (mew-imap-get-rtrs pnm)))
@@ -551,7 +564,7 @@
      ((and (eq directive 'scan) (not get-body))
       (mew-imap-set-truncated pnm t)
       (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822.HEADER" uid))
-     ((or (= lim 0) (< (string-to-int siz) lim))
+     ((or (= lim 0) (< (string-to-number siz) lim))
       (mew-imap-set-truncated pnm nil)
       (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822" uid))
      (t
@@ -566,8 +579,10 @@
 	 (rtr (car rtrs))
 	 (uid (nth 0 rtr))
 	 (siz (nth 1 rtr))
+	 (del (nth 2 rtr))
 	 (fld-msg (nth 3 rtr)) ;; 'exec, 'jobs, 'scan, ...
 	 (truncated (mew-imap-get-truncated pnm))
+	 (case (mew-imap-get-case pnm))
 	 fld msg vec file msg-file lmsg mark)
     (cond
      ((null fld-msg)
@@ -575,31 +590,11 @@
      ((stringp fld-msg)
       (setq fld fld-msg))
      ((listp fld-msg)
-      (setq fld (nth 0 fld-msg))
-      (setq msg (nth 1 fld-msg))
+      (mew-set '(fld msg) fld-msg)
       (setq lmsg msg)))
-    ;;
-    (goto-char (point-min))
-    (while (looking-at "^\\*")
-      (forward-line))
-    (delete-region (point-min) (point))
-    ;; deleting ")"
-    (goto-char (point-max))
-    ;; skip the last response
-    (forward-line -2)
-    ;; skip untagged responses if any
-    (while (looking-at "^\\*")
-      (forward-line -1))
-    ;; We cannot count up the octet number of literal in Emacs.
-    ;; Assuming that the message ends with CRLF, the cursor now locates
-    ;; either
-    ;;   *)
-    ;; or
-    ;;   * ATTRIBUTE VAL ...)
-    ;; This also assumes that all attributes are stored in a line.
-    (delete-region (point) (point-max))
     ;; line delimiters
-    (mew-eol-fix-for-read)
+    (goto-char (point-min))
+    (mew-crlf-to-lf)
     (cond
      ((eq directive 'scan)
       (setq msg uid) 
@@ -611,7 +606,9 @@
     (goto-char (point-min))
     (cond
      (truncated
-      (mew-header-insert-xmu uid siz t))
+      (mew-header-insert-xmu uid siz t (and (eq directive 'inc)
+					    (or (mew-imap-get-case pnm)
+						mew-case-default))))
      ((eq directive 'scan)
       (mew-header-insert-xmu uid siz nil))
      ((and (eq directive 'get) (mew-folder-imapp (mew-case:folder-folder fld)))
@@ -634,12 +631,19 @@
       (mew-scan-set-message vec msg)
       (if (and (eq directive 'scan)
 	       (string= (mew-case:folder-folder fld) mew-imap-inbox-folder))
-	  (setq mark (mew-scan-inbox-action vec)))
+	  (setq mark (mew-scan-inbox-action vec case)))
       (mew-scan-body vec)
       (mew-set-buffer-multibyte nil)
       (mew-scan-insert-line fld vec width lmsg mark)
       (mew-imap-set-rtrs pnm (cdr rtrs))
+      (cond
+       ((eq directive 'inc)
+	(if truncated
+	    (mew-imap-set-dels pnm (delete uid (mew-imap-get-dels pnm)))))
+       ((eq directive 'get)
+	(if del (mew-imap-set-dels pnm (cons uid (mew-imap-get-dels pnm))))))
       (mew-imap-set-status pnm "fetch")
+      (mew-imap-set-bytes pnm nil)
       (mew-imap-command-fetch pro pnm))))
 
 (defun mew-imap-command-expunge (pro pnm)
@@ -662,7 +666,7 @@
     (error ())))
 
 (defun mew-imap-command-post-namespace (pro pnm)
-  (when (looking-at "\\* NAMESPACE ")
+  (when (looking-at "^\\* NAMESPACE ")
     (let* ((beg (match-end 0))
 	   (case (mew-imap-get-case pnm))
 	   (prefix-list (mew-imap-prefix-list case))
@@ -686,8 +690,7 @@
 	  (while sexp
 	    (setq ent (car sexp))
 	    (setq sexp (cdr sexp))
-	    (setq my (nth 0 ent))
-	    (setq sp (nth 1 ent))
+	    (mew-set '(my sp) ent)
 	    (when (member my prefix-list)
 	      (cond
 	       (sp
@@ -707,7 +710,7 @@
   (mew-imap-command-list pro pnm))
 
 (defun mew-imap-command-list (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm)
+  (mew-net-status (mew-imap-get-status-buf pnm)
 		  "Listing"
 		  nil
 		  (mew-imap-secure-p pnm))
@@ -721,12 +724,11 @@
 	 (namespace (mew-imap-get-namespace pnm))
 	 (specials (mew-imap-get-specials pnm))
 	 ret mailboxes friends)
-    (mew-eol-fix-for-read)
     (unless namespace
       (setq namespace (mew-imap-namespace-create nil "" "" nil)) ;; xxx separator?
       (mew-imap-set-namespace pnm namespace))
     (setq ret (mew-imap-command-post-list-tree namespace case))
-    (setq mailboxes (nth 0 ret) friends (nth 1 ret))
+    (mew-set '(mailboxes friends) ret)
     (unless case (setq case mew-case-default))
     (if specials (setq mailboxes (nconc specials mailboxes)))
     (if queue (mew-folder-insert queue mailboxes nil))
@@ -737,24 +739,28 @@
 (defun mew-imap-command-post-list-tree (namespace case)
   (let* ((case-fold-search t)
 	 (my   (format "^%s" (regexp-quote
-			      (mew-imap-namespace-my-prefix namespace))))
+			      (or (mew-imap-namespace-my-prefix namespace) ""))))
 	 (user (format "^%s" (regexp-quote
-			      (mew-imap-namespace-user-prefix namespace))))
+			      (or (mew-imap-namespace-user-prefix namespace) ""))))
 	 (sep (mew-imap-namespace-sep namespace))
 	 subfolder friendp mbx mailbox mailboxes subnm friends)
+    (goto-char (point-min))
+    (mew-crlf-to-lf)
+    (goto-char (point-min))
     (when sep
-      (setq subfolder (format "\\([^%s\n]+\\)$" sep))
+      (setq subfolder (format "\\([^%s\n\r]+\\)$" sep))
       (setq friendp (mew-imap-friend-regex sep case)))
     (while (not (eobp))
       (catch 'break
-	(when (looking-at "\\* LIST ([^)\n]*) \"?\\([^\"\n]*\\)\"? \"?\\([^\"\n]*\\)\"?")
+	(when (looking-at "^\\* LIST ([^)\n]*) \"?\\([^\"\n]*\\)\"? \"?\\([^\"\n]*\\)\"?")
 	  (unless sep
 	    ;; NAMESPACE may return NIL NIL NIL
 	    (setq sep (mew-match-string 1))
 	    (mew-imap-namespace-change-sep namespace sep)
-	    (setq subfolder (format "\\([^%s\n]+\\)$" sep))
+	    (setq subfolder (format "\\([^%s\n\r]+\\)$" sep))
 	    (setq friendp (mew-imap-friend-regex sep case)))
 	  (setq mbx (mew-match-string 2))
+	  (setq mbx (mew-imap-utf-7-decode-string mbx))
 	  (cond
 	   ((mew-imap-inbox-str-p mbx)
 	    (throw 'break nil))
@@ -806,7 +812,7 @@
       (setq len1 len2))))
 
 (defun mew-imap-command-delete (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm)
+  (mew-net-status (mew-imap-get-status-buf pnm)
 		  "Deleting"
 		  nil
 		  (mew-imap-secure-p pnm))
@@ -815,7 +821,7 @@
     (mew-imap-process-send-string pro pnm "DELETE \"%s\"" mailbox)))
 
 (defun mew-imap-command-rename (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm)
+  (mew-net-status (mew-imap-get-status-buf pnm)
 		  "Renaming"
 		  nil
 		  (mew-imap-secure-p pnm))
@@ -1020,7 +1026,8 @@
 ;;;
 
 (defsubst mew-imap-bnm-to-mailbox (bnm)
-  "case:%inbox -> inbox
+  "
+   case:%inbox -> inbox
    case:%foo   -> inbox.foo
    case:%~foo  -> user.foo
    case:%#foo  -> foo"
@@ -1053,11 +1060,12 @@
 	(if my-prefix (concat my-prefix str) str))))))
 
 (defun mew-imap-expand-folder (case str)
-  "foo   -> foo
+  "
+   foo   -> foo
    ~foo  -> user.foo
    #foo  -> #foo
 "
-  (when (stringp str)
+  (when (and (stringp str) (not (string= str "")))
     (if (char-equal (aref str 0) ?~)
 	(let* ((namespace (mew-imap-namespace case))
 	       (user-prefix (mew-imap-namespace-user-prefix namespace)))
@@ -1120,7 +1128,7 @@
 	  (or no-msg (message "Connecting to the IMAP server..."))
 	  (setq pro (open-network-stream pnm nil server sprt))
 	  (process-kill-without-query pro)
-	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
+	  (mew-set-process-cs pro mew-cs-binary mew-cs-text-for-net)
 	  (or no-msg (message "Connecting to the IMAP server...done")))
       (quit
        (or no-msg (message "Cannot connect to the IMAP server"))
@@ -1147,11 +1155,14 @@
 	 (sslp (mew-imap-ssl case))
 	 (sslport (mew-imap-ssl-port case))
 	 ;; dirty but necessary for migration
-	 (mailbox (unless (eq directive 'list) (mew-imap-bnm-to-mailbox bnm)))
+	 (mailbox (unless (eq directive 'list)
+		    (mew-imap-utf-7-encode-string
+		     (mew-imap-bnm-to-mailbox bnm))))
 	 (pnm (mew-imap-info-name case mailbox))
 	 (buf (get-buffer-create (mew-imap-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
-	 process sshname sshpro sslname sslpro lport info jobs)
+	 process sshname sshpro sslname sslpro lport info jobs tls
+	 thread-info to-buf)
     (if (mew-imap-get-process pnm)
 	(message "Another IMAP process is running. Try later")
       (cond
@@ -1163,7 +1174,8 @@
 	  (when lport
 	    (setq process (mew-imap-open pnm "localhost" lport no-msg)))))
        (sslp
- 	(setq sslpro (mew-open-ssl-stream server sslport))
+	(if (string= port sslport) (setq tls mew-tls-imap))
+ 	(setq sslpro (mew-open-ssl-stream case server sslport tls))
  	(when sslpro
  	  (setq sslname (process-name sslpro))
 	  (setq lport (mew-ssl-pnm-to-lport sslname))
@@ -1176,6 +1188,7 @@
 	    (mew-imap-exec-recover bnm))
 	(mew-summary-lock process "IMAPing" (or sshpro sslpro))
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
+	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
 	(mew-info-clean-up pnm)
 	(mew-imap-set-no-msg pnm no-msg) ;; must come here
@@ -1192,6 +1205,7 @@
 	(mew-imap-set-status pnm "greeting")
 	(mew-imap-set-directive pnm directive)
 	(mew-imap-set-bnm pnm bnm)
+	(mew-imap-set-status-buf pnm bnm)
 	(mew-imap-set-rcnt pnm 1)
 	(mew-imap-set-rgcnt pnm 1)
 	(mew-imap-set-dgcnt pnm 1)
@@ -1201,14 +1215,25 @@
 	(mew-imap-set-size pnm (mew-imap-size case))
 	(mew-imap-set-mailbox pnm mailbox)
 	(mew-imap-set-case pnm case)
+	(mew-imap-set-bytes pnm nil)
 	(cond
 	 ((eq directive 'biff)
 	  (mew-imap-set-passwd pnm (nth 0 args))) ;; password
 	 ((eq directive 'inc)
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
 	  (mew-imap-set-flush pnm (nth 0 args)) ;; no-flush
 	  (mew-imap-set-delete pnm (mew-imap-delete case)))
 	 ((eq directive 'get)
-	  (mew-imap-set-refs pnm (nth 0 args)))
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
+	  (mew-imap-set-refs pnm (nth 0 args))
+	  (setq thread-info (nth 1 args))
+	  (mew-imap-set-thread-info pnm thread-info)
+	  (setq to-buf (nth 0 thread-info))
+	  (when to-buf
+	    (mew-imap-set-status-buf pnm to-buf)
+	    (save-excursion
+	      (set-buffer to-buf)
+	      (mew-summary-lock process "IMAPing" (or sshpro sslpro)))))
 	 ((eq directive 'scan)
 	  (mew-imap-set-range pnm (nth 0 args))
 	  (mew-imap-set-get-body pnm (nth 1 args))
@@ -1216,12 +1241,17 @@
 	      (progn
 		(mew-imap-set-mdb pnm (mew-summary-mark-collect4))
 		(mew-net-folder-clean))
-	    (mew-net-invalid-cache-clean))
+	    (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
+	    (mew-imap-set-mdb pnm (mew-summary-mark-collect5))
+	    (mew-net-invalid-cache-invisible))
 	  (when (and (string= mailbox mew-imap-inbox-string)
-		     (mew-imap-spam-field case) (mew-imap-spam-word case))
+		     (or (mew-imap-spam-pattern case)
+			 (and (mew-imap-spam-field case)
+			      (mew-imap-spam-word case))))
 	    (mew-imap-set-spam pnm t)
 	    (mew-imap-set-spam-field pnm (mew-imap-spam-field case))
-	    (mew-imap-set-spam-word pnm (mew-imap-spam-word case))))
+	    (mew-imap-set-spam-word pnm (mew-imap-spam-word case))
+	    (mew-imap-set-spam-pattern pnm (mew-imap-spam-pattern case))))
 	 ((eq directive 'sync)
 	  )
 	 ((eq directive 'exec)
@@ -1263,13 +1293,18 @@
     (save-excursion
       (set-buffer (get-buffer-create mew-buffer-debug))
       (goto-char (point-max))
-      (insert (format "\n<%s>\n%s\n" label string)))))
+      (let ((start (point)))
+	(insert (format "\n<%s>\n%s\n" label string))
+	(goto-char start)
+	(mew-crlf-to-lf))
+      (goto-char (point-max)))))
 
 (defun mew-imap-filter (process string)
   (let* ((pnm (process-name process))
 	 (status (mew-imap-get-status pnm))
 	 (tag (mew-imap-get-tag pnm))
 	 (eos (format "^%s \\(OK\\|NO \\[TRYCREATE\\]\\|NO\\|BAD\\)" tag))
+	 (bytes (mew-imap-get-bytes pnm))
 	 stay next func code)
     (mew-imap-debug (upcase status) string)
     (mew-filter
@@ -1277,15 +1312,34 @@
      (goto-char (point-max))
      (mew-set-buffer-multibyte nil)
      (insert string)
-     (when (string= status "fetch")
-       (mew-net-status2 (mew-imap-get-bnm pnm)
+     (cond
+      ((string= status "fetch")
+       (mew-net-status2 (mew-imap-get-status-buf pnm)
 			(mew-imap-get-rttl pnm)
 			(mew-imap-get-rcnt pnm)
 			(nth 1 (car (mew-imap-get-rtrs pnm)))
 			nil
-			(mew-imap-secure-p pnm)))
-     (cond
-      ((and (goto-char (point-max)) (= (forward-line -1) 0) (looking-at "\\*"))
+			(mew-imap-secure-p pnm))
+       (goto-char (point-min))
+       (when (and (null bytes)
+		  (looking-at "^\\*[^{]*{\\([0-9]+\\)}"))
+	 (setq bytes (string-to-number (mew-match-string 1)))
+	 (mew-imap-set-bytes pnm bytes))
+       (when bytes
+	 (goto-char (point-min))
+	 (while (and (looking-at "^\\*") (= (forward-line) 0))
+	   (delete-region (point-min) (point))))
+       (if (or (null bytes) (< (point-max) bytes)
+	       (save-excursion (goto-char (point-max)) (not (bolp))))
+	   (setq stay t)
+	 (goto-char (point-max))
+	 (forward-line -1)
+	 (while (looking-at "^\\*") (forward-line -1))
+	 (if (not (looking-at eos))
+	     (setq stay t)
+	   (delete-region (1+ bytes) (point-max))
+	   (setq next (mew-imap-fsm-next status "OK")))))
+      ((and (goto-char (point-max)) (= (forward-line -1) 0) (looking-at "^\\*"))
        ;; untagged message
        (if (string= status "greeting")
 	   (setq next (mew-imap-fsm-next "greeting" "OK"))
@@ -1297,6 +1351,11 @@
        (setq code (mew-match-string 1))
        (setq next (mew-imap-fsm-next status code)))
       (t
+       ;; xxx what about in this case?
+       ;; * untag
+       ;; * untag
+       ;; tag
+       ;; * untag
        (setq stay t)))
      (unless stay
        (unless next (setq next "logout"))
@@ -1319,7 +1378,7 @@
 	 (rttl (mew-imap-get-rttl pnm))
 	 (dttl (mew-imap-get-dttl pnm))
 	 (jcnt (mew-imap-get-jcnt pnm))
-	 (bnm (mew-imap-get-bnm pnm))
+	 (bnm (or (mew-imap-get-bnm pnm) (current-buffer))) ;; C-c C-k
 	 (flush (mew-imap-get-flush pnm))
 	 (kils (mew-imap-get-kils pnm))
 	 (movs (mew-imap-get-movs pnm))
@@ -1329,7 +1388,8 @@
 	 (done (mew-imap-get-done pnm))
 	 (error (mew-imap-get-error pnm))
 	 (file (mew-expand-folder bnm mew-imap-msgid-file))
-	 (buf (process-buffer process)))
+	 (buf (process-buffer process))
+	 (thread-info (mew-imap-get-thread-info pnm)))
     (mew-imap-debug "IMAP SENTINEL" event)
     (set-process-buffer process nil)
     (set-buffer bnm)
@@ -1346,13 +1406,16 @@
 	  (cond
 	   ((eq directive 'exec)
 	    (mew-imap-exec-recover bnm))
+	   ((eq directive 'scan)
+	    (mew-summary-visible-buffer bnm))
 	   ((or (eq directive 'delete) (eq directive 'rename))
 	    (setq mew-summary-buffer-process-error t)))
 	  (when (mew-imap-get-dispatched pnm)
 	    (cond
 	     ((eq directive 'scan)
-	      (setq msgid (int-to-string (1- (string-to-int msgid))))
-	      (mew-lisp-save file msgid nil 'unlimit)
+	      (when msgid
+		(setq msgid (number-to-string (1- (string-to-number msgid))))
+		(mew-lisp-save file msgid nil 'unlimit))
 	      (setq leftp t))
 	     ((eq directive 'inc)
 	      ;; uidl is reversed.
@@ -1363,9 +1426,10 @@
 	    (when leftp
 	      (mew-imap-message
 	       pnm
-	       "%d message retrieved. %d messages are left due to an error."
+	       "%d message retrieved. %d messages are left due to an error"
 	       (- rttl lefts) lefts)
 	      (mew-summary-folder-cache-save))))
+      (if thread-info (mew-summary-retrieve-message-for-thread thread-info))
       (cond
        ((eq directive 'list)
 	(mew-imap-message pnm "Collecting mailbox list...done"))
@@ -1410,11 +1474,11 @@
 	 ((= rttl 1)
 	  (mew-imap-message pnm "1 message retrieved")
 	  (mew-lisp-save file msgid nil 'unlimit)
-	  (mew-summary-folder-cache-save))
+	  (mew-net-invalid-cache-clean))
 	 (t
 	  (mew-imap-message pnm "%s messages retrieved" rttl)
 	  (mew-lisp-save file msgid nil 'unlimit)
-	  (mew-summary-folder-cache-save))))
+	  (mew-net-invalid-cache-clean))))
        ((eq directive 'exec)
 	(if kils (mew-mark-exec-unlink bnm kils))
 	(when movs
@@ -1440,7 +1504,7 @@
 	    (mew-imap-message pnm "1 job processed")
 	  (mew-imap-message pnm "%d jobs processed" jcnt)))))
     ;;
-    (mew-net-status-clear (mew-imap-get-bnm pnm))
+    (mew-net-status-clear (mew-imap-get-status-buf pnm))
     (mew-info-clean-up pnm)
     (set-buffer-modified-p nil)
     (mew-summary-unlock)
@@ -1451,8 +1515,8 @@
     (unless (eq directive 'biff)
       (run-hooks 'mew-imap-sentinel-non-biff-hook))
     (run-hooks 'mew-imap-sentinel-hook)
-     (when (and mew-auto-flush-queue flush)
-       (mew-smtp-flush-queue mew-case-output))))
+    (when (and mew-auto-flush-queue flush)
+      (mew-smtp-flush-queue mew-case))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1460,26 +1524,30 @@
 ;;;
 
 (defun mew-imap-biff ()
-  (let* ((case mew-case-input)
+  (let* ((case mew-case)
 	 (inbox (mew-proto-inbox-folder nil case))
+	 (case:inbox (mew-case-folder case inbox))
 	 (tag (mew-imap-passtag2 case))
 	 passwd)
-    (save-excursion
-      (set-buffer inbox)
-      (when (and (mew-summary-exclusive-p 'no-msg)
-		 (and mew-use-cached-passwd
-		      (setq passwd (mew-passwd-get-passwd tag))))
-	(mew-imap-retrieve case 'biff inbox passwd)))))
+    (when (get-buffer case:inbox)
+      (save-excursion
+	(set-buffer case:inbox)
+	(when (and (mew-summary-exclusive-p 'no-msg)
+		   (and mew-use-cached-passwd
+			(setq passwd (mew-passwd-get-passwd tag))))
+	  (mew-imap-retrieve case 'biff case:inbox passwd))))))
 
 (defun mew-imap-check ()
   "See if messages are arrived by IMAP."
   (interactive)
-  (let* ((case mew-case-input)
-	 (inbox (mew-proto-inbox-folder nil case)))
-    (save-excursion
-      (set-buffer inbox)
-      (when (mew-summary-exclusive-p)
-	(mew-imap-retrieve case 'biff inbox)))))
+  (let* ((case mew-case)
+	 (inbox (mew-proto-inbox-folder nil case))
+	 (case:inbox (mew-case-folder case inbox)))
+    (when (get-buffer case:inbox)
+      (save-excursion
+	(set-buffer case:inbox)
+	(when (mew-summary-exclusive-p)
+	  (mew-imap-retrieve case 'biff case:inbox))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1562,7 +1630,7 @@
 
 (defun mew-imap-folder-insert (case folder)
   (let* ((sep (mew-imap-separator case))
-	 (regex (format "\\([^%s%s\n]+\\)$" sep mew-folder-imap))
+	 (regex (format "\\([^%s%s\n\r]+\\)$" sep mew-folder-imap))
 	 (mailboxes (mew-imap-folder-alist case))
 	 (friends (mew-imap-friend-folder-list case))
 	 (friendp (mew-imap-friend-regex sep case))
@@ -1599,11 +1667,66 @@
       (mew-summary-folder-cache-clean qfld)
       (mew-imap-retrieve case 'jobs qfld jobs))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; UTF-7 IMAP
+;;;
+
+(defun mew-imap-utf-7-decode-string (str)
+  (if (or (null mew-cs-utf-16be) (not (string-match "&" str)))
+      str ;; fast path
+    (let ((regex "&[A-Za-z0-9+,]+-")
+	  padn)
+      (with-temp-buffer
+	(mew-set-buffer-multibyte t)
+	(insert str)
+	(goto-char (point-min))
+	(while (re-search-forward regex nil t)
+	  (narrow-to-region (match-beginning 0) (match-end 0))
+	  (goto-char (point-min))
+	  (delete-char 1)
+	  (while (search-forward "," nil t)
+	    (replace-match "/" nil t))
+	  (goto-char (point-max))
+	  (delete-char -1)
+	  (setq padn (- 4 (% (- (point-max) (point-min)) 4)))
+	  (unless (= padn 4)
+	    (insert (make-string padn ?=)))
+	  (base64-decode-region (point-min) (point-max))
+	  (decode-coding-region (point-min) (point-max) mew-cs-utf-16be)
+	  (goto-char (point-max))
+	  (widen))
+	(buffer-substring-no-properties (point-min) (point-max))))))
+
+(defun mew-imap-utf-7-encode-string (str)
+  (if (or (null mew-cs-utf-16be) (string-match "^[ -%'-~]+$" str))
+      str ;; fast path
+    (let ((regex "[^ -%'-~]+"))
+      (with-temp-buffer
+	(mew-set-buffer-multibyte t)
+	(insert str)
+	(goto-char (point-min))
+	(while (re-search-forward regex nil t)
+	  (narrow-to-region (match-beginning 0) (match-end 0))
+	  (encode-coding-region (point-min) (point-max) mew-cs-utf-16be)
+	  (base64-encode-region (point-min) (point-max))
+	  (goto-char (point-min))
+	  (insert "&")
+	  (when (re-search-forward "=+" nil t)
+	    (delete-region (match-beginning 0) (match-end 0)))
+	  (goto-char (point-min))
+	  (while (search-forward "/" nil t)
+	    (replace-match "," nil t))
+	  (goto-char (point-max))
+	  (insert "-")
+	  (widen))
+	(buffer-substring-no-properties (point-min) (point-max))))))
+
 (provide 'mew-imap)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2003 Mew developing team.
+;; Copyright (C) 2002-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
