@@ -66,8 +66,9 @@
   (define-key mew-summary-mode-map "f"    'mew-summary-forward)
   (define-key mew-summary-mode-map "F"    'mew-summary-multi-forward)
   (define-key mew-summary-mode-map "r"    'mew-summary-resend)
-  (define-key mew-summary-mode-map "@"    'mew-summary-multi)
+  (define-key mew-summary-mode-map "$"    'mew-summary-escape)
   (define-key mew-summary-mode-map "*"    'mew-summary-review)
+  (define-key mew-summary-mode-map "\eu"  'mew-summary-unread)
   (define-key mew-summary-mode-map "y"    'mew-summary-save)
   (define-key mew-summary-mode-map "Y"    'mew-summary-cite)
   (define-key mew-summary-mode-map "u"    'mew-summary-undo)
@@ -76,13 +77,15 @@
   (define-key mew-summary-mode-map "p"    'mew-summary-display-up)
   (define-key mew-summary-mode-map "\C-n" 'mew-summary-next-line)
   (define-key mew-summary-mode-map "\C-p" 'mew-summary-previous-line)
+  (define-key mew-summary-mode-map "e"    'mew-summary-exchange-marks)
   (define-key mew-summary-mode-map "M"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "Md"   'mew-summary-mark-duplicated)
+  (define-key mew-summary-mode-map "M*"   'mew-summary-redo)
   (define-key mew-summary-mode-map "N"    'mew-summary-display-review-down)
   (define-key mew-summary-mode-map "P"    'mew-summary-display-review-up)
   (define-key mew-summary-mode-map "\en"  'mew-summary-find-keyword-down)
   (define-key mew-summary-mode-map "\ep"  'mew-summary-find-keyword-up)
-  (define-key mew-summary-mode-map "w"    'mew-summary-send)
+  (define-key mew-summary-mode-map "w"    'mew-summary-write)
   (define-key mew-summary-mode-map "W"    'mew-summary-send-to-others)
   (define-key mew-summary-mode-map "B"    'mew-summary-burst)
   (define-key mew-summary-mode-map "K"    'mew-summary-old-burst)
@@ -98,7 +101,6 @@
   (define-key mew-summary-mode-map "\C-c\C-a" 'mew-summary-addrbook-add)
   (define-key mew-summary-mode-map "\C-c\C-c" 'mew-summary-send-message)
   (define-key mew-summary-mode-map "\C-c\C-e" 'mew-summary-execute-external)
-  (define-key mew-summary-mode-map "\C-c\C-y" 'mew-summary-execute-command)
   (define-key mew-summary-mode-map "\C-c\t"   'mew-summary-find-file)
   (define-key mew-summary-mode-map "\C-c\C-f" 'mew-pgp-fetch-key)
   (define-key mew-summary-mode-map "\C-c\C-v" 'mew-pgp-select)
@@ -123,15 +125,21 @@
   (define-key mew-summary-mode-map "("    'mew-summary-thread-sibling-up)
   (define-key mew-summary-mode-map ")"    'mew-summary-thread-sibling-down)
   (define-key mew-summary-mode-map "m"	  (make-sparse-keymap))
-  (define-key mew-summary-mode-map "m@"   'mew-summary-mark-multi)
+  (define-key mew-summary-mode-map "md"   'mew-summary-mark-delete)
+  (define-key mew-summary-mode-map "m\ed" 'mew-summary-mark-unlink)
+  (define-key mew-summary-mode-map "m$"   'mew-summary-mark-escape)
   (define-key mew-summary-mode-map "m*"   'mew-summary-mark-review)
+  (define-key mew-summary-mode-map "m\eu" 'mew-summary-mark-unread)
   (define-key mew-summary-mode-map "ms"   'mew-summary-mark-swap)
+  (define-key mew-summary-mode-map "mo"   'mew-summary-mark-refile)
+  (define-key mew-summary-mode-map "mc"   'mew-summary-mark-copy)
   (define-key mew-summary-mode-map "mr"   'mew-summary-mark-regexp)
   (define-key mew-summary-mode-map "ma"   'mew-summary-mark-all)
   (define-key mew-summary-mode-map "mu"   'mew-summary-mark-undo-all)
   (define-key mew-summary-mode-map "mI"   'mew-summary-mark-retrieve-message)
   (define-key mew-summary-mode-map "ml"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "mlc"  'mew-summary-mark-local-copy)
+  (define-key mew-summary-mode-map "mli"  'mew-summary-mark-imap-copy)
   (define-key mew-summary-mode-map "V"    'mew-summary-virtual-old)
   (define-key mew-summary-mode-map "/"    'mew-summary-virtual)
   (define-key mew-summary-mode-map "?"    'mew-summary-pick-mark)
@@ -146,7 +154,7 @@
   (define-key mew-summary-mode-map "t("   'mew-thread-all-graft)
   (define-key mew-summary-mode-map "t)"   'mew-thread-all-prune)
   (define-key mew-summary-mode-map "t*"   'mew-thread-mark-review)
-  (define-key mew-summary-mode-map "t@"   'mew-thread-mark-multi)
+  (define-key mew-summary-mode-map "t$"   'mew-thread-mark-escape)
   (define-key mew-summary-mode-map "td"   'mew-thread-mark-delete)
   (define-key mew-summary-mode-map "to"   'mew-thread-mark-refile)
   (define-key mew-summary-mode-map "tc"   'mew-thread-mark-copy)
@@ -162,11 +170,29 @@
   (define-key mew-summary-mode-map "zo"   'mew-summary-toggle-pgp)
   (define-key mew-summary-mode-map "zv"   'mew-summary-toggle-header-veil)
   (define-key mew-summary-mode-map "zw"   'mew-summary-toggle-warning)
+  (define-key mew-summary-mode-map "zi"   'mew-summary-toggle-invisible)
+  (define-key mew-summary-mode-map "z "   'mew-summary-toggle-mark-regex)
   (define-key mew-summary-mode-map "l"	  (make-sparse-keymap))
   (define-key mew-summary-mode-map "lc"   'mew-summary-local-copy)
+  (define-key mew-summary-mode-map "li"   'mew-summary-imap-copy)
   (define-key mew-summary-mode-map "lh"   'mew-summary-learn-ham)
   (define-key mew-summary-mode-map "ls"   'mew-summary-learn-spam)
   (define-key mew-summary-mode-map "="    'mew-summary-info)
+  (define-key mew-summary-mode-map "k"	  (make-sparse-keymap))
+  (define-key mew-summary-mode-map "k?"   'mew-nmz-search-mark)
+  (define-key mew-summary-mode-map "k/"   'mew-nmz-virtual)
+  (define-key mew-summary-mode-map "kV"   'mew-nmz-virtual)
+  (define-key mew-summary-mode-map "kg"   'mew-nmz-gcnmz)
+  (define-key mew-summary-mode-map "km"   'mew-nmz-mknmz)
+  (define-key mew-summary-mode-map "kM"   'mew-nmz-mknmz-all-folders)
+  (define-key mew-summary-mode-map "ks"   'mew-nmz-mknmz-save-mewmknmz)
+  (define-key mew-summary-mode-map "kK"   'mew-nmz-mknmz-kill-process)
+  (define-key mew-summary-mode-map "ku"   'mew-nmz-mark-unindexed)
+  (define-key mew-summary-mode-map "k^"   'mew-nmz-search-parent)
+  (define-key mew-summary-mode-map "kp"   'mew-nmz-search-parent)
+  (define-key mew-summary-mode-map "kn"   'mew-nmz-search-child)
+  (define-key mew-summary-mode-map "kN"   'mew-nmz-namazu)
+  (define-key mew-summary-mode-map "kj"   'mew-nmz-original-message)
   ;;
   ;; not provided in Virtual mode
   ;;
@@ -179,11 +205,7 @@
   (define-key mew-summary-mode-map "X"    'mew-summary-exec-one)
   (define-key mew-summary-mode-map "lx"   'mew-summary-exec-offline)
   (define-key mew-summary-mode-map "lX"   'mew-summary-exec-offline-one)
-  (define-key mew-summary-mode-map "mo"   'mew-summary-mark-refile)
-  (define-key mew-summary-mode-map "mc"   'mew-summary-mark-copy)
-  (define-key mew-summary-mode-map "md"   'mew-summary-mark-delete)
   (define-key mew-summary-mode-map "m/"   'mew-summary-mark-virtual)
-  (define-key mew-summary-mode-map "m\ed" 'mew-summary-mark-unlink)
   (define-key mew-summary-mode-map "mxd"   'mew-summary-exec-delete)
   (define-key mew-summary-mode-map "mx\ed" 'mew-summary-exec-unlink)
   (define-key mew-summary-mode-map "mxo"   'mew-summary-exec-refile)
@@ -211,8 +233,8 @@
     ["Delete"       mew-summary-delete       (mew-summary-p)]
     ["Refile"       mew-summary-refile       (mew-summary-case1)]
     ["Refile again" mew-summary-refile-again (mew-summary-case1)]
-    ["Mark multi"   mew-summary-mark-multi   t]
     ["Mark review"  mew-summary-mark-review  t]
+    ["Mark escape"  mew-summary-mark-escape  t]
     ["Undo"         mew-summary-undo         t]
     ["Undo all"     mew-summary-undo-all     t]
     ["Execute"      mew-summary-exec         (mew-summary-p)]
@@ -240,8 +262,10 @@
      ["Multi forward" mew-summary-multi-forward (mew-summary-case1)]
      )
     ("Select"
-     ["Search then Mark" mew-summary-search-mark (mew-summary-or-thread-p)]
-     ["Virtual mode"     mew-summary-virtual     (mew-summary-or-virtual-p)]
+     ["Pick then mark"     mew-summary-pick-mark   (mew-pickable-p)]
+     ["Grep then mark"     mew-summary-grep-mark   (mew-pickable-p)]
+     ["Command then mark"  mew-summary-cmd-mark    (mew-pickable-p)]
+     ["Virtual mode"       mew-summary-virtual     (mew-summary-or-virtual-p)]
      )
     ("Misc"
      ["Clean +trash"         mew-summary-clean-trash      t]
@@ -280,10 +304,13 @@
   (define-key mew-message-mode-map "a"    'mew-message-reply)
   (define-key mew-message-mode-map "A"    'mew-message-reply-with-citation)
   (define-key mew-message-mode-map "f"    'mew-message-forward)
-  (define-key mew-message-mode-map "r"    'mew-message-resend))
+  (define-key mew-message-mode-map "r"    'mew-message-resend)
+  (define-key mew-message-mode-map "k"	  (make-sparse-keymap))
+  (define-key mew-message-mode-map "kp"   'mew-nmz-search-msgid-at-point)
+  (define-key mew-message-mode-map "kr"   'mew-nmz-search-msgid-region))
 
 (defvar mew-message-mode-menu-spec
-  '("Mew/message"
+  '("Mew"
     ["Next part"    mew-message-next-msg     t]
     ["Prev part"    mew-message-prev-msg     t]
     ["Next page"    mew-message-next-page    t]
@@ -296,16 +323,19 @@
 ;;;
 
 (defun mew-draft-share-keymap (symmap)
+  (if (featurep 'meadow)
+      (define-key (symbol-value symmap) [drag-n-drop] 'mew-draft-dnd-for-meadow))
+  (define-key (symbol-value symmap) "\C-x\C-s" 'mew-draft-save-buffer)
   (define-key (symbol-value symmap) "\C-c\C-m" 'mew-draft-make-message)
   (define-key (symbol-value symmap) "\C-c\C-c" 'mew-draft-send-message)
   (define-key (symbol-value symmap) "\C-c\C-o" 'mew-draft-set-case)
   (define-key (symbol-value symmap) "\C-c\C-a" 'mew-draft-prepare-attachments)
   (define-key (symbol-value symmap) "\C-c\C-l" 'mew-draft-rehighlight)
   (define-key (symbol-value symmap) "\C-c\C-q" 'mew-draft-kill)
-  (define-key (symbol-value symmap) "\C-c\C-s" 'mew-pgp-sign-message)
-  (define-key (symbol-value symmap) "\C-c\C-e" 'mew-pgp-encrypt-message)
-  (define-key (symbol-value symmap) "\C-c\C-b" 'mew-pgp-sign-encrypt-message)
-  (define-key (symbol-value symmap) "\C-c\C-r" 'mew-pgp-encrypt-sign-message)
+  (define-key (symbol-value symmap) "\C-c\C-s" 'mew-draft-sign-message)
+  (define-key (symbol-value symmap) "\C-c\C-e" 'mew-draft-encrypt-message)
+  (define-key (symbol-value symmap) "\C-c\C-b" 'mew-draft-sign-encrypt-message)
+  (define-key (symbol-value symmap) "\C-c\C-r" 'mew-draft-encrypt-sign-message)
   (define-key (symbol-value symmap) "\C-c\C-p\C-a"
     'mew-draft-toggle-privacy-always)
   (define-key (symbol-value symmap) "\C-c\C-p\C-e"
@@ -324,7 +354,6 @@
 (unless mew-draft-body-map
   (setq mew-draft-body-map (make-sparse-keymap))
   (mew-set-keymap-parent mew-draft-body-map text-mode-map)
-  (define-key mew-draft-body-map "\C-x\C-s"     'mew-draft-save-buffer)
   (define-key mew-draft-body-map "\C-c\t"       'mew-draft-insert-signature)
   (define-key mew-draft-body-map "\C-c\C-y"     'mew-draft-cite)
   (define-key mew-draft-body-map "\C-c\C-t"     'mew-draft-yank)
@@ -398,7 +427,7 @@
 
 (defvar mew-draft-mode-menu-spec
   (list  ;; need to eval mew-draft-mode-toolbar-menu
-   "Mew/Draft"
+   "Mew"
    ["Cite"                mew-draft-cite t]
    ["Cite without Label"  mew-draft-yank t]
    mew-draft-mode-toolbar-menu
@@ -442,7 +471,7 @@
   (define-key mew-header-mode-map "\C-c\C-o" 'mew-draft-set-case))
 
 (defvar mew-header-mode-menu-spec
-  '("Mew/Header"
+  '("Mew"
     ["Queue Message"   mew-header-make-message t]
     ["Send Message"    mew-header-send-message t]
     ["Set cases"       mew-draft-set-case      t]
@@ -480,6 +509,7 @@
   (define-key mew-draft-attach-map "Q"      'mew-attach-quoted-printable)
   (define-key mew-draft-attach-map "S"      'mew-attach-pgp-sign)
   (define-key mew-draft-attach-map "E"      'mew-attach-pgp-enc)
+  (define-key mew-draft-attach-map "\es"    'mew-attach-smime-sign)
   (define-key mew-draft-attach-map "p"      'mew-attach-pgp-public-key)
   (define-key mew-draft-attach-map "U"      'mew-attach-undo)
   (define-key mew-draft-attach-map "\C-c\t" 'mew-draft-insert-signature)
@@ -514,7 +544,15 @@
   (define-key mew-input-map "\t"     'mew-input-complete)
   (define-key mew-input-map [tab]    'mew-input-complete)
   (define-key mew-input-map "?"      'mew-input-complete)
-  (define-key mew-input-map "\C-c\t" 'mew-circular-complete-switch))
+  (define-key mew-input-map "\C-c\t" 'mew-circular-complete-switch)
+  (define-key mew-input-map [menu-bar minibuf]
+    (cons "Mew" (make-sparse-keymap "Mew")))
+  (define-key mew-input-map [menu-bar minibuf quit]
+    (list 'menu-item "Quit" 'keyboard-escape-quit
+          :help "Abort input and exit minibuffer"))
+  (define-key mew-input-map [menu-bar minibuf return]
+    (list 'menu-item "Enter" 'exit-minibuffer
+          :help "Terminate input and exit minibuffer")))
 
 (unless mew-input-folder-map
   (setq mew-input-folder-map (copy-keymap mew-input-map))
@@ -580,8 +618,16 @@
 ;;; Icons
 ;;;
 
-(defvar mew-icon-directory (if (fboundp 'locate-data-directory)
-			       (locate-data-directory "mew") nil))
+(defun mew-which-mew-etc ()
+  (let ((file "mew.el")
+	(path load-path))
+    (catch 'loop
+      (while path
+	(if (file-exists-p (expand-file-name file (car path)))
+	    (throw 'loop (expand-file-name "etc" (car path)))
+	  (setq path (cdr path)))))))
+
+(defvar mew-icon-directory (mew-which-mew-etc))
 
 (defvar mew-icon-separate nil)
 (defvar mew-icon-blank nil)
@@ -614,7 +660,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2003 Mew developing team.
+;; Copyright (C) 2000-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
