@@ -16,7 +16,8 @@
   "Delete refile state and delete the mark."
   (mew-summary-refile-remove-body)
   (unless (mew-virtual-p)
-    (mew-refile-reset msg)))
+    (mew-refile-reset msg)
+    (mew-summary-refile-unlog fld)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -50,8 +51,8 @@
 	  (setq msg (mew-summary-message-number))
 	  (mew-mark-unmark)
 	  (if func (funcall func fld msg))
-	  (when (mew-virtual-p)
-	    (mew-summary-unmark-in-physical fld msg func)))))))
+	  (if (mew-virtual-p)
+	      (mew-summary-unmark-in-physical fld msg func)))))))
 
 ;;
 
@@ -59,10 +60,9 @@
   "Cancel all marks according to what you input."
   (interactive)
   (let ((char (mew-input-mark)))
-    (if (and char (not (char-equal char ?\r)))
-	(mew-mark-undo-mark char))))
+    (if char (mew-mark-undo-mark char))))
 
-(defun mew-mark-undo-mark (mark &optional no-msg)
+(defun mew-mark-undo-mark (mark &optional no-msg virtual-only)
   "Undo MARK on the entire buffer.
 If optional argument NO-MSG is non-nil, no message is displayed."
   (or no-msg (message "Unmarking..."))
@@ -70,7 +70,8 @@ If optional argument NO-MSG is non-nil, no message is displayed."
   (let* ((regex (mew-mark-regex mark))
 	 (func (mew-markdb-func-undo mark))
 	 (case-fold-search nil)
-	 alist fld msg)
+	 (reviewp (char-equal mark mew-mark-review))
+	 alist fld msg reviews)
     (or (fboundp func) (setq func nil))
     (save-excursion
       (goto-char (point-min))
@@ -80,10 +81,13 @@ If optional argument NO-MSG is non-nil, no message is displayed."
 	(mew-mark-remove)
 	(when (mew-virtual-p)
 	  (mew-mark-alist-set alist fld msg))
+	(when reviewp (setq reviews (cons msg reviews)))
 	(if func (funcall func fld msg))
 	(forward-line))
       (set-buffer-modified-p nil)
-      (mew-summary-unmark-in-physical-alist alist func)))
+      (mew-sinfo-set-mark-review (nreverse reviews))
+      (unless virtual-only
+	(mew-summary-unmark-in-physical-alist alist func))))
   (or no-msg (message "Unmarking...done")))
 
 ;;
@@ -98,6 +102,25 @@ If optional argument NO-MSG is non-nil, no message is displayed."
       (setq marks (cdr marks)))
     (message "Unmarking...done")))
 
+;;
+
+(defun mew-summary-redo ()
+  "\\<mew-summary-mode-map>Recover the `*' marks which are canceled by the last `\\[mew-summary-undo-all]*'."
+  (interactive)
+  (let ((mark mew-mark-review)
+	(reviews (mew-sinfo-get-mark-review))
+	msg)
+    (save-excursion
+      (mew-mark-undo-mark mark 'no-msg)
+      (goto-char (point-min))
+      (while reviews
+	(setq msg (car reviews))
+	(setq reviews (cdr reviews))
+	(when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	  (mew-mark-put mark)
+	  (forward-line))))
+    (set-buffer-modified-p nil)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Basic functions and macros for mark
@@ -110,18 +133,20 @@ If optional argument NO-MSG is non-nil, no message is displayed."
     (when (looking-at mew-regex-mark)
       (mew-sumsyn-mark)))) ;; return char
 
-(defalias 'mew-summary-marked-p 'mew-summary-get-mark)
+(defun mew-summary-markable ()
+  (let ((mark (mew-summary-get-mark)))
+    (or (null mark) (char-equal mark mew-mark-unread))))
 
-(defsubst mew-summary-mark-as (mark)
+(defun mew-summary-mark-as (mark)
   "Mark this message with MARK if possible."
-  (unless (mew-summary-marked-p)
+  (when (mew-summary-markable)
     (mew-mark-put mark)))
 
 (defsubst mew-mark-put (mark)
   (save-excursion
     (mew-summary-goto-mark)
-    (mew-elet
-     (let* ((beg (point)) (end (1+ beg)))
+    (let* ((beg (point)) (end (1+ beg)))
+      (mew-elet
        (mew-front-sticky beg end) ;; for XEmacs
        ;; This code is awkward but for invisible.
        (forward-char 1)
@@ -129,21 +154,24 @@ If optional argument NO-MSG is non-nil, no message is displayed."
        (forward-char -2)
        (delete-char 1)
        (mew-front-nonsticky beg end)
-       (mew-highlight-mark-line mark)))))
+       (mew-highlight-mark-line mark))
+      (mew-mark-hist-set (mew-summary-message-number) mark))))
 
 (defsubst mew-mark-remove ()
   (save-excursion
     (mew-summary-goto-mark)
-    (mew-elet
-     (let* ((beg (point)) (end (1+ beg)))
+    (let* ((mark mew-mark-read)
+	   (beg (point)) (end (1+ beg)))
+      (mew-elet
        (mew-front-sticky beg end) ;; for XEmacs
        ;; This code is awkward but for invisible.
        (forward-char 1)
-       (insert-and-inherit " ") ;; inherit highlight
+       (insert-and-inherit (char-to-string mark)) ;; inherit highlight
        (forward-char -2)
        (delete-char 1)
        (mew-front-nonsticky beg end)
-       (mew-highlight-unmark-line)))))
+       (mew-highlight-unmark-line))
+      (mew-mark-hist-set (mew-summary-message-number) mark))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -191,23 +219,45 @@ cons pairs of folder name and message number."
       (while (re-search-forward mew-regex-mark nil t)
 	(setq mrk (mew-sumsyn-mark))
 	(setq msg (mew-summary-message-number))
-	(setq ret (cons (cons msg mrk) ret))
+	(setq ret (cons (list msg mrk) ret))
 	(forward-line))
       (nreverse ret))))
 
-(defun mew-summary-mark-recover (mdb)
+(defun mew-summary-mark-collect5 ()
+  (save-excursion
+    (let ((beg (mew-net-invalid-cache-start))
+	  ret mrk msg)
+      (when beg
+	(goto-char beg)
+	(while (re-search-forward mew-regex-mark nil t)
+	  (setq mrk (mew-sumsyn-mark))
+	  (setq msg (mew-summary-message-number))
+	  (setq msg (number-to-string (string-to-number msg)))
+	  (setq ret (cons (list msg mrk) ret))
+	  (forward-line))
+	(nreverse ret)))))
+
+(defun mew-summary-mark-recover (mdb &optional refdb refs)
   (let ((opos (point))
-	ent msg mrk)
+	ent msg mrk new-ref ref-ent)
     (goto-char (point-min))
     (while mdb
       (setq ent (car mdb))
       (setq mdb (cdr mdb))
-      (setq msg (car ent))
-      (setq mrk (cdr ent))
+      (mew-set '(msg mrk) ent)
       (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
-	(mew-mark-put mrk)
+	(if (or (not (char-equal mrk mew-mark-refile)) (null refdb))
+	    (mew-mark-put mrk)
+	  (when (setq ref-ent (assoc msg refdb))
+	    (mew-mark-put mrk)
+	    (setq new-ref (cons ref-ent new-ref))))
 	(forward-line)))
-    (goto-char opos)))
+    (while refs
+      (when (setq ref-ent (assoc (car refs) refdb))
+	(setq new-ref (cons ref-ent new-ref)))
+      (setq refs (cdr refs)))
+    (goto-char opos)
+    (nreverse new-ref)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -238,7 +288,32 @@ See also mew-mark-afterstep-spec."
       (mew-decode-syntax-delete)
       (beginning-of-line)
       (mew-push-mark)
-      (mew-summary-display-after mew-summary-mark-direction)))))
+      (let ((mew-inherit-after-marking t))
+	(mew-summary-display-after mew-summary-mark-direction))))))
+
+(defun mew-mark-hist-set (msg mark)
+  (let* ((hist (mew-sinfo-get-mark-hist))
+	 (ent (assoc msg hist))
+	 (num (string-to-number msg))
+	 curr prev)
+    (cond
+     (ent
+      (if mark (setcar (nthcdr 1 ent) mark)))
+     (t
+      (setq ent (list msg mark))
+      (setq curr hist)
+      (catch 'loop
+	(while curr
+	  (if (> (string-to-number (nth 0 (car curr))) num)
+	      (progn
+		(if (null prev)
+		    (setq hist (cons ent hist))
+		  (setcdr prev (cons ent (cdr prev))))
+		(throw 'loop nil)))
+	  (setq prev curr)
+	  (setq curr (cdr curr)))
+	(setq hist (nconc hist (list ent))))
+      (mew-sinfo-set-mark-hist hist)))))
 
 (defun mew-mark-put-mark (newmark &optional no-msg valid-only)
   "Put the NEWMARK on the current line if possible.
@@ -349,8 +424,9 @@ original position."
   (save-excursion
     (set-buffer fld)
     (save-excursion
-      (goto-char (point-min))
-      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+      (when (or (re-search-forward  (mew-regex-sumsyn-msg msg) nil t)
+		(re-search-backward (mew-regex-sumsyn-msg msg) nil t))
+	(beginning-of-line)
 	(mew-mark-put new-mark)
 	(set-buffer-modified-p nil)))))
 
@@ -358,14 +434,15 @@ original position."
   (save-excursion
     (set-buffer fld)
     (save-excursion
-      (goto-char (point-min))
-      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+      (when (or (re-search-forward  (mew-regex-sumsyn-msg msg) nil t)
+		(re-search-backward (mew-regex-sumsyn-msg msg) nil t))
+	(beginning-of-line)
 	(mew-mark-remove)
 	(if func (funcall func fld msg))
 	(set-buffer-modified-p nil)))))
 
 (defmacro mew-mark-alist-set (alist fld msg)
-  `(let ((imsg (string-to-int ,msg))
+  `(let ((imsg (string-to-number ,msg))
 	 (fld-msgs (assoc ,fld ,alist)))
      (if fld-msgs
 	 (nconc fld-msgs (list imsg))
@@ -384,7 +461,7 @@ original position."
 	  (save-excursion
 	    (goto-char (point-min))
 	    (while msgs
-	      (setq msg (int-to-string (car msgs)))
+	      (setq msg (number-to-string (car msgs)))
 	      (setq msgs (cdr msgs))
 	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
 		(if func (funcall func))
@@ -405,7 +482,7 @@ original position."
 	  (save-excursion
 	    (goto-char (point-min))
 	    (while msgs
-	      (setq msg (int-to-string (car msgs)))
+	      (setq msg (number-to-string (car msgs)))
 	      (setq msgs (cdr msgs))
 	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
 		(mew-mark-remove)
@@ -432,19 +509,18 @@ See also '\\[mew-summary-mark-refile]', '\\[mew-summary-mark-delete]', '\\[mew-s
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Multi: "@" in Summary mode
+;;; Escape: "$" in Summary mode
 ;;;
 
-(defun mew-summary-multi (&optional count)
+(defun mew-summary-escape (&optional count)
   "\\<mew-summary-mode-map>
-Put the multi mark (default is '@') in COUNT times
-for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-uudecode]', '\\[mew-summary-burst-multi]'."
+Put the escape mark (default is '$') in COUNT times."
   (interactive "P")
-  (mew-mark-put-mark-loop 'mew-summary-multi-one count nil))
+  (mew-mark-put-mark-loop 'mew-summary-escape-one count nil))
 
-(defun mew-summary-multi-one (&optional no-msg)
-  "Put the multi mark (default is '@') on this message."
-  (mew-mark-put-mark mew-mark-multi no-msg))
+(defun mew-summary-escape-one (&optional no-msg)
+  "Put the escape mark (default is '$') on this message."
+  (mew-mark-put-mark mew-mark-escape no-msg))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -476,6 +552,21 @@ for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-u
   "Put the unlink mark (default is 'X') on this message."
   (mew-mark-put-mark mew-mark-unlink no-msg 'valid-only))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Unread: "U" in Summary mode
+;;;
+
+(defun mew-summary-unread (&optional count)
+  "\\<mew-summary-mode-map>
+Put the unread mark (default is 'U') in COUNT times."
+  (interactive "P")
+  (mew-mark-put-mark-loop 'mew-summary-unread-one count nil))
+
+(defun mew-summary-unread-one (&optional no-msg)
+  "Put the unread mark (default is 'U') on this message."
+  (mew-mark-put-mark mew-mark-unread no-msg))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; All messages
@@ -495,7 +586,7 @@ for '\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-u
   "Put the '*' mark onto all messages which are not marked between
 BEG and END."
   (interactive "r")
-  (let ((regex (mew-mark-regex ? )) ;; not marked
+  (let ((regex (mew-mark-regex mew-sp)) ;; not marked
 	(mark mew-mark-review) ;; someday ...
 	fld msg alist)
     (save-excursion
@@ -538,14 +629,14 @@ BEG and END."
       (while (if args
 		 (re-search-forward regex nil t)
 	       (mew-summary-search-regexp-visible regex))
-	(unless (mew-summary-marked-p)
-	  (when (mew-sumsyn-match mew-regex-sumsyn-short)
-	    (when (mew-virtual-p)
-	      (setq fld (mew-sumsyn-folder-name))
-	      (setq msg (mew-sumsyn-message-number))
-	      (mew-mark-alist-set alist fld msg))
-	    (mew-mark-put mark)
-	    (setq n (1+ n))))
+	(when (and (mew-summary-markable)
+		   (mew-sumsyn-match mew-regex-sumsyn-short))
+	  (when (mew-virtual-p)
+	    (setq fld (mew-sumsyn-folder-name))
+	    (setq msg (mew-sumsyn-message-number))
+	    (mew-mark-alist-set alist fld msg))
+	  (mew-mark-put mark)
+	  (setq n (1+ n)))
 	(forward-line))
       (set-buffer-modified-p nil))
     (mew-summary-mark-in-physical-alist alist mark)
@@ -594,21 +685,26 @@ BEG and END."
   (interactive)
   (mew-summary-exchange-mark mew-mark-review mew-mark-unlink 'valid-only))
 
-(defun mew-summary-mark-multi ()	;; * -> @
-  "Change the '*' mark into the '@' mark."
+(defun mew-summary-mark-escape ()	;; * -> $
+  "Change the '*' mark into the '$' mark."
   (interactive)
-  (mew-summary-exchange-mark mew-mark-review mew-mark-multi))
+  (mew-summary-exchange-mark mew-mark-review mew-mark-escape))
 
-(defun mew-summary-mark-review ()	;; @ -> *
-  "Change the '@' mark into the '*' mark."
+(defun mew-summary-mark-review ()	;; $ -> *
+  "Change the '$' mark into the '*' mark."
   (interactive)
-  (mew-summary-exchange-mark mew-mark-multi mew-mark-review))
+  (mew-summary-exchange-mark mew-mark-escape mew-mark-review))
 
-(defun mew-summary-mark-swap ()		;; @ <-> *
-  "Swap the '@' mark and the '*' mark."
+(defun mew-summary-mark-unread ()	;; * -> U
+  "Change the '*' mark into the 'U' mark."
   (interactive)
-  (mew-summary-exchange-mark mew-mark-multi mew-mark-tmp)
-  (mew-summary-exchange-mark mew-mark-review mew-mark-multi)
+  (mew-summary-exchange-mark mew-mark-review mew-mark-unread))
+
+(defun mew-summary-mark-swap ()		;; $ <-> *
+  "Swap the '$' mark and the '*' mark."
+  (interactive)
+  (mew-summary-exchange-mark mew-mark-escape mew-mark-tmp)
+  (mew-summary-exchange-mark mew-mark-review mew-mark-escape)
   (mew-summary-exchange-mark mew-mark-tmp mew-mark-review))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -658,7 +754,7 @@ region."
 	    (catch 'loop
 	      (while t
 		(when (and (mew-summary-message-number)
-			   (not (mew-summary-marked-p)))
+			   (mew-summary-markable))
 		  (setq my-id (mew-summary-my-id))
 		  (when (> (length my-id) 0)
 		    (setq dup-id (intern-soft my-id ids))
@@ -690,7 +786,7 @@ region."
 	 ((= count 0)
 	  (message "Marking duplications...done  (no duplication)"))
 	 ((= count 1)
-	  (message "Marking duplications...done  (1 msg is marked)" count))
+	  (message "Marking duplications...done  (1 msg is marked)"))
 	 (t
 	  (message "Marking duplications...done  (%d msgs are marked)" count))))))))
 
@@ -700,12 +796,19 @@ region."
 ;;;
 
 (defun mew-summary-set-walk-mark ()
-  (let ((char (mew-input-mark)))
+  (let* ((msg (format "Input mark(%c): " mew-mark-default-walk))
+	 (mew-mark-spec (cons (list 13) mew-mark-spec)) ;; adding \r
+	 (char (mew-input-mark msg)))
     (cond
+     ((null char)
+      ;; mew-input-mark displays a message.
+      )
      ((char-equal char ?\r)
       (setq mew-mark-walk mew-mark-default-walk)
-      (message "Target mark was set to '%s'" (char-to-string mew-mark-walk)))
-     (char (setq mew-mark-walk char)))))
+      (message "Target mark was set to '%c'" mew-mark-walk))
+     (char
+      (setq mew-mark-walk char)
+      (message "Target mark was set to '%c'" mew-mark-walk)))))
 
 (defun mew-summary-down-mark (mark)
   (let ((case-fold-search nil))
@@ -728,7 +831,7 @@ the specified mark."
   (if arg
       (mew-summary-set-walk-mark)
     (if (mew-summary-down-mark mew-mark-walk)
-	(mew-summary-display nil))))
+	(mew-summary-display))))
 
 (defun mew-summary-up-mark (mark)
   (let ((case-fold-search nil))
@@ -748,52 +851,71 @@ the specified mark."
   (if arg
       (mew-summary-set-walk-mark)
     (if (mew-summary-up-mark mew-mark-walk)
-	(mew-summary-display nil))))
+	(mew-summary-display))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Mark exchange
+;;;
+
+(defun mew-summary-input-exchangeable-mark (msg)
+  (let (mark)
+    (catch 'loop
+      (while t
+	(setq mark (mew-input-mark msg))
+	(cond
+	 ((null mark)
+	  (message "That is not a mark")
+	  (mew-let-user-read))
+	 ((mew-markdb-statefullp mark)
+	  (message "'%c' cannot be exchanged" mark)
+	  (mew-let-user-read))
+	 (t
+	  (throw 'loop mark)))))))
+
+(defun mew-summary-exchange-marks ()
+  "Exchange the first input mark to the second one.
+The 'o' mark is not exchangeable."
+  (interactive)
+  (let* ((from (mew-summary-input-exchangeable-mark "Input mark from: "))
+	 (to  (mew-summary-input-exchangeable-mark "Input mark to: "))
+	 (regex (mew-mark-regex from))
+	 (case-fold-search nil))
+    (unless (char-equal from to)
+      (save-excursion
+	(goto-char (point-min))
+	(while (re-search-forward regex nil t)
+	  (mew-summary-undo-one 'nomsg)
+	  (mew-mark-put-mark to 'nomsg))
+	(forward-line)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Clean up marks!
 ;;;
 
-(defun mew-mark-clean (askp &optional sum)
-  "Process marked messages for this folder."
-  (if sum
-      (set-buffer sum)
-    (setq sum (mew-summary-folder-name 'ext)))
-  (if (and (mew-summary-p)
-           (mew-summary-mark-exist-p mew-mark-clean)
-	   (or (not askp)
-	       (y-or-n-p (format "Marks exist in %s. Process them? " sum))))
-      (mew-summary-exec)))
-
 (defun mew-mark-init ()
-  (add-hook 'kill-emacs-hook 'mew-mark-clean-up))
+  (add-hook 'kill-emacs-hook 'mew-sinfo-clean-up))
 
-(defun mew-mark-clean-up ()
-  "Process marked messages for all Summary modes.
-Typically called by kill-emacs-hook."
-  (remove-hook 'kill-emacs-hook 'mew-mark-clean-up)
+(defun mew-sinfo-clean-up ()
+  "Saving marks. Typically called by kill-emacs-hook."
+  (remove-hook 'kill-emacs-hook 'mew-sinfo-clean-up)
   (mew-decode-syntax-delete)
   (let ((bufs mew-buffers) buf)
     (save-excursion
       (while bufs
 	(setq buf (car bufs))
 	(setq bufs (cdr bufs))
-	(when (and (bufferp (get-buffer buf))
-		   (mew-folder-localp buf)) ;; xxx
-	  (mew-mark-clean mew-ask-mark-process buf))))))
-
-;; Saving marks is a really bad idea.
-;; First because there is no way to fill the gap if the folder is newer
-;; than the cache at quitting.
-;; Even if the cache is newer, saving marks faces dilemma if 
-;; multiple Emacses run.
+	(when (bufferp (get-buffer buf))
+	  (set-buffer buf)
+	  (when (mew-summary-p)
+	    (mew-sinfo-save)))))))
 
 (provide 'mew-mark)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2003 Mew developing team.
+;; Copyright (C) 1997-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
