@@ -400,13 +400,12 @@ folder and displays it in addition to its bound key."
 ;;; Prefix hack
 ;;;
 
-(defun mew-input-folder-complete-case-input (prefix-char)
-  (when (and mew-use-case-input-completion
+(defun mew-input-folder-complete-case (prefix-char)
+  (when (and mew-use-case-completion
 	     (eq mew-input-complete-function 'mew-complete-folder)
 	     (mew-folder-remotep (char-to-string prefix-char))
-	     (and mew-case-input
-		  (not (string= mew-case-input mew-case-default))))
-    (insert mew-case-input ":")))
+	     (and mew-case (not (string= mew-case mew-case-default))))
+    (insert mew-case ":")))
 
 (defun mew-input-folder-prefix ()
   "A function to insert a folder prefix in minibuffer.
@@ -446,7 +445,7 @@ it is deleted automatically."
       (cond
        ((or (= pos 0)
 	    (save-excursion (forward-char -1) (looking-at ",")))
-	(mew-input-folder-complete-case-input last-command-char)
+	(mew-input-folder-complete-case last-command-char)
 	(insert last-command-char))
        ;;
        ((or (and (= pos 1)
@@ -455,12 +454,12 @@ it is deleted automatically."
 	    (save-excursion (forward-char -2) (looking-at ",[-+%$]")))
 	(forward-char -1)
 	(delete-char 1)
-	(mew-input-folder-complete-case-input last-command-char)
+	(mew-input-folder-complete-case last-command-char)
 	(insert last-command-char))
        ;;
        ((save-excursion
 	  (forward-char -2)
-	  (looking-at ":[-+*%$]"))
+	  (looking-at ":[-+%$*]"))
 	(forward-char -1)
 	(delete-char 1)
 	(when (memq last-command-char '(?+ ?*))
@@ -501,6 +500,7 @@ it is deleted automatically."
 	  (throw 'continue nil))
 	(setq case:folder (mew-canonicalize-case-folder case:folder))
 	(setq ret (cons case:folder ret))))
+    (setq ret (delete nil ret))
     (nreverse ret)))
 
 (defun mew-input-local-folder (folder)
@@ -746,8 +746,8 @@ it is deleted automatically."
 	  mew-range-all ;; buffer is empty.
 	(forward-line -1)
 	(mew-summary-goto-message)
-	(concat (int-to-string
-		 (1+ (string-to-int (mew-summary-message-number))))
+	(concat (number-to-string
+		 (1+ (string-to-number (mew-summary-message-number))))
 		"-")))))
 
 (defun mew-input-range (folder askp)
@@ -794,7 +794,7 @@ it is deleted automatically."
      ((string= range mew-range-str-all)      t)
      ((string= range mew-range-str-update) nil)
      ((string-match "^last:\\([0-9]+\\)$" range)
-      (string-to-int (match-string 1 range)))
+      (string-to-number (match-string 1 range)))
      (t nil)))) ;; update just in case
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -842,9 +842,9 @@ it is deleted automatically."
 
 (defun mew-convert-to-home-dir (dir)
   (let* ((chome (file-name-as-directory mew-home))
-	 (ehome (expand-file-name chome)))
+	 (ehome (concat "^" (regexp-quote (expand-file-name chome)))))
     (if (string-match ehome dir)
-	(concat chome (substring dir (match-end 0) nil))
+	(concat chome (substring dir (match-end 0)))
       dir)))
 
 (defvar mew-summary-previous-directory nil)
@@ -862,6 +862,8 @@ it is deleted automatically."
      (setq ret (mew-input-file-name ,prompt def))
      (if ,preservep
 	 (setq ,previous (file-name-directory (mew-convert-to-home-dir ret))))
+     (if (and ,file (file-directory-p ret))
+	 (setq ret (expand-file-name (file-name-nondirectory ,file) ret)))
      ret))
 
 (defun mew-summary-input-file-name (&optional prompt file)
@@ -943,7 +945,7 @@ it is deleted automatically."
 
 (defvar mew-input-case-hist nil)
 
-(defun mew-input-case (default msg &optional edit)
+(defun mew-input-case (default &optional edit)
   (mew-input-clear)
   (unless default (setq default mew-case-default))
   (let ((mew-input-complete-function 'mew-complete-case)
@@ -953,13 +955,13 @@ it is deleted automatically."
 	case lst ret)
     (if edit
 	(setq case (read-from-minibuffer
-		    (format "%s case value: " msg)
+		    "Case value: "
 		    default
 		    mew-input-map
                     nil
                     'mew-input-case-hist))
       (setq case (read-from-minibuffer
-		  (format "%s case value (%s): " msg default)
+		  (format "Case value (%s): " default)
 		  ""
 		  mew-input-map
                   nil
@@ -978,18 +980,17 @@ it is deleted automatically."
 ;;; Mark
 ;;;
 
-(defun mew-input-mark ()
+(defun mew-input-mark (&optional msg)
   (let ((ociea cursor-in-echo-area) char)
     (unwind-protect
 	(progn
-	  (message "Input mark: ")
+	  (message (or msg "Input mark: "))
 	  (setq cursor-in-echo-area t)
 	  (setq char (read-char))
 	  (unless (char-equal char ?\r)
-	    (message "Input mark: %s" (char-to-string char))))
+	    (message "%s%c" (or msg "Input mark: ") char)))
       (setq cursor-in-echo-area ociea))
     (cond
-     ((char-equal char ?\r) char)
      ((mew-markdb-by-mark char) char)
      (t (message "Mark %c is not supported" char)
 	nil))))
@@ -1085,11 +1086,47 @@ it is deleted automatically."
     (if (string= name "") (setq name default))
     name))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Encoding
+;;;
+
+(defun mew-input-encoding ()
+  (let* ((encoding-list (list mew-b64 mew-qp))
+	 (encoding-alist (mapcar (lambda (x) (list x)) encoding-list))
+	 (prompt (format "Lines are too long. Input encoding (%s): " 
+			 mew-default-encoding))
+	 (cte (completing-read prompt encoding-alist)))
+    (if (string= cte "") (setq cte mew-default-encoding))
+    (setq cte (downcase cte))
+    (while (not (member cte encoding-list))
+      (setq prompt (format "'%s' is unknown. Input encoding (%s): " 
+			   cte mew-default-encoding))
+      (setq cte (completing-read prompt encoding-alist))
+      (if (string= cte "") (setq cte mew-default-encoding))
+      (setq cte (downcase cte)))
+    cte))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Command
+;;;
+
+(defun mew-input-command (&optional default)
+  (let ((program (read-string "Command args: " default)))
+    (setq program (mew-split-quoted program mew-sp ?\" ?\"))
+    (setq program (delete "" program))
+    (while (not (mew-which-exec (car program)))
+      (setq program (read-string "Command args: " default))
+      (setq program (mew-split-quoted program mew-sp ?\" ?\"))
+      (setq program (delete "" program)))
+    program))
+
 (provide 'mew-minibuf)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2003 Mew developing team.
+;; Copyright (C) 1997-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
