@@ -22,7 +22,8 @@
     ("nntp"  . "119")
     ("nntps" . "563")
     ("imap"  . "143")
-    ("imaps" . "993")))
+    ("imaps" . "993")
+    ("submission" . "587")))
 
 (defun mew-serv-to-port (serv)
   (if (string-match "^[0-9]+$" serv)
@@ -96,7 +97,7 @@
 (defun mew-net-status2 (buf rttl rcnt siz &optional zero secure)
   (when mew-use-net-status
     (when (<= rcnt rttl)
-      (let ((msiz (string-to-int siz))
+      (let ((msiz (string-to-number siz))
 	    (bsiz (if zero 0 (buffer-size)))
 	    percent substatus)
 	(if (and (numberp mew-net-status-percent-size)
@@ -129,6 +130,8 @@
     (set-buffer buf)
     (mew-elet (put-text-property (point-min) (point-max) 'invisible nil))
     (setq mew-summary-buffer-raw t)
+    (if (eq (get-buffer-window buf) (selected-window))
+	(mew-summary-cook-window))
     (set-buffer-modified-p nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -150,7 +153,7 @@
   (setq mew-biff-string nil))
 
 (defun mew-biff-setup ()
-  (let ((inbox (mew-proto-inbox-folder nil mew-case-input))
+  (let ((inbox (mew-proto-inbox-folder nil mew-case))
 	func)
     (if (not mew-use-biff)
 	(mew-biff-clean-up)
@@ -237,13 +240,13 @@
 	  file nxt)
       (cond
        (msg
-	(setq nxt (int-to-string (1+ (string-to-int msg))))
+	(setq nxt (number-to-string (1+ (string-to-number msg))))
 	(funcall set pnm nxt)
 	(setq file (mew-expand-folder bnm msg)))
        (t
 	(setq file (mew-folder-new-message bnm))
 	(setq msg (file-name-nondirectory file))
-	(setq nxt (int-to-string (1+ (string-to-int msg))))
+	(setq nxt (number-to-string (1+ (string-to-number msg))))
 	(funcall set pnm nxt)))
       (cons msg file))))
 
@@ -252,17 +255,29 @@
 ;;; Queue
 ;;;
 
+(defun mew-queue-check-new-message (file)
+  (if (not (file-exists-p (concat file mew-queue-work-suffix)))
+      file
+    (let* ((dir (file-name-directory file))
+	   (num (file-name-nondirectory file))
+	   (n (1+ (string-to-number num))))
+      (while (or (file-exists-p (format "%s%d" dir n))
+		 (file-exists-p (format "%s%d%s" dir n mew-queue-work-suffix)))
+	(setq n (1+ n)))
+      (format "%s%d" dir n))))
+
 (defun mew-queue-enqueue (work qfld) ;; 1.wrk or draft/1
   (let (orig info iwrk)
     (if (not (string-match (concat mew-queue-work-suffix "$") work))
 	;; draft/1
-	(setq orig (mew-folder-new-message qfld))
+	(setq orig (mew-queue-check-new-message (mew-folder-new-message qfld)))
       ;; 1.wrk
       (setq orig (file-name-sans-extension work)) ;; 1
       (setq iwrk (concat orig mew-queue-info-suffix mew-queue-work-suffix))
       (mew-delete-file iwrk) ;; 1.mqi.wrk
-      (if (file-exists-p orig) (setq orig (mew-folder-new-message qfld))))
-    (rename-file work orig 'override)
+      (if (file-exists-p orig)
+	  (setq orig (mew-queue-check-new-message (mew-folder-new-message qfld)))))
+    (rename-file work orig) ;; An error is signaled if orig exists.
     (unless mew-use-nfs-hack
       ;; This cause an error when using NFS.
       (mew-set-file-modes orig))
@@ -275,9 +290,9 @@
 	 (info (concat orig mew-imapq-info-suffix)) ;; 1.iqi
 	 (iwrk (concat info mew-queue-work-suffix))) ;; 1.iqi.wrk
     (when (file-exists-p orig)
-      (setq orig (mew-folder-new-message dir))
+      (setq orig (mew-queue-check-new-message (mew-folder-new-message dir)))
       (setq info (concat orig mew-imapq-info-suffix)))
-    (rename-file work orig 'override)
+    (rename-file work orig) ;; An error is signaled if orig exists.
     (rename-file iwrk info 'override)))
 
 (defun mew-queue-insert-file (pnm n qfld msg) ;; 1
@@ -374,7 +389,8 @@
 	  (mew-mark-kill-line))))
     (when ulks
       (setq ulks (nreverse ulks))
-      (mew-mark-exec-delete bnm ulks)
+      (let ((mew-trash-folder nil))
+	(mew-mark-exec-delete bnm ulks))
       (mew-summary-folder-cache-save))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -392,21 +408,26 @@
     (setq mew-summary-buffer-raw nil)
     (mew-summary-folder-cache-save)))
 
+(defun mew-net-invalid-cache-start ()
+  (save-excursion
+    (goto-char (point-min))
+    (when (re-search-forward mew-regex-sumsyn-invalid nil t)
+      (beginning-of-line)
+      (point))))
+
+(defun mew-net-invalid-cache-invisible ()
+  (let ((beg (mew-net-invalid-cache-start)))
+    (when beg
+      (mew-elet
+       (put-text-property beg (point-max) 'invisible t)))))
+
 (defun mew-net-invalid-cache-clean ()
   (let* ((bnm (mew-summary-folder-name 'ext))
 	 (dir (mew-expand-folder bnm))
-	 (msgs (mew-dir-messages dir mew-regex-message-files4))
-	 beg)
-    (save-excursion
-      (goto-char (point-min))
-      (when (re-search-forward mew-regex-sumsyn-invalid nil t)
-	(beginning-of-line)
-	(setq beg (point))
-	(goto-char (point-max))
-	(mew-elet
-	 (delete-region beg (point)))
-	(mew-summary-unlink-msgs bnm msgs)
-	(mew-summary-folder-cache-save)))))
+	 (msgs (mew-dir-messages dir mew-regex-message-files4)))
+    (mew-summary-unlink-msgs bnm msgs)
+    (mew-mark-kill-invisible)
+    (mew-summary-folder-cache-save)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -427,34 +448,35 @@
 If 'mew-auto-flush-queue' is non-nil, +queue is flushed.
 If called with '\\[universal-argument]', +queue is not flushed."
   (interactive "P")
-  (let* ((case mew-case-input)
-	 (dont (not (mew-use-retrieve case)))
-	 (inbox (mew-inbox-folder case))
-	 (dir (mew-expand-folder inbox))
-	 (mailbox (mew-mailbox-type case)))
-    (if dont
-	(message "This command is not allowed to use")
-      (unless (file-directory-p dir)
-	(mew-make-directory dir))
-      ;; for mew-summary-exchange-point.
-      (set-marker mew-summary-inbox-position (point) (current-buffer))
-      (mew-summary-switch-to-folder inbox)
-      ;; in the inbox
-      (when (mew-summary-exclusive-p)
-	(mew-summary-reset)
+  (let* ((case mew-case)
+	 (proto (mew-proto case))
+	 (inbox (mew-proto-inbox-folder proto case))
+	 (case:inbox (mew-case-folder case inbox))
+	 mailbox)
+    ;; for mew-summary-exchange-point.
+    (mew-sinfo-set-ret-pos (point))
+    (mew-summary-visit-folder case:inbox)
+    ;; in the inbox
+    (when (mew-summary-exclusive-p)
+      (cond
+       ((mew-folder-remotep inbox)
+	(mew-summary-ls nil 'goend 'update))
+       (t ;; local
 	;;
+	(mew-summary-reset)
 	(mew-summary-retrieve-gap inbox)
 	;;
 	(goto-char (point-max))
 	(mew-sinfo-set-start-point (point))
 	(mew-sinfo-set-direction 'down)
+	(setq mailbox (mew-mailbox-type case))
 	(cond
 	 ((eq mailbox 'pop)
 	  (mew-pop-retrieve case 'inc inbox (not no-flush)))
 	 ((eq mailbox 'imap)
 	  (mew-imap-retrieve case 'inc inbox (not no-flush)))
 	 ((eq mailbox 'mbox)
-	  (mew-mbox-retrieve case 'inc inbox (not no-flush))))))))
+	  (mew-mbox-retrieve case 'inc inbox (not no-flush)))))))))
 
 (defun mew-summary-scan-boot (proto case)
   ;; called by mew()
@@ -467,7 +489,7 @@ If called with '\\[universal-argument]', +queue is not flushed."
     (mew-summary-switch-to-folder case:inbox)
     ;; in the inbox
     ;; for mew-summary-exchange-point.
-    (set-marker mew-summary-inbox-position (point) (current-buffer))
+    (mew-sinfo-set-ret-pos (point))
     (when (mew-summary-exclusive-p)
       (mew-summary-reset)
       ;;
@@ -495,13 +517,18 @@ If called with '\\[universal-argument]', +queue is not flushed."
 
 (defun mew-mbox-retrieve (case directive inbox &optional flush)
   ;; directive is 'inc
-  (let ((mbox-command (mew-mbox-command case))
-	(mbox-command-arg (mew-mbox-command-arg case))
-	opts)
+  (let* ((mbox-command (mew-mbox-command case))
+	 (mbox-command-arg (mew-mbox-command-arg case))
+	 dir mode opts)
     (if (not (mew-which-exec mbox-command))
 	(message "'%s' not found!" mbox-command)
       (if mbox-command-arg
-	  (setq opts (list "-e" mbox-command "-m" mbox-command-arg))
+	  (if (not (string= mbox-command "incm"))
+	      (setq opts (list "-e" mbox-command "-m" mbox-command-arg))
+	    (setq dir (mew-expand-folder inbox))
+	    (setq mode (mew-get-file-modes dir))
+	    (setq opts (list "-e" mbox-command
+			     "-m" (format "%s -p %d" mbox-command-arg mode))))
 	(setq opts (list "-e" mbox-command)))
       (mew-local-retrieve 'inc inbox flush opts))))
 
@@ -510,6 +537,25 @@ If called with '\\[universal-argument]', +queue is not flushed."
 ;;; Retrieving truncated messages
 ;;;
 
+(defun mew-summary-get-inherit-case (fld msg)
+  (let ((buf (mew-cache-hit fld msg))
+	uidl delbuf)
+    (unless buf
+      (save-excursion
+	(setq buf (generate-new-buffer mew-buffer-prefix))
+	(setq delbuf t)
+	(set-buffer buf)
+	(mew-erase-buffer)
+	(mew-insert-message fld msg
+			    mew-cs-text-for-read mew-header-reasonable-size)))
+    (save-excursion
+      (set-buffer buf)
+      (setq uidl (mew-header-get-value mew-x-mew-uidl:))
+      (prog1
+	  (when (and uidl (string-match "^[^ ]+ [0-9]+ \\(.+\\)$" uidl))
+	    (mew-match-string 1 uidl))
+	(if delbuf (mew-remove-buffer buf))))))
+
 (defun mew-summary-retrieve-message (&optional rev-del)
   "Retrieve the rest of a truncated('T') message.
 
@@ -522,45 +568,85 @@ the mailbox. When executed with '\\[universal-argument]',
 In a REMOTE folder: case and protocol are determined by the folder.
 The message in the server side is always retained."
   (interactive "P")
-  (mew-summary-only
-   (mew-summary-msg
-    (when (mew-summary-exclusive-p)
-      (when (mew-sumsyn-match mew-regex-sumsyn-long)
-	(let* ((msg (mew-sumsyn-message-number))
-	       (uid (mew-sumsyn-message-uid))
-	       (siz (mew-sumsyn-message-size))
-	       (bnm (mew-summary-folder-name 'ext))
-	       (folder (mew-sinfo-get-folder))
-	       (case (mew-sinfo-get-case))
-	       del rtr rtrs mailbox)
-	  (when (mew-folder-localp folder)
-	    (setq case mew-case-input)
-	    (setq mailbox (mew-mailbox-type case))
-	    (cond
-	     ((eq mailbox 'pop)
-	      (setq del (eq (mew-pop-delete case) t)));; delete may be number
-	     ((eq mailbox 'imap)
-	      (setq del (eq (mew-imap-delete case) t))));; delete may be number
-	    (if rev-del (setq del (not del))))
-	  (setq rtr (list uid siz del (list bnm msg)))
-	  (setq rtrs (list rtr))
-	  (cond
-	   ((not rtrs)
-	    (message "No message to be retrieved"))
-	   ((not (mew-msg-validp msg))
-	    (message "No valid message to be retrieved"))
-	   ((mew-folder-localp folder)
-	    (cond
-	     ((eq mailbox 'pop)
-	      (mew-pop-retrieve case 'get bnm rtrs))
-	     ((eq mailbox 'imap)
-	      (mew-imap-retrieve case 'get bnm rtrs))))
-	   ((mew-folder-popp folder)
-	    (mew-pop-retrieve case 'get bnm rtrs))
-	   ((mew-folder-imapp folder)
-	    (mew-imap-retrieve case 'get bnm rtrs))
-	   ((mew-folder-nntpp folder)
-	    (mew-nntp-retrieve case 'get bnm rtrs)))))))))
+  (mew-summary-msg
+   (when (mew-sumsyn-match mew-regex-sumsyn-long)
+     (let* ((msg (mew-sumsyn-message-number))
+	    (uid (mew-sumsyn-message-uid))
+	    (siz (mew-sumsyn-message-size))
+	    (bnm (mew-summary-folder-name))
+	    (cbuf (mew-summary-folder-name 'ext)) 
+	    folder case del rtr rtrs mailbox thread-info)
+       (when (mew-thread-p)
+	 (setq thread-info (list cbuf bnm msg)))
+       (save-excursion
+	 (set-buffer bnm)
+	 (when (mew-summary-exclusive-p)
+	   (setq folder (mew-sinfo-get-folder))
+	   (setq case (mew-sinfo-get-case))
+	   (when (mew-folder-localp folder)
+	     (setq case (or (mew-summary-get-inherit-case folder msg) mew-case))
+	     (setq mailbox (mew-mailbox-type case))
+	     (cond
+	      ((eq mailbox 'pop)
+	       (setq del (eq (mew-pop-delete case) t))) ;; delete may be number
+	      ((eq mailbox 'imap)
+	       (setq del (eq (mew-imap-delete case) t))))	;; delete may be number
+	     (if rev-del (setq del (not del))))
+	   (when (and uid (not (string= uid "")))
+	     (setq rtr (list uid siz del (list bnm msg)))
+	     (setq rtrs (list rtr)))
+	   (cond
+	    ((not rtrs)
+	     (message "No message to be retrieved"))
+	    ((not (mew-msg-validp msg))
+	     (message "No valid message to be retrieved"))
+	    ((mew-folder-localp folder)
+	     (cond
+	      ((eq mailbox 'pop)
+	       (mew-pop-retrieve case 'get bnm rtrs thread-info))
+	      ((eq mailbox 'imap)
+	       (mew-imap-retrieve case 'get bnm rtrs thread-info))))
+	    ((mew-folder-popp folder)
+	     (mew-pop-retrieve case 'get bnm rtrs thread-info))
+	    ((mew-folder-imapp folder)
+	     (mew-imap-retrieve case 'get bnm rtrs thread-info))
+	    ((mew-folder-nntpp folder)
+	     (mew-nntp-retrieve case 'get bnm rtrs thread-info)))))))))
+
+(defun mew-summary-retrieve-message-for-thread (thread-info)
+  (let ((to-buf (nth 0 thread-info))
+	(from-buf (nth 1 thread-info))
+	(msg (nth 2 thread-info))
+	str start beg end idt-str level length)
+    (save-excursion
+      (set-buffer from-buf)
+      (save-excursion
+	(goto-char (point-min))
+	(re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	(beginning-of-line)
+	(setq start (point))
+	(forward-line)
+	(setq str (buffer-substring start (point))))
+      (set-buffer to-buf)
+      (save-excursion
+	(goto-char (point-min))
+	(re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	(beginning-of-line)
+	(setq start (point))
+	(setq beg (mew-thread-next-property start))
+	(setq level (mew-thread-get-property beg))
+	(setq length (* mew-thread-indent-length level))
+	(setq end (+ beg length))
+	(setq idt-str (buffer-substring beg end))
+	(forward-line)
+	(mew-elet
+	 (delete-region start (point))
+	 (insert str)
+	 (goto-char beg)
+	 (if (= level 0) (setq end (1+ end)))
+	 (insert idt-str)
+	 (mew-thread-put-property beg end level)))
+      (mew-summary-unlock))))
 
 (defun mew-summary-mark-retrieve-message (&optional rev-del)
   "Retrieve the rest of truncated('T') messages marked with '*'.
@@ -578,37 +664,57 @@ The messages in the server side is always retained."
    (let* ((bnm (mew-summary-folder-name 'ext))
 	  (folder (mew-sinfo-get-folder))
 	  (case (mew-sinfo-get-case))
-	  msg uid siz del rtr rtrs mailbox)
-     (when (mew-folder-localp folder)
-       (setq case mew-case-input)
-       (setq mailbox (mew-mailbox-type case))
-       (cond
-	((eq mailbox 'pop)
-	 (setq del (eq (mew-pop-delete case) t))) ;; delete may be number
-	((eq mailbox 'imap)
-	 (setq del (eq (mew-imap-delete case) t)))) ;; delete may be number
-       (if rev-del (setq del (not del))))
+	  msg uid siz del rtr rtrs case-rtrs mailbox)
      (save-excursion
        (goto-char (point-min))
-       (while (re-search-forward mew-regex-msg-review nil t)
-	 (when (mew-sumsyn-match mew-regex-sumsyn-long)
+       (if (mew-folder-localp folder)
+	   (let (tmp)
+	     (while (re-search-forward mew-regex-msg-review nil t)
+	       (when (mew-sumsyn-match mew-regex-sumsyn-long)
+		 (setq uid (mew-sumsyn-message-uid))
+		 (setq msg (mew-sumsyn-message-number))
+		 (setq siz (mew-sumsyn-message-size))
+		 (setq case (or (mew-summary-get-inherit-case folder msg) mew-case))
+		 (setq mailbox (mew-mailbox-type case))
+		 (when (and (mew-msg-validp msg) (mew-msg-truncatedp siz))
+		   (cond
+		    ((eq mailbox 'pop)
+		     (setq del (eq (mew-pop-delete case) t))) ;; delete may be number
+		    ((eq mailbox 'imap)
+		     (setq del (eq (mew-imap-delete case) t))))	;; delete may be number
+		   (if rev-del (setq del (not del)))
+		   (setq rtr (list uid siz del (list bnm msg)))
+		   (if (setq tmp (nth 2 (assoc case case-rtrs)))
+		       (nconc tmp (list rtr))
+		     (setq case-rtrs (cons (list case mailbox (list rtr)) case-rtrs))))
+		 (forward-line)))
+	     (setq case-rtrs (nreverse case-rtrs)))
+	 (while (re-search-forward mew-regex-msg-review nil t)
+	   (when (mew-sumsyn-match mew-regex-sumsyn-long)
 	   (setq uid (mew-sumsyn-message-uid))
 	   (setq msg (mew-sumsyn-message-number))
 	   (setq siz (mew-sumsyn-message-size))
-	   (when (mew-msg-validp msg)
+	   (when (and (mew-msg-validp msg) (mew-msg-truncatedp siz))
 	     (setq rtr (list uid siz del (list bnm msg)))
 	     (setq rtrs (cons rtr rtrs))))
-	 (forward-line)))
-     (if (not rtrs)
+	   (forward-line))
+	 (setq rtrs (nreverse rtrs))))
+     (if (and (not rtrs) (not case-rtrs))
 	 (message "No message to be retrieved")
-       (setq rtrs (nreverse rtrs))
        (cond
 	((mew-folder-localp folder)
+	 (while case-rtrs
+	 (setq case (nth 0 (car case-rtrs)))
+	 (setq mailbox (nth 1 (car case-rtrs)))
+	 (setq rtrs (nth 2 (car case-rtrs)))
+	 (setq case-rtrs (cdr case-rtrs))
 	 (cond
 	  ((eq mailbox 'pop)
 	   (mew-pop-retrieve case 'get bnm rtrs))
 	  ((eq mailbox 'imap)
-	   (mew-imap-retrieve case 'get bnm rtrs))))
+	   (mew-imap-retrieve case 'get bnm rtrs)))
+	 (when case-rtrs
+	   (mew-rendezvous mew-summary-buffer-process))))
 	((mew-folder-popp folder)
 	 (mew-pop-retrieve case 'get bnm rtrs))
 	((mew-folder-imapp folder)
@@ -616,7 +722,6 @@ The messages in the server side is always retained."
 	((mew-folder-nntpp folder)
 	 (mew-nntp-retrieve case 'get bnm rtrs)))))))
 
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Folder clean up
@@ -638,10 +743,10 @@ The messages in the server side is always retained."
   ;; => ("1" "3:5" "7:8" "10")
   (let (pack a b s)
     (when (setq a (car msgs))
-      (setq a (string-to-int a))
+      (setq a (string-to-number a))
       (setq msgs (cdr msgs))
       (while (setq b (car msgs))
-	(setq b (string-to-int b))
+	(setq b (string-to-number b))
 	(setq msgs (cdr msgs))
 	(if (= (1+ a) b)
 	    (if (null s)
@@ -679,7 +784,7 @@ The messages in the server side is always retained."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2003 Mew developing team.
+;; Copyright (C) 2002-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
