@@ -29,7 +29,8 @@
     "user" "account"
     "size" "get-body" "no-msg" "case" "msgdb" "done" "dispatched" "error"
     "max"
-    "newsgroup" "msgid" "truncated"))
+    "newsgroup" "msgid" "truncated"
+    "thread-info" "status-buf"))
 
 (mew-info-defun "mew-nntp-" mew-nntp-info-list)
 
@@ -68,7 +69,7 @@
   (or (mew-nntp-get-ssh-process pnm) (mew-nntp-get-ssl-process pnm)))
 
 (defun mew-nntp-command-mode-reader (pro pnm)
-  (mew-net-status (mew-nntp-get-bnm pnm)
+  (mew-net-status (mew-nntp-get-status-buf pnm)
 		  "Connecting"
 		  nil
 		  (mew-nntp-secure-p pnm))
@@ -109,7 +110,8 @@
 	max)
     (if (and mew-nntp-skip-uidl (eq directive 'get))
 	(mew-nntp-command-dispatch pro pnm directive refs nil)
-      (mew-net-status bnm "Checking" nil (mew-nntp-secure-p pnm))
+      (mew-net-status (mew-nntp-get-status-buf pnm)
+		      "Checking" nil (mew-nntp-secure-p pnm))
       (cond
        ((eq directive 'scan)
 	(if (eq range nil) ;; update
@@ -119,13 +121,13 @@
 	  (progn
 	    (cond
 	     ((stringp max)
-	      (setq max (string-to-int max)))
+	      (setq max (string-to-number max)))
 	     (max ;; backward compatibility
 	      ;; reversed
 	      (setq max (car max))
-	      (setq max (string-to-int max)))
+	      (setq max (string-to-number max)))
 	     (t
-	      (setq max (string-to-int (mew-match-string 1)))
+	      (setq max (string-to-number (mew-match-string 1)))
 	      (setq max (1- max))))
 	    (mew-nntp-set-max pnm max)
 	    (mew-nntp-process-send-string pro "XOVER %d-" (1+ max)))
@@ -151,7 +153,7 @@
 	(if rtr (setq rtrs (cons rtr rtrs))))
        ((eq directive 'scan)
 	(if (and uid (or range ;; all, last:n
-			 (> (string-to-int uid) max))) ;; update
+			 (> (string-to-number uid) max))) ;; update
 	    (setq rtrs (cons (list uid siz) rtrs))))
        ((eq directive 'sync)
 	(if uid (setq hlds (cons uid hlds))))))
@@ -188,7 +190,7 @@
       (mew-nntp-command-article pro pnm)))))
 
 (defun mew-nntp-command-article (pro pnm)
-  (mew-net-status2 (mew-nntp-get-bnm pnm) 
+  (mew-net-status2 (mew-nntp-get-status-buf pnm)
 		   (mew-nntp-get-rttl pnm)
 		   (mew-nntp-get-rcnt pnm)
 		   (nth 1 (car (mew-nntp-get-rtrs pnm)))
@@ -212,7 +214,7 @@
      ((and (eq directive 'scan) (not get-body))
       (mew-nntp-set-truncated pnm t)
       (mew-nntp-process-send-string pro "HEAD %s" uid))
-     ((or (= lim 0) (<= (string-to-int siz) lim))
+     ((or (= lim 0) (<= (string-to-number siz) lim))
       (mew-nntp-set-truncated pnm nil)
       (mew-nntp-process-send-string pro "ARTICLE %s" uid))
      (t
@@ -235,8 +237,7 @@
      ((stringp fld-msg)
       (setq fld fld-msg))
      ((listp fld-msg)
-      (setq fld (nth 0 fld-msg))
-      (setq msg (nth 1 fld-msg))
+      (mew-set '(fld msg) fld-msg)
       (setq lmsg msg)))
     (goto-char (point-min))
     (forward-line)
@@ -283,7 +284,7 @@
     (mew-nntp-command-article pro pnm)))
 
 (defun mew-nntp-command-list (pro pnm)
-  (mew-net-status (mew-nntp-get-bnm pnm)
+  (mew-net-status (mew-nntp-get-status-buf pnm)
 		  "Listing"
 		  nil
 		  (mew-nntp-secure-p pnm))
@@ -420,7 +421,8 @@
 	 (pnm (mew-nntp-info-name case newsgroup))
 	 (buf (get-buffer-create (mew-nntp-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
-	 process sshname sshpro sslname sslpro lport)
+	 process sshname sshpro sslname sslpro lport tls
+	 thread-info to-buf)
     (if (mew-nntp-get-process pnm)
 	(message "Another NNTP process is running. Try later")
       (cond
@@ -432,7 +434,8 @@
 	  (when lport
 	    (setq process (mew-nntp-open pnm "localhost" lport no-msg)))))
        (sslp
-	(setq sslpro (mew-open-ssl-stream server sslport))
+	(if (string= port sslport) (setq tls mew-tls-nntp))
+	(setq sslpro (mew-open-ssl-stream case server sslport tls))
 	(when sslpro
 	  (setq sslname (process-name sslpro))
 	  (setq lport (mew-ssl-pnm-to-lport sslname))
@@ -443,6 +446,7 @@
       (when process
 	(mew-summary-lock process "NNTPing" (or sshpro sslpro))
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
+	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
 	(mew-info-clean-up pnm)
 	(mew-nntp-set-no-msg pnm no-msg) ;; must come here
@@ -457,6 +461,7 @@
 	(mew-nntp-set-status pnm "greeting")
 	(mew-nntp-set-directive pnm directive)
 	(mew-nntp-set-bnm pnm bnm)
+	(mew-nntp-set-status-buf pnm bnm)
 	(mew-nntp-set-rcnt pnm 1)
 	(mew-nntp-set-rttl pnm 0)
 	(mew-nntp-set-size pnm (mew-nntp-size case))
@@ -465,13 +470,24 @@
 	;;
 	(cond
 	 ((eq directive 'get)
-	  (mew-nntp-set-refs pnm (nth 0 args)))
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
+	  (mew-nntp-set-refs pnm (nth 0 args))
+	  (setq thread-info (nth 1 args))
+	  (mew-nntp-set-thread-info pnm thread-info)
+	  (setq to-buf (nth 0 thread-info))
+	  (when to-buf
+	    (mew-nntp-set-status-buf pnm to-buf)
+	    (save-excursion
+	      (set-buffer to-buf)
+	      (mew-summary-lock process "NNTPing" (or sshpro sslpro)))))
 	 ((eq directive 'scan)
 	  (mew-nntp-set-range pnm (nth 0 args))
 	  (mew-nntp-set-get-body pnm (nth 1 args))
-	  (when (mew-nntp-get-range pnm)
-	    (mew-nntp-set-mdb pnm (mew-summary-mark-collect4))
-	    (mew-net-folder-clean)))
+	  (if (mew-nntp-get-range pnm)
+	      (progn
+		(mew-nntp-set-mdb pnm (mew-summary-mark-collect4))
+		(mew-net-folder-clean))
+	    (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))))
 	 ((eq directive 'sync)
 	  ))
 	(mew-sinfo-set-start-point (point)) ;; after erase-buffer
@@ -503,7 +519,7 @@
      (mew-set-buffer-multibyte nil)
      (insert string)
      (when (string= status "article")
-       (mew-net-status2 (mew-nntp-get-bnm pnm)
+       (mew-net-status2 (mew-nntp-get-status-buf pnm)
 			(mew-nntp-get-rttl pnm)
 			(mew-nntp-get-rcnt pnm)
 			(nth 1 (car (mew-nntp-get-rtrs pnm)))
@@ -545,13 +561,14 @@
 	 (sshpro (mew-nntp-get-ssh-process pnm))
 	 (sslpro (mew-nntp-get-ssl-process pnm))
 	 (rttl (mew-nntp-get-rttl pnm))
-	 (bnm (mew-nntp-get-bnm pnm))
+	 (bnm (or (mew-nntp-get-bnm pnm) (current-buffer)))
 	 (hlds (mew-nntp-get-hlds pnm))
 	 (msgid (mew-nntp-get-msgid pnm))
 	 (done (mew-nntp-get-done pnm))
 	 (error (mew-nntp-get-error pnm))
 	 (file (mew-expand-folder bnm mew-nntp-msgid-file))
-	 (buf (process-buffer process)))
+	 (buf (process-buffer process))
+	 (thread-info (mew-nntp-get-thread-info pnm)))
     (mew-nntp-debug "NNTP SENTINEL" event)
     (set-process-buffer process nil)
     (set-buffer bnm)
@@ -566,15 +583,16 @@
 	  (when (mew-nntp-get-dispatched pnm)
 	    (cond
 	     ((eq directive 'scan)
-	      (setq msgid (int-to-string (1- (string-to-int msgid))))
+	      (setq msgid (number-to-string (1- (string-to-number msgid))))
 	      (mew-lisp-save file msgid nil 'unlimit)
 	      (setq recovered t)))
 	    (when recovered
 	      (mew-nntp-message
 	       pnm
-	       "%d message retrieved. %d messages are left due to an error."
+	       "%d message retrieved. %d messages are left due to an error"
 	       (- rttl lefts) lefts)
 	      (mew-summary-folder-cache-save))))
+      (if thread-info (mew-summary-retrieve-message-for-thread thread-info))
       (cond
        (error 
 	;; retain the error message
@@ -607,7 +625,7 @@
 	  (mew-lisp-save file msgid nil 'unlimit)
 	  (mew-summary-folder-cache-save))))))
     ;;
-    (mew-net-status-clear (mew-nntp-get-bnm pnm))
+    (mew-net-status-clear (mew-nntp-get-status-buf pnm))
     (mew-info-clean-up pnm)
     (set-buffer-modified-p nil)
     (mew-summary-unlock)
@@ -680,7 +698,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2003 Mew developing team.
+;; Copyright (C) 1999-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
