@@ -18,7 +18,7 @@ then put the '*' mark onto them. 'mewl' is called as a picking
 command. If called with '\\[universal-argument]', the target is
 the region."
   (interactive "P")
-  (mew-summary-or-thread
+  (mew-pickable
    (let ((func 'mew-summary-pick-with-mewl))
      (if (and mew-prog-mewl
 	      (not (string= mew-prog-mewl ""))
@@ -32,7 +32,7 @@ then put the '*' mark onto them. 'mew-prog-grep' is called as a
 grep command. If called with '\\[universal-argument]', the target
 is the region."
   (interactive "P")
-  (mew-summary-or-thread
+  (mew-pickable
    (let ((func 'mew-summary-pick-with-grep))
      (if (and mew-prog-grep
 	      (not (string= mew-prog-grep ""))
@@ -48,7 +48,7 @@ then put the '*' mark onto them. You can asked to specify a grep
 command.  If called with '\\[universal-argument]', the target is
 the region."
   (interactive "P")
-  (mew-summary-or-thread
+  (mew-pickable
    (let ((func 'mew-summary-pick-with-cmd)
 	 (mew-inherit-grep-cmd (read-string "Command opts pattern: "
                                             (car mew-input-cmd-hist)
@@ -113,8 +113,9 @@ the region."
 	  (message "Marking 1 message...")
 	(message "Marking %d messages..." n))
       (save-excursion
-	;; Thread folder
-	(when (mew-thread-p)
+	(cond
+	 ((mew-thread-p)
+	  ;; Thread mode
 	  (setq threadmsgs (mew-summary-thread-get-msglst
 			    (mew-vinfo-get-top) 'separator))
 	  (setq total (length threadmsgs))
@@ -129,7 +130,19 @@ the region."
 	      (mew-summary-mark-as mew-mark-review))
 	    (setq msgs (cdr msgs)))
 	  (set-buffer-modified-p nil))
-	;; Physical folder
+	 ((mew-virtual-p)
+	  ;; Virtual mode
+	  (goto-char (point-min))
+	  (setq msgsback nil)
+	  (while (and msgs (not (eobp)))
+	    (when (re-search-forward (mew-regex-sumsyn-msg (car msgs)) nil t)
+	      (setq msgsback (cons (car msgs) msgsback))
+	      (mew-summary-mark-as mew-mark-review)
+	      (forward-line))
+	    (setq msgs (cdr msgs)))
+	  (setq msgsback (nreverse msgsback))
+	  (set-buffer-modified-p nil)))
+	;; Summary mode (physical)
 	(when (get-buffer folder)
 	  (set-buffer folder)
 	  (save-excursion
@@ -156,11 +169,11 @@ the region."
    (let ((fld (mew-expand-folder2 folder))
 	 i min max msgs range)
      (when src-msgs
-       (setq min (string-to-int (car src-msgs)))
+       (setq min (string-to-number (car src-msgs)))
        (setq max min)
        (setq src-msgs (cdr src-msgs))
        (while src-msgs
-	 (setq i (string-to-int (car src-msgs)))
+	 (setq i (string-to-number (car src-msgs)))
 	 (if (< i min) (setq min i))
 	 (if (> i max) (setq max i))
 	 (setq src-msgs (cdr src-msgs)))
@@ -172,7 +185,7 @@ the region."
 	(apply 'call-process
 	       mew-prog-mewl nil t nil
 	       "-b" mew-mail-path
-	       "-l" (int-to-string mew-scan-max-field-length)
+	       "-l" (number-to-string mew-scan-max-field-length)
 	       "-p" pattern fld range))
        (goto-char (point-min))
        (while (not (eobp))
@@ -192,7 +205,7 @@ the region."
 	 (default-directory dir) ;; buffer local
 	 msgs nxt)
     ;; no sort here
-    (if (= (length src-msgs) 1) (setq src-msgs (cons "/dev/null" src-msgs)))
+    (if (= (length src-msgs) 1) (setq src-msgs (cons null-device src-msgs)))
     (setq pattern (mew-cs-encode-arg pattern))
     (with-temp-buffer
       (cd dir)
@@ -212,9 +225,9 @@ the region."
 	    (setq msgs (cons (mew-match-string 0) msgs)))
 	(forward-line)))
     (setq msgs (mew-uniq-list msgs))
-    (setq msgs (mapcar 'string-to-int msgs))
+    (setq msgs (mapcar 'string-to-number msgs))
     (setq msgs (sort msgs '<))
-    (mapcar 'int-to-string msgs)))
+    (mapcar 'number-to-string msgs)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -229,7 +242,7 @@ the region."
 	 (cmd mew-inherit-grep-cmd)
 	 msgs nxt arg)
     ;; no sort here
-    (if (= (length src-msgs) 1) (setq src-msgs (cons "/dev/null" src-msgs)))
+    (if (= (length src-msgs) 1) (setq src-msgs (cons null-device src-msgs)))
     (with-temp-buffer
       (cd dir)
       (mew-piolet
@@ -249,9 +262,9 @@ the region."
 	    (setq msgs (cons (mew-match-string 0) msgs)))
 	(forward-line)))
     (setq msgs (mew-uniq-list msgs))
-    (setq msgs (mapcar 'string-to-int msgs))
+    (setq msgs (mapcar 'string-to-number msgs))
     (setq msgs (sort msgs '<))
-    (mapcar 'int-to-string msgs)))
+    (mapcar 'number-to-string msgs)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -415,7 +428,7 @@ keyword, execute this command with '\\[universal-argument]'."
       ""
     (let ((first (string-to-char str))
 	  asc key rest eq-flag val args)
-      (if (memq first '(?\( ?\! ?\& ?\| ?= ? ?\)))
+      (if (memq first (list ?\( ?\! ?\& ?\| ?= mew-sp ?\)))
 	  (concat (char-to-string first)
 		  (mew-pick-macro-expand-string (substring str 1)))
 	(if (string-match "=\\| \\|)\\|&\\||" str)
@@ -458,7 +471,7 @@ keyword, execute this command with '\\[universal-argument]'."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
