@@ -13,7 +13,7 @@
   mew-pop-folder-alist)
 
 ;; It is impossible to implement mew-pop-skip-uidl since
-;; messages cannot be gaind access by UID.
+;; messages cannot be gained access by UID.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -28,7 +28,8 @@
     "user" "auth" "auth-list" "key" "passwd" "account"
     "size" "truncated" "get-body"
     "flush" "no-msg" "msgdb" "done" "dispatched" "error"
-    "body-lines" "delete"))
+    "body-lines" "delete" "case"
+    "thread-info" "status-buf"))
 
 (mew-info-defun "mew-pop-" mew-pop-info-list)
 
@@ -42,6 +43,8 @@
     ("capa"          t   ("\\+OK" . "auth") ("-ERR" . "pswd"))
     ("auth-cram-md5" nil ("\\+OK" . "pwd-cram-md5") ("-ERR" . "wpwd"))
     ("pwd-cram-md5"  nil ("\\+OK" . "list") ("-ERR" . "wpwd"))
+    ("auth-plain"    nil ("\\+OK" . "pwd-plain") ("-ERR" . "wpwd"))
+    ("pwd-plain"     nil ("\\+OK" . "list") ("-ERR" . "wpwd"))
     ("apop"          nil ("\\+OK" . "list") ("-ERR" . "wpwd"))
     ("user"          nil ("\\+OK" . "pass") ("-ERR" . "wpwd2"))
     ("pass"          nil ("\\+OK" . "list") ("-ERR" . "wpwd"))
@@ -70,7 +73,7 @@
   (or (mew-pop-get-ssh-process pnm) (mew-pop-get-ssl-process pnm)))
 
 (defun mew-pop-command-capa (pro pnm)
-  (mew-net-status (mew-pop-get-bnm pnm)
+  (mew-net-status (mew-pop-get-status-buf pnm)
 		  "Auth'ing"
 		  nil
 		  (mew-pop-secure-p pnm))
@@ -93,7 +96,8 @@
 	  (progn
 	    (mew-pop-set-auth pnm auth)
 	    (funcall func pro pnm))
-	(mew-pop-debug "<AUTH>" "No preferred POP AUTH.\n"))))
+	(mew-pop-debug "<AUTH>" "No preferred POP AUTH.\n")
+	(mew-pop-command-wpwd2 pro pnm))))
    (t
     (mew-pop-command-pswd pro pnm))))
 
@@ -141,7 +145,8 @@
 	(clear-pass t))
     (goto-char (point-min))
     (cond
-     ((re-search-forward " lock" nil t) ;; very ad hoc
+     ((or (looking-at "-ERR \\[IN-USE\\]") ;; RFC2449
+	  (re-search-forward " lock" nil t)) ;; for old servers: very ad hoc
       (mew-pop-message pnm "The mailbox is locked!")
       (setq clear-pass nil))
      ((or (eq auth 'pass) (eq auth 'user))
@@ -160,15 +165,18 @@
 
 (defun mew-pop-command-wpwd2 (pro pnm)
   (let ((directive (mew-pop-get-directive pnm))
+	(status (mew-pop-get-status pnm))
 	(bnm (mew-pop-get-bnm pnm)))
-    (mew-pop-message pnm "Stronger password scheme should be used!")
+    (if (string= status "auth")
+	(mew-pop-message pnm "No POP AUTH available!")
+      (mew-pop-message pnm "Stronger password scheme should be used!"))
     (mew-passwd-set-passwd (mew-pop-passtag pnm) nil)
     (if (eq directive 'exec)
 	(mew-summary-visible-buffer bnm))
     (mew-pop-command-quit2 pro pnm)))
 
 (defun mew-pop-command-list (pro pnm)
-  (mew-net-status (mew-pop-get-bnm pnm)
+  (mew-net-status (mew-pop-get-status-buf pnm)
 		  "Checking"
 		  nil
 		  (mew-pop-secure-p pnm))
@@ -327,7 +335,7 @@
 
 (defun mew-pop-command-dels (pro pnm)
   (mew-net-status1
-   (mew-pop-get-bnm pnm) "Deleting"
+   (mew-pop-get-status-buf pnm) "Deleting"
    (mew-pop-get-dttl pnm) (mew-pop-get-dcnt pnm) (mew-pop-secure-p pnm))
   (let ((directive (mew-pop-get-directive pnm))
 	(dels (mew-pop-get-dels pnm))
@@ -362,7 +370,7 @@
       (mew-pop-command-retr pro pnm)))))
 
 (defun mew-pop-command-retr (pro pnm)
-  (mew-net-status2 (mew-pop-get-bnm pnm)
+  (mew-net-status2 (mew-pop-get-status-buf pnm)
 		   (mew-pop-get-rttl pnm)
 		   (mew-pop-get-rcnt pnm)
 		   (nth 2 (car (mew-pop-get-rtrs pnm)))
@@ -387,7 +395,7 @@
      ((and (eq directive 'scan) (not get-body))
       (mew-pop-set-truncated pnm t)
       (mew-pop-process-send-string pro "TOP %s 0" num))
-     ((or (= lim 0) (< (string-to-int siz) lim))
+     ((or (= lim 0) (< (string-to-number siz) lim))
       (mew-pop-set-truncated pnm nil)
       (mew-pop-process-send-string pro "RETR %s" num))
      (t
@@ -406,6 +414,7 @@
 	 (del  (nth 3 rtr))
 	 (fld-msg (nth 4 rtr))
 	 (truncated (mew-pop-get-truncated pnm))
+	 (case (mew-pop-get-case pnm))
 	 fld msg vec file msg-file lmsg mark)
     (cond
      ((null fld-msg)
@@ -413,8 +422,7 @@
      ((stringp fld-msg)
       (setq fld fld-msg))
      ((listp fld-msg)
-      (setq fld (nth 0 fld-msg))
-      (setq msg (nth 1 fld-msg))
+      (mew-set '(fld msg) fld-msg)
       (setq lmsg msg)))
     ;; deleting +OK
     (goto-char (point-min))
@@ -429,7 +437,9 @@
     (goto-char (point-min))
     (cond
      (truncated
-      (mew-header-insert-xmu uid siz t))
+      (mew-header-insert-xmu uid siz t (and (eq directive 'inc)
+					    (or (mew-pop-get-case pnm)
+						mew-case-default))))
      ((eq directive 'scan)
       (mew-header-insert-xmu uid siz nil))
      ((and (eq directive 'get) (mew-folder-popp (mew-case:folder-folder fld)))
@@ -451,7 +461,11 @@
       (mew-scan-set-folder vec fld)
       (mew-scan-set-message vec msg)
       (if (or (eq directive 'inc) (eq directive 'scan))
-	  (setq mark (mew-scan-inbox-action vec)))
+	  (setq mark (mew-scan-inbox-action vec case)))
+      (if (and mark
+	       (eq directive 'scan)
+	       (not (memq mark (list mew-mark-delete mew-mark-unlink))))
+	  (setq mark nil))
       (mew-scan-body vec)
       (mew-set-buffer-multibyte nil)
       (mew-scan-insert-line fld vec width lmsg mark)
@@ -482,7 +496,8 @@
 ;;;
 
 (defvar mew-pop-auth-alist
-  '(("CRAM-MD5" mew-pop-command-auth-cram-md5)))
+  '(("CRAM-MD5" mew-pop-command-auth-cram-md5)
+    ("PLAIN"    mew-pop-command-auth-plain)))
 
 (defsubst mew-pop-auth-get-func (auth)
   (nth 1 (mew-assoc-case-equal auth mew-pop-auth-alist 0)))
@@ -502,6 +517,18 @@
     (setq cram-md5 (mew-cram-md5 user passwd challenge))
     (mew-pop-process-send-string pro cram-md5)))
 
+(defun mew-pop-command-auth-plain (pro pnm)
+  (mew-pop-process-send-string pro "AUTH PLAIN")
+  (mew-pop-set-status pnm "auth-plain"))
+
+(defun mew-pop-command-pwd-plain (pro pnm)
+  (let* ((prompt (format "POP PLAIN password (%s): "
+                         (mew-pop-get-account pnm)))
+         (passwd (mew-pop-input-passwd prompt pnm))
+	 (user (mew-pop-get-user pnm))
+	 (plain (mew-base64-encode-string (format "\0%s\0%s" user passwd))))
+    (mew-pop-process-send-string pro "%s" plain)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Sub functions
@@ -618,7 +645,8 @@
 	 (pnm (mew-pop-info-name case))
 	 (buf (get-buffer-create (mew-pop-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
-	 process sshname sshpro sslname sslpro lport)
+	 process sshname sshpro sslname sslpro lport tls
+	 thread-info to-buf)
     (if (mew-pop-get-process pnm)
 	(message "Another POP process is running. Try later")
       (cond
@@ -630,7 +658,8 @@
 	  (when lport
 	    (setq process (mew-pop-open pnm "localhost" lport no-msg)))))
        (sslp
-	(setq sslpro (mew-open-ssl-stream server sslport))
+	(if (string= port sslport) (setq tls mew-tls-pop))
+	(setq sslpro (mew-open-ssl-stream case server sslport tls))
 	(when sslpro
 	  (setq sslname (process-name sslpro))
 	  (setq lport (mew-ssl-pnm-to-lport sslname))
@@ -645,6 +674,7 @@
 	      (mew-summary-visible-buffer bnm))
 	(mew-summary-lock process "POPing" (or sshpro sslpro))
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
+	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
 	(mew-info-clean-up pnm)
 	(mew-pop-set-no-msg pnm no-msg) ;; must come here
@@ -661,26 +691,40 @@
 	(mew-pop-set-status pnm "greeting")
 	(mew-pop-set-directive pnm directive)
 	(mew-pop-set-bnm pnm bnm)
+	(mew-pop-set-status-buf pnm bnm)
 	(mew-pop-set-rcnt pnm 1)
 	(mew-pop-set-dcnt pnm 1)
 	(mew-pop-set-rttl pnm 0)
 	(mew-pop-set-dttl pnm 0)
 	(mew-pop-set-size pnm (mew-pop-size case))
+	(mew-pop-set-case pnm case)
 	(mew-pop-set-body-lines pnm (mew-pop-body-lines case))
 	(cond
 	 ((eq directive 'biff)
 	  (mew-pop-set-passwd pnm (nth 0 args))) ;; password
 	 ((eq directive 'inc)
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
 	  (mew-pop-set-flush pnm (nth 0 args)) ;; no-flush
 	  (mew-pop-set-delete pnm (mew-pop-delete case)))
 	 ((eq directive 'get)
-	  (mew-pop-set-refs pnm (nth 0 args)))
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
+	  (mew-pop-set-refs pnm (nth 0 args))
+	  (setq thread-info (nth 1 args))
+	  (mew-pop-set-thread-info pnm thread-info)
+	  (setq to-buf (nth 0 thread-info))
+	  (when to-buf
+	    (mew-pop-set-status-buf pnm to-buf)
+	    (save-excursion
+	      (set-buffer to-buf)
+	      (mew-summary-lock process "POPing" (or sshpro sslpro)))))
 	 ((eq directive 'scan)
 	  (mew-pop-set-range pnm (nth 0 args))
 	  (mew-pop-set-get-body pnm (nth 1 args))
-	  (when (mew-pop-get-range pnm)
-	    (mew-pop-set-mdb pnm (mew-summary-mark-collect4))
-	    (mew-net-folder-clean)))
+	  (if (mew-pop-get-range pnm)
+	      (progn
+		(mew-pop-set-mdb pnm (mew-summary-mark-collect4))
+		(mew-net-folder-clean))
+	    (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))))
 	 ((eq directive 'sync)
 	  )
 	 ((eq directive 'exec)
@@ -716,7 +760,7 @@
      (mew-set-buffer-multibyte nil)
      (insert string)
      (when (string= status "retr")
-       (mew-net-status2 (mew-pop-get-bnm pnm)
+       (mew-net-status2 (mew-pop-get-status-buf pnm)
 			(mew-pop-get-rttl pnm)
 			(mew-pop-get-rcnt pnm)
 			(nth 2 (car (mew-pop-get-rtrs pnm)))
@@ -757,7 +801,7 @@
 	 (rttl (mew-pop-get-rttl pnm))
 	 (dttl (mew-pop-get-dttl pnm))
 	 (left (mew-pop-get-left pnm))
-	 (bnm (mew-pop-get-bnm pnm))
+	 (bnm (or (mew-pop-get-bnm pnm) (current-buffer)))
 	 (flush (mew-pop-get-flush pnm))
 	 (kils (mew-pop-get-kils pnm))
 	 (hlds (mew-pop-get-hlds pnm))
@@ -765,7 +809,8 @@
 	 (done (mew-pop-get-done pnm))
  	 (error (mew-pop-get-error pnm))
 	 (file (mew-expand-folder bnm mew-pop-msgid-file))
-	 (buf (process-buffer process)))
+	 (buf (process-buffer process))
+	 (thread-info (mew-pop-get-thread-info pnm)))
     (mew-pop-debug "POP SENTINEL" event)
     (set-process-buffer process nil)
     (set-buffer bnm)
@@ -793,9 +838,10 @@
 	    (when recovered
 	      (mew-pop-message
 	       pnm
-	       "%d message retrieved. %d messages are left due to an error."
+	       "%d message retrieved. %d messages are left due to an error"
 	       (- rttl lefts) lefts)
 	      (mew-summary-folder-cache-save))))
+      (if thread-info (mew-summary-retrieve-message-for-thread thread-info))
       (cond
        (error 
  	;; retain the error message
@@ -854,7 +900,7 @@
 	 (t
 	  (mew-pop-message pnm "%d messages deleted" dttl))))))
     ;;
-    (mew-net-status-clear (mew-pop-get-bnm pnm))
+    (mew-net-status-clear (mew-pop-get-status-buf pnm))
     (mew-info-clean-up pnm)
     (set-buffer-modified-p nil)
     (mew-summary-unlock)
@@ -866,7 +912,7 @@
       (run-hooks 'mew-pop-sentinel-non-biff-hook))
     (run-hooks 'mew-pop-sentinel-hook)
     (when (and mew-auto-flush-queue flush)
-      (mew-smtp-flush-queue mew-case-output))))
+      (mew-smtp-flush-queue mew-case))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -874,32 +920,36 @@
 ;;;
 
 (defun mew-pop-biff ()
-  (let* ((case mew-case-input)
+  (let* ((case mew-case)
 	 (inbox (mew-proto-inbox-folder nil case))
+	 (case:inbox (mew-case-folder case inbox))
 	 (tag (mew-pop-passtag2 case))
 	 passwd)
-    (save-excursion
-      (set-buffer inbox)
-      (when (and (mew-summary-exclusive-p 'no-msg)
-		 (and mew-use-cached-passwd
-		      (setq passwd (mew-passwd-get-passwd tag))))
-	(mew-pop-retrieve case 'biff inbox passwd)))))
+    (when (get-buffer case:inbox)
+      (save-excursion
+	(set-buffer case:inbox)
+	(when (and (mew-summary-exclusive-p 'no-msg)
+		   (and mew-use-cached-passwd
+			(setq passwd (mew-passwd-get-passwd tag))))
+	  (mew-pop-retrieve case 'biff case:inbox passwd))))))
 
 (defun mew-pop-check ()
   "See if messages are arrived by POP."
   (interactive)
-  (let* ((case mew-case-input)
-	 (inbox (mew-proto-inbox-folder nil case)))
-    (save-excursion
-      (set-buffer inbox)
-      (when (mew-summary-exclusive-p)
-	(mew-pop-retrieve case 'biff inbox)))))
+  (let* ((case mew-case)
+	 (inbox (mew-proto-inbox-folder nil case))
+	 (case:inbox (mew-case-folder case inbox)))
+    (when (get-buffer case:inbox)
+      (save-excursion
+	(set-buffer case:inbox)
+	(when (mew-summary-exclusive-p)
+	  (mew-pop-retrieve case 'biff case:inbox))))))
 
 (provide 'mew-pop)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2003 Mew developing team.
+;; Copyright (C) 1999-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
