@@ -261,13 +261,15 @@ guessed values."
     (string-match regex string)
     (setq match (cdr (cdr (match-data))))
     (while match
-      (setq str (substring string (car match) (car (cdr match))))
+      (setq str nil)
+      (if (car match)
+	  (setq str (substring string (car match) (car (cdr match)))))
       (setq strs (cons str strs))
       (setq match (cdr (cdr match))))
     (setq strs (cons nil (nreverse strs))) ;; cons nil for 0th
     (while (string-match "\\\\\\([1-9]\\)" template)
-      (setq repl (nth (string-to-int (match-string 1 template)) strs))
-      (setq template (replace-match repl nil nil template)))
+      (setq repl (nth (string-to-number (match-string 1 template)) strs))
+      (setq template (replace-match (or repl "") nil nil template)))
     template))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -393,11 +395,11 @@ guessed values."
   (let* ((msgid (mew-idstr-get-first-id
 		 (mew-header-get-value mew-message-id:)))
 	 (folder (car chosen))
-	 ;; ohter people's honest opinion and my honest opinion.
+	 ;; other people's honest opinion and my honest opinion.
 	 (oho info)
 	 (mho (cdr (assoc 'mew-refile-guess-by-thread info))))
     (when (and msgid chosen)
-      ;; if my opninion was right, I learn it.
+      ;; if my opinion was right, I learn it.
       ;; or a folder was not in other people's opinion,
       ;; I accept it.
       (catch 'match
@@ -602,13 +604,17 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	  (regexp-quote mew-override-body-close)))
 
 (defun mew-summary-refile-override-body (folders-str)
-  (save-excursion
-    (when (mew-summary-goto-body 'after)
-      (when (looking-at mew-regex-override-body)
+  (let* ((open mew-override-body-open)
+	 (close mew-override-body-close)
+	 (len (+ (length folders-str) (length open) (length close))))
+    (save-excursion
+      (when (mew-summary-goto-body 'after)
+	(when (looking-at mew-regex-override-body)
+	  (mew-elet
+	   (delete-region (point) (match-end 0))))
 	(mew-elet
-	 (delete-region (point) (match-end 0))))
-      (mew-elet
-       (insert-and-inherit mew-override-body-open folders-str mew-override-body-close)))))
+	 (insert-and-inherit open folders-str close))))
+    len))
 
 (defun mew-summary-refile-remove-body ()
   (save-excursion
@@ -633,6 +639,18 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	(insert "== src=" fld " dst=" folders-str " id=" msg-id)
 	(insert " date=" (mew-time-ctz-to-logtime (current-time)) "\n")))))
 
+(defun mew-summary-refile-unlog (fld)
+  (save-excursion
+    (let ((msg-id (mew-summary-my-id))
+	  start)
+      (set-buffer (get-buffer-create (mew-refile-log-buffer fld)))
+      (goto-char (point-min))
+      (when (search-forward (concat " id=" msg-id) nil t)
+	(beginning-of-line)
+	(setq start (point))
+	(forward-line)
+	(delete-region start (point))))))
+
 (defun mew-refile-decode-subject ()
   (save-excursion
     (goto-char (point-min))
@@ -668,10 +686,9 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
      (t
       (if exp-flds
 	  (setq folders exp-flds)
-	(unless auto
+	(unless (or auto (mew-sinfo-get-disp-msg))
 	  ;; need to make a cache or a message buffer.
-	  (mew-summary-toggle-disp-msg 'on)
-	  (mew-summary-display nil))
+	  (mew-summary-display))
 	(setq buf (mew-cache-hit fld msg))
 	(unless buf
 	  (save-excursion
@@ -686,8 +703,10 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	  (save-excursion
 	    (set-buffer fld)
 	    (setq cur-folders (cdr (mew-refile-get msg)))))
-	(setq learn-info (mew-refile-decide-folders
-			  buf msg cur-folders auto exfld))
+	(condition-case nil
+	    (setq learn-info (mew-refile-decide-folders
+			      buf msg cur-folders auto exfld))
+	  (quit (goto-char pos)))
 	(setq folders (car learn-info)))
       ;; we must prevent refiling a message to +queue
       (while folders
@@ -707,17 +726,19 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	    (save-excursion
 	      (set-buffer fld)
 	      (save-excursion
-		(goto-char (point-min))
-		(when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+		(when (or (re-search-forward  (mew-regex-sumsyn-msg msg) nil t)
+			  (re-search-backward (mew-regex-sumsyn-msg msg) nil t))
 		  (mew-refile-reset msg)
 		  (mew-refile-set msg folders)
+		  (if cur-folders (mew-summary-refile-unlog fld))
 		  (mew-summary-refile-log fld folders-str)
 		  (mew-summary-refile-override-body folders-str)
-		  (mew-mark-put mew-mark-refile)
+		  ;; marked in mew-mark-put-mark
 		  (set-buffer-modified-p nil)))))
 	   (t
 	    (mew-refile-reset msg)
 	    (mew-refile-set msg folders)
+	    (if cur-folders (mew-summary-refile-unlog fld))
 	    (mew-summary-refile-log fld folders-str)))
 	  (mew-summary-refile-override-body folders-str)
 	  (mew-mark-put-mark mew-mark-refile no-msg)
@@ -738,10 +759,11 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	mew-inherit-refile-case
 	mew-inherit-refile-proto
 	fld msg guess)
-    (mew-summary-goto-message)
-    (when (mew-sumsyn-match mew-regex-sumsyn-short)
-      (setq fld (mew-sumsyn-folder-name))
-      (setq msg (mew-sumsyn-message-number)))
+    (save-excursion
+      (mew-summary-goto-message)
+      (when (mew-sumsyn-match mew-regex-sumsyn-short)
+	(setq fld (mew-sumsyn-folder-name))
+	(setq msg (mew-sumsyn-message-number))))
     (setq mew-inherit-refile-case  (mew-case:folder-case fld))
     (setq mew-inherit-refile-proto (mew-case:folder-folder fld))
     (with-temp-buffer
@@ -751,8 +773,8 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
       (setq guess (mew-refile-guess nil t)))
     (mew-window-configure 'message)
     ;; message buffer
+    (mew-summary-display-preamble)
     (mew-elet
-     (mew-erase-buffer)
      (save-excursion
        ;; report result of guess.
        (insert (format "** Guess result: %s\n" (car guess)))
@@ -770,8 +792,7 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	 (insert (format "return: %s\n"
 			 (mapconcat 'identity (cdr (car guess)) ",")))
 	 (setq guess (cdr guess)))))
-    (mew-message-clear-end-of)
-    (set-buffer-modified-p nil)
+    (mew-summary-display-postscript 'no-hook)
     (select-window win)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -800,11 +821,11 @@ refile folder."
 (defun mew-summary-auto-refile (&optional mew-mark-review-only)
   "Refile each message in the folder automatically. If 
 'mew-refile-auto-refile-skip-any-mark' is non-nil,
-any previousely marked message will be skipped.
+any previously marked message will be skipped.
 If '\\[universal-argument]' is specified, only messages marked with
-'mew-mark-review' will be conserned."
+'mew-mark-review' will be concerned."
   (interactive "P")
-  (mew-summary-or-thread
+  (mew-pickable
    (mew-summary-not-in-draft
     (mew-summary-local-or-imap
      (mew-decode-syntax-delete)
@@ -850,7 +871,7 @@ If '\\[universal-argument]' is specified, only messages marked with
   "\\<mew-summary-mode-map>
 Put the refile mark onto all messages marked with '*', 
 with the current folder as a candidate in addition to guessed folders.
-This is very convenient to refile all messages picked by '\\[mew-summary-search-mark]'."
+This is very convenient to refile all messages  picked by '\\[mew-summary-pick-mark]' and \"\\[mew-summary-grep-mark]\"."
   (interactive)
   (mew-summary-not-in-draft
    (mew-summary-local-or-imap
@@ -871,7 +892,7 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
 (defun mew-summary-mark-refile ()
   "\\<mew-summary-mode-map>
 Put the refile mark onto all messages marked with '*'.
-This is very convenient to refile all messages picked by '\\[mew-summary-search-mark]'."
+This is very convenient to refile all messages picked by '\\[mew-summary-pick-mark]' and \"\\[mew-summary-grep-mark]\"."
   (interactive)
   (mew-summary-not-in-draft
    (mew-summary-local-or-imap
@@ -880,9 +901,9 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
       (message "Mark refiling...")
       (save-excursion
 	(save-window-excursion
-	  (goto-char (point-min))
 	  (catch 'loop
-	    (while (re-search-forward mew-regex-msg-review nil t)
+	    (while (or (re-search-forward  mew-regex-msg-review nil t)
+                       (re-search-backward mew-regex-msg-review nil t))
 	      (setq last (mew-summary-refile-body last nil 'no-msg))
 	      (unless last (throw 'loop t))
 	      (forward-line)))
@@ -892,7 +913,7 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994-2003 Mew developing team.
+;; Copyright (C) 1994-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
