@@ -51,13 +51,15 @@
 effect to this function."
   (let ((mark-delete mew-mark-delete)
 	(mark-review mew-mark-review)
+	(unread (and mew-use-unread-mark (mew-sinfo-get-unread-mark)))
 	duplicated review id)
     (when mew-summary-form-mark-delete
       (if (mew-characterp mew-summary-form-mark-delete)
 	  (setq mark-delete mew-summary-form-mark-delete))
       (when (setq id (mew-idstr-get-first-id (MEW-ID)))
-	(if (member id (mew-sinfo-get-scan-id)) ;; in Summary mode
-	    (setq duplicated t)
+	(if (member id (mew-sinfo-get-scan-id))
+	    (unless (mew-scan-message-invalidp)
+	      (setq duplicated t))
 	  (mew-sinfo-set-scan-id (cons id (mew-sinfo-get-scan-id))))))
     (when mew-summary-form-mark-review
       (if (mew-characterp mew-summary-form-mark-review)
@@ -74,7 +76,8 @@ effect to this function."
     (cond
      (duplicated (char-to-string mark-delete))
      (review     (char-to-string mark-review))
-     (t " "))))
+     (unread     (char-to-string mew-mark-unread))
+     (t          (char-to-string mew-mark-read)))))
 
 (defun mew-summary-form-type ()
   "A function to return a mark of content type."
@@ -131,7 +134,7 @@ effect to this function."
 	(setq year (+ year 2000)))
        ((< year 100)
 	(setq year (+ year 1900))))
-      (int-to-string year))))
+      (number-to-string year))))
 
 (defun mew-summary-form-size ()
   "A function to return the size of the message. Should be used
@@ -140,7 +143,7 @@ with -4. See also 'mew-summary-form-size-0k' and 'mew-summary-form-size-huge'."
 	(SIZE (mew-scan-uid-size (MEW-UID)))
 	(i 0) size unit)
     (if (and SIZE (string-match "^[0-9]+$" SIZE))
-	(setq size (string-to-int SIZE))
+	(setq size (string-to-number SIZE))
       (setq size (mew-file-get-size (mew-expand-folder (MEW-FLD) (MEW-NUM)))))
     (while (and (< i len-1) (>= size 1000))
       (setq size (/ size 1000))
@@ -152,40 +155,43 @@ with -4. See also 'mew-summary-form-size-0k' and 'mew-summary-form-size-huge'."
 	  "0k"
 	(concat
 	 (if (integerp size)
-	     (int-to-string size)
+	     (number-to-string size)
 	   (format "%.0f" size))
 	 unit)))))
 
 (defun mew-summary-form-extract-addr (addr)
   "Extract addr according to 'mew-summary-form-extract-rule'."
-  (let* ((func (if mew-addrbook-for-summary
-		   (mew-addrbook-func mew-addrbook-for-summary)))
-	 (raw (or (mew-addrstr-parse-address addr) ""))
-	 (rules mew-summary-form-extract-rule)
-	 rule ret nickname)
-    (catch 'matched
-      (while rules
-	(setq rule (car rules))
-	(setq rules (cdr rules))
-	(cond
-	 ((and (eq rule 'name)
-	       (or (string-match "^\"\\([^\"]+\\)\"[ \t]*<[^>]+>" addr)
-		   (string-match "^\\([^<]+\\)<[^>]+>" addr)))
-	  (throw 'matched (setq ret (mew-chop (match-string 1 addr)))))
-	 ((and (eq rule 'comment)
-	       (string-match "^[^(]+(\\(.+\\))" addr))
-	  (throw 'matched (setq ret (mew-chop (match-string 1 addr)))))
-	 ((eq rule 'address)
-	  (throw 'matched (setq ret raw)))
-	 ((and (eq rule 'nickname)
-	       ;; set nickname here for efficiency
-	       (or nickname
-		   (setq nickname (if func (funcall func raw)))))
-	  (throw 'matched (setq ret nickname)))
-	 ((and (stringp rule)
-	       (string-match rule addr))
-	  (throw 'matched (setq ret (mew-chop (match-string 1 addr))))))))
-    (or ret addr)))
+  (condition-case nil
+      (let* ((func (if mew-addrbook-for-summary
+		       (mew-addrbook-func mew-addrbook-for-summary)))
+	     (raw (or (mew-addrstr-parse-address addr) ""))
+	     (rules mew-summary-form-extract-rule)
+	     rule ret nickname)
+	(catch 'matched
+	  (while rules
+	    (setq rule (car rules))
+	    (setq rules (cdr rules))
+	    (cond
+	     ((and (eq rule 'name)
+		   (or (string-match "^\"\\([^\"]+\\)\"[ \t]*<[^>]+>" addr)
+		       (string-match "^\\([^<]+\\)<[^>]+>" addr)))
+	      (throw 'matched (setq ret (mew-chop (match-string 1 addr)))))
+	     ((and (eq rule 'comment)
+		   (string-match "^[^(]+(\\(.+\\))" addr))
+	      (throw 'matched (setq ret (mew-chop (match-string 1 addr)))))
+	     ((eq rule 'address)
+	      (throw 'matched (setq ret raw)))
+	     ((and (eq rule 'nickname)
+		   ;; set nickname here for efficiency
+		   (or nickname
+		       (setq nickname (if func (funcall func raw)))))
+	      (throw 'matched (setq ret nickname)))
+	     ((and (stringp rule)
+		   (string-match rule addr))
+	      (throw 'matched (setq ret (mew-chop (match-string 1 addr))))))))
+	(or ret addr))
+    ;; a version of downcase causes error if argument is non-ascii.
+    (error mew-error-broken-address)))
 
 (defun mew-summary-form-from ()
   "A function to return an address.
@@ -235,11 +241,12 @@ Address is converted by 'mew-summary-form-extract-addr'. See also
 'mew-summary-form-header' is prepended. If COLUMN is non-nil, the number
 of thread indent position is returned."
   (let (ret form col)
-    (setq ret (mew-folder-spec folder mew-summary-form-list
+    (setq ret (mew-folder-spec folder
+			       mew-summary-form-list
 			       mew-summary-form-list-string-type
 			       mew-summary-form-list-list-type))
     (if ret
-	(setq form (nth 0 ret) col (nth 1 ret))
+	(mew-set '(form col) ret)
       (setq form mew-summary-form))
     (setq form (append mew-summary-form-header form))
     (if column
@@ -263,6 +270,12 @@ of thread indent position is returned."
 	  (setq col (1+ col))))))
     (or ret mew-thread-column)))
 
+(defun mew-get-unread-mark (folder)
+  (car (mew-folder-spec folder
+			mew-unread-mark-list
+			mew-unread-mark-list-string-type
+			mew-unread-mark-list-list-type)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; The engine function to call mew-summary-form-*
@@ -276,7 +289,7 @@ of thread indent position is returned."
 	  spec)
       (if (symbolp spec)
 	  (setq width 1 func spec)
-	(setq width (nth 0 spec) func (nth 1 spec)))
+	(mew-set '(width func) spec))
       (when (and (integerp width) (symbolp func))
 	(when (= width 0)
 	  (setq width (- WIDTH TOTAL 1))
@@ -367,16 +380,16 @@ of thread indent position is returned."
 	   (mew-refile-reset msg)
 	   (mew-refile-set msg (mew-split mark-or-dst ?,))
 	   (mew-summary-refile-log folder mark-or-dst)
-	   (mew-summary-refile-override-body mark-or-dst)
+	   (setq med (+ med (mew-summary-refile-override-body mark-or-dst)))
 	   (mew-mark-put mew-mark-refile))
+	  ((mew-characterp mark-or-dst) ;; mew-inbox-action-alist
+	   (mew-mark-put mark-or-dst))
 	  (mark
 	   (mew-summary-mark-as mark))
 	  ((and mew-use-highlight-mark ;; mew-summary-form-mark
 		(setq mark (mew-summary-get-mark)) ;; duplicated, etc
 		(setq face (mew-highlight-mark-get-face mark)))
-	   (put-text-property beg med 'face face))
-	  ((mew-characterp mark-or-dst) ;; mew-inbox-action-alist
-	   (mew-summary-mark-as mark-or-dst)))
+	   (put-text-property beg med 'face face)))
 	 (if mew-use-highlight-mouse-line
 	     (put-text-property
 	      beg med 'mouse-face mew-highlight-mouse-line-face))
@@ -442,8 +455,8 @@ of thread indent position is returned."
     (if (and n (< n len)) (setq ali (nth n mew-scan-fields-alias)))
     (if (stringp ali) (symbol-function (intern-soft (concat "MEW-" ali))))))
 
-(defun mew-scan-inbox-action (mew-vec)
-  (let ((alist mew-inbox-action-alist)
+(defun mew-scan-inbox-action (mew-vec case)
+  (let ((alist (mew-inbox-action-alist case))
 	ent key act val val-func ret mark-or-dst regex-list)
     (catch 'loop
       (while alist
@@ -574,35 +587,35 @@ of thread indent position is returned."
 ;;;
 
 (defsubst mew-scan-uid-uid (uid)
-  (nth 0 (mew-split uid 32)))
+  (nth 0 (mew-split uid mew-sp)))
 
 (defsubst mew-scan-uid-size (uid)
-  (nth 1 (mew-split uid 32)))
+  (nth 1 (mew-split uid mew-sp)))
 
-(defun mew-header-insert-xmu (uid siz truncated)
+(defun mew-header-insert-xmu (uid siz truncated &optional case)
   (when (and (stringp uid) (stringp siz))
-    (setq siz (int-to-string (string-to-int siz))) ;; removing 0
+    (setq siz (number-to-string (string-to-number siz))) ;; removing 0
     (let (fields)
-      (if truncated
-	  (setq fields (concat uid " 0" siz)) ;; e.g. 0500 == truncated
-	(setq fields (concat uid " " siz)))
+      (if (not truncated)
+	  (setq fields (concat uid " " siz))
+	(setq fields (concat uid " 0" siz)) ;; e.g. 0500 == truncated
+	(if case (setq fields (concat fields " " case))))
       (save-excursion
 	(mew-header-delete-lines (list mew-x-mew-uidl:)))
       (mew-header-insert mew-x-mew-uidl: fields 'no-fold))))
 
 (defun mew-scan-message-truncatedp ()
-  (let ((siz (mew-scan-uid-size (MEW-UID))))
-    (and siz (string-match "^0" siz))))
+  (mew-msg-truncatedp (mew-scan-uid-size (MEW-UID))))
 
 (defun mew-scan-message-invalidp ()
-  (and (MEW-NUM) (string-match "^0" (MEW-NUM))))
+  (mew-msg-invalidp (MEW-NUM)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Scanning a folder
 ;;;
 
-(defun mew-summary-ls (&optional header-only goend)
+(defun mew-summary-ls (&optional header-only goend update)
   "List this folder asynchronously.
 
 In a LOCAL folder: messages in the local folder are scanned according
@@ -611,7 +624,7 @@ to the range which you specify.
 In a REMOTE folder: messages in the server's folder are cached
 according to the range which you specify. If
 'mew-pop-header-only'/'mew-imap-header-only'/'mew-nntp-header-only' is
-non-nil, only headers of messages are cacched. If executed with
+non-nil, only headers of messages are cached. If executed with
 '\\[universal-argument]', these variables are considered reversed."
   (interactive "P")
   (mew-summary-only
@@ -625,12 +638,18 @@ non-nil, only headers of messages are cacched. If executed with
 	    scanp range dir-newp)
        (mew-summary-folder-cache-load)
        (cond
+	(update
+	 (setq askp nil)
+	 (setq range nil) ;; update
+	 (setq scanp t))
 	((interactive-p) ;; "s"
 	 (setq askp t)
 	 (setq scanp t))
 	((mew-summary-folder-dir-newp) ;; "g"
 	 (setq askp nil)
 	 (setq scanp t)))
+       ;; for mew-summary-exchange-point.
+       (mew-sinfo-set-ret-pos (point))
        (if (mew-summary-folder-dir-newp) (setq dir-newp t))
        (if (or (interactive-p) goend) (goto-char (point-max)))
        (set-buffer-modified-p nil)
@@ -646,7 +665,7 @@ non-nil, only headers of messages are cacched. If executed with
 		(not (mew-folder-imap-queuep)))
 	   (if (and dir-newp (mew-folder-imapp fld))
 	       (mew-local-retrieve 'scan bnm (mew-range-update bnm))
-	     (setq range (mew-input-range-remote bnm))
+	     (if askp (setq range (mew-input-range-remote bnm)))
 	     (when (eq range 'sync)
 	       (setq range nil)
 	       (setq directive 'sync))
@@ -694,7 +713,7 @@ non-nil, only headers of messages are cacched. If executed with
 	 (t2 (mew-file-get-time cache)))
     (if (and (null t1)
 	     (file-directory-p dir)
-	     (mew-dir-messages (mew-expand-folder dir)))
+	     (mew-dir-messages dir))
 	t
       (mew-compare-times t1 t2))))
 
@@ -727,7 +746,8 @@ non-nil, only headers of messages are cacched. If executed with
 
 (defun mew-summary-folder-cache-load ()
   (let* ((folder (mew-summary-folder-name 'ext))
-	 (cache (mew-expand-folder folder mew-summary-cache-file)))
+	 (cache (mew-expand-folder folder mew-summary-cache-file))
+	 refile refs)
     (when (and (file-readable-p cache)
 	       (mew-summary-folder-cache-newp))
       (mew-elet
@@ -739,7 +759,11 @@ non-nil, only headers of messages are cacched. If executed with
        (if (= (point-max) 1)
 	   (setq mew-summary-buffer-raw nil)
 	 (setq mew-summary-buffer-raw t))
-       (mew-mark-undo-mark mew-mark-refile 'no-msg)
+       (mew-sinfo-load)
+       (setq refs (mew-summary-mark-collect mew-mark-refile))
+       (setq refile (mew-summary-mark-recover
+		     (mew-sinfo-get-mark-hist) (mew-sinfo-get-refile) refs))
+       (mew-sinfo-set-refile refile)
        (mew-summary-set-count-line)
        (set-buffer-modified-p nil)))))
 
@@ -771,7 +795,9 @@ non-nil, only headers of messages are cacched. If executed with
 	   (write-region (point-min) (point-max) cache nil 'no-msg))
 	  (mew-set-file-modes cache))
 	(mew-summary-set-count-line)
-	(mew-sinfo-set-cache-time (mew-file-get-time cache))))))
+	(mew-sinfo-set-cache-time (mew-file-get-time cache))
+	(mew-sinfo-save)
+	(mew-sinfo-set-mark-hist nil)))))
 
 ;; See also mew-net-folder-clean.
 (defun mew-summary-folder-cache-clean (folder)
@@ -789,7 +815,7 @@ non-nil, only headers of messages are cacched. If executed with
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
