@@ -13,12 +13,44 @@
 ;;;
 
 (defvar mew-sinfo-list
-  '("scan-id" "find-key" "cursor-line" "direction" "start-point"
-    "cache-time" "summary-form" "refile" "refile-back"
-    "disp-msg" "case" "folder" "proto" "mid-marker" "mid-line" "ttl-line"))
+  '(;; parameters to be saved
+    "refile"  "mark-hist"
+    ;; parameters used internally
+    "scan-id" "find-key" "cursor-line" "direction" "start-point"
+    "cache-time" "summary-form" "unread-mark" "refile-back"
+    "disp-msg" "case" "folder" "proto" "mid-marker" "mid-line" "ttl-line"
+    "mark-review" "ret-pos"))
 
 (mew-blinfo-defun 'mew-sinfo mew-sinfo-list)
 
+(defvar mew-sinfo-list-save-length 2)
+(defvar mew-sinfo-file ".mew-sinfo")
+
+(defun mew-sinfo-save ()
+  (when (mew-summary-p)
+    (let* ((n mew-sinfo-list-save-length)
+	   (data (make-vector n nil))
+	   (bnm (mew-summary-folder-name 'ext))
+	   (file (mew-expand-folder bnm mew-sinfo-file))
+	   (i 0))
+      (while (< i n)
+	(aset data i (aref mew-sinfo i))
+	(setq i (1+ i)))
+      (mew-lisp-save file data nil 'unlimit))))
+
+(defun mew-sinfo-load ()
+  (when (mew-summary-p)
+    (let* ((n mew-sinfo-list-save-length)
+	   (bnm (mew-summary-folder-name 'ext))
+	   (file (mew-expand-folder bnm mew-sinfo-file))
+	   (i 0)
+	   data)
+      (setq data (mew-lisp-load file))
+      (when data
+	(while (< i n)
+	  (aset mew-sinfo i (aref data i))
+	  (setq i (1+ i)))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Macros
@@ -49,15 +81,15 @@
     (t ,@body)))
 
 (defmacro mew-summary-multi-msgs (&rest body)
-  "Collect messages marked with '@' and set their corresponding
+  "Collect messages marked with '*' and set their corresponding
 files to FILES."
-  `(let* ((FLD-MSGS (mew-summary-mark-collect2 mew-mark-multi))
+  `(let* ((FLD-MSGS (mew-summary-mark-collect2 mew-mark-review))
 	  (FLD-MSG-LIST FLD-MSGS) ;; may be used in body
 	  FILES ;; may be used in body
 	  fld-msg)
      (cond
       ((null FLD-MSGS)
-       (message "No %s marks" (char-to-string mew-mark-multi)))
+       (message "No %c marks" mew-mark-review))
       (t
        ;; a little bit complicated because of Virtual mode
        (while FLD-MSGS
@@ -119,13 +151,11 @@ This macro is used to prohibit using a command in Summary mode."
      (message "This command can be used in Thread folder only"))
     (t ,@body)))
 
-(defmacro mew-summary-or-thread (&rest body)
-  "See if this buffer is Thread folder.
-This macro is used to prohibit using a command in Virtual mode
-to the exclusion of Thread folder."
+(defmacro mew-pickable (&rest body)
+  "See if pick can be used for this folder."
   `(cond
-    ((not (mew-summary-or-thread-p))
-     (message "This command can be used in Summary mode or Thread folder"))
+    ((not (mew-pickable-p))
+     (message "This command cannot be used in this folder"))
     (t ,@body)))
 
 (defmacro mew-summary-not-in-queue (&rest body)
@@ -185,7 +215,7 @@ to the exclusion of Thread folder."
 
 (defun mew-summary-get-region ()
   "Get a region according to 'mew-summary-region-include-cursor-line'
-and retern (beg . end)."
+and return (beg . end)."
   (save-excursion
     (let ((beg (region-beginning))
 	  (end (region-end)))
@@ -224,7 +254,8 @@ and retern (beg . end)."
 
 (defvar mew-mode-line-target "%p")
 (defvar mew-mode-line-format
-  `("" mew-summary-buffer-left-msgs
+  `(""
+    (mew-summary-buffer-left-msgs mew-summary-buffer-left-msgs "L%l")
     (mew-summary-buffer-raw "*")
     (mew-summary-buffer-secure-process ,mew-secure-format2)))
 
@@ -260,16 +291,10 @@ and retern (beg . end)."
   (setq mew-summary-buffer-left-msgs nil))
 
 (defun mew-summary-mode-name (name)
-  (let ((in (if (mew-case-default-p mew-case-input)
-		"" mew-case-input))
-	(out (if (mew-case-default-p mew-case-output)
-		 "" mew-case-output)))
-    (if (and (string= in "") (string= out ""))
+  (let ((case (if (mew-case-default-p mew-case) "" mew-case)))
+    (if (string= case "")
 	(setq mode-name name)
-      (if (or (and mew-case-synchronize (string= in out))
-              mew-ask-flush-case)
-	  (setq mode-name (format "%s %s" name in))
-	(setq mode-name (format "%s %s:%s" name in out))))
+      (setq mode-name (format "%s %s" name case)))
     (force-mode-line-update)))
 
 (defun mew-summary-mode-line ()
@@ -319,7 +344,7 @@ and retern (beg . end)."
 
 (defun mew-summary-mode ()
   "\\<mew-summary-mode-map>
-Mew Summary mode:: major mode to visualize messages in a folder.
+Mew Summary mode:: major mode to show a list of messages in a folder.
 
 The keys that are defined for both Summary mode and Virtual mode are:
 
@@ -337,10 +362,9 @@ The keys that are defined for both Summary mode and Virtual mode are:
 	the current message or part again. This is a convenient way to
 	get back to the beginning of the current message or part.
 
-\\[mew-summary-prev-page]	Back-scroll this message. Unnecessary header
-	fields are hidden over the window. Type
-	'\\[mew-summary-prev-page]' to see them when a message is
-	displayed.
+\\[mew-summary-prev-page]	Back-scroll this message. 
+	Unnecessary header fields are hidden over the window. 
+	Type '\\[mew-summary-prev-page]' to see them when a message is displayed.
 
 \\[mew-summary-analyze-again]	1) If this command executed on a message, the cache of the
 	message is removed and the message is analyzed then displayed.
@@ -373,9 +397,10 @@ The keys that are defined for both Summary mode and Virtual mode are:
 	the HTML part and displays it.
 
 \\[mew-summary-analyze-again-alternative]	This command analyzes the message again with 
-	'mew-use-alternative' reversed.
+	'mew-use-alternative' and 'mew-use-text-body' reversed.
 
 \\[mew-summary-display-asis]	Insert this message or part into Message mode in the raw format.
+
 \\[mew-summary-trace-path]	Parse the Received: fields and display them in Message mode.
 
 \\[mew-summary-find-file]	Open this message and makes it read-only.
@@ -385,24 +410,22 @@ The keys that are defined for both Summary mode and Virtual mode are:
 \\[mew-summary-scroll-up]	Make this message scroll up with one line.
 \\[mew-summary-scroll-down]	Make this message scroll down with one line.
 
-\\[mew-summary-display-down]	Move to below then display. Targets
-	includes parts, messages marked with '*', and non-marked
-	messages. When called with '\\[universal-argument]', parts are
-	skipped.
-\\[mew-summary-display-up]	Move to above then display. Targets
-	includes parts, messages marked with '*', and non-marked
-	messages. When called with '\\[universal-argument]', parts are
-	skipped.
+\\[mew-summary-display-down]	Move to below then display. 
+	Targets includes parts, messages marked with '*', and non-marked
+	messages. When called with '\\[universal-argument]', parts are skipped.
+\\[mew-summary-display-up]	Move to above then display. 
+	Targets includes parts, messages marked with '*', and non-marked
+	messages. When called with '\\[universal-argument]', parts are skipped.
 
-\\[mew-summary-goto-line]	Jump to a line according to the number
-	which you input.
+\\[mew-summary-goto-line]	Jump to a line according to the number which you input.
 \\[mew-summary-jump-top]	Go to the beginning of this Summary mode.
 \\[mew-summary-jump-bottom]	Go to the end of this Summary mode.
 
-\\[mew-summary-retrieve]	Retrieve messages to +inbox asynchronously according to
-	'mew-mailbox-type'. 
+\\[mew-summary-retrieve]	Retrieve messages to +inbox asynchronously
+	 according to 'mew-mailbox-type'. 
 	If 'mew-auto-flush-queue' is non-nil, +queue is flushed.
 	If called with '\\[universal-argument]', +queue is not flushed.
+
 \\[mew-summary-retrieve-message]	Retrieve the rest of a truncated('T') message.
 	In a LOCAL folder: 'mew-input-case' is used for a case.  If
 	'mew-pop-delete'/'mew-imap-delete' is non-nil, delete the message from
@@ -417,104 +440,95 @@ The keys that are defined for both Summary mode and Virtual mode are:
 	In a REMOTE folder: messages in the server's folder are cached
 	according to the range which you specify. If
 	'mew-pop-header-only'/'mew-imap-header-only'/'mew-nntp-header-only' is
-	non-nil, only headers of messages are cacched. If executed with
+	non-nil, only headers of messages are cached. If executed with
 	'\\[universal-argument]', these variables are considered reversed.
 
 \\[mew-summary-goto-folder]	Go to the folder which you specify.  If executed with
-	'\\[universal-argument]', the cursor always goes to the bottom of
-	Summary mode.
+	'\\[universal-argument]', the cursor always goes to the bottom of Summary mode.
 
-\\[mew-summary-send]	Write a message. A new draft is prepared in Draft mode.
+\\[mew-summary-write]	Write a message. A new draft is prepared in Draft mode.
+	If called with '\\[universal-argument]', the From: address of
+	the current message is copied to To: in a draft.
 
-\\[mew-summary-reply]	Answer to this message. A new draft is prepared
-	in Draft mode. Mew automatically decides To: and Cc:.
-	When executed with '\\[universal-argument]', answer only to 
-	the sender.
-\\[mew-summary-reply-with-citation]	Answer to this message. A new
-	draft is prepared in Draft mode. Mew automatically decides To:
-	and Cc: and cites the body.When executed with
-	'\\[universal-argument]', answer only to the sender.
+\\[mew-summary-reply]	Answer to this message. A new draft is prepared	in Draft mode.
+	Mew automatically decides To: and Cc:.
+	When executed with '\\[universal-argument]', answer only to the sender.
+\\[mew-summary-reply-with-citation]	Answer to this message. A new draft is prepared in Draft mode.
+	Mew automatically decides To:and Cc: and cites the body.
+	When executed with '\\[universal-argument]', answer only to the sender.
 
-\\[mew-summary-forward]	Forward this message to a third person. A new
-	draft is prepared in Draft mode and this message is
+\\[mew-summary-forward]	Forward this message to a third person. 
+	A new draft is prepared in Draft mode and this message is
 	automatically attached.
-\\[mew-summary-multi-forward]	Forward messages marked with '@' to a
-	third person. A new draft is prepared in Draft mode and this
-	message is automatically attached.
+\\[mew-summary-multi-forward]	Forward messages marked with '*' to a third person.
+	A new draft is prepared in Draft mode and 
+	this message is automatically attached.
 
-\\[mew-summary-reedit]	Edit this message again to retry sending. Or
-	edit this rfc822 part typically included MIME-encapsulated
-	error message.  In the +draft folder, it just edits the
-	message. Otherwise, copy the message to the +draft folder,
-	then edit.
+\\[mew-summary-reedit]	Edit this message again to retry sending. 
+	Or edit this rfc822 part typically included MIME-encapsulated
+	error message.  In the +draft folder, it just edits the	message. 
+	Otherwise, copy the message to the +draft folder, then edit.
 
 \\[mew-summary-edit-again]	Edit an old fashioned error message in
 	which the original message is encapsulated after after strings
 	defined in 'mew-summary-edit-again-regex'
 	(e.g. \"----- Original message follows -----\").
 
-\\[mew-summary-review]	Put the review the '*' mark on this message.
-	Use '\\[mew-summary-display-review-down]' or
-	'\\[mew-summary-display-review-up]' to jump to a message
-	marked with '*'.  It can overlay '@'. The cursor stays always.
-	See also '\\[mew-summary-mark-refile]',
-	'\\[mew-summary-mark-delete]', '\\[mew-summary-mark-regexp]',
-	and '\\[mew-summary-mark-all]'.
+\\[mew-summary-review]	Put the review mark '*' on this message.
+	Use '\\[mew-summary-display-review-down]' or '\\[mew-summary-display-review-up]' to jump to a message marked with '*'.  
+	The cursor stays always.
+	See also '\\[mew-summary-mark-refile]',	'\\[mew-summary-mark-delete]', '\\[mew-summary-mark-regexp]', and '\\[mew-summary-mark-all]'.
 
 \\[mew-summary-display-review-down]	Jump to the message marked with	'*' below.
 \\[mew-summary-display-review-up]	Jump to the message marked with '*' above.
 
-\\[mew-summary-multi]	Put the multi the '@' mark on this message for
-	'\\[mew-summary-multi-forward]', '\\[mew-summary-unshar]', '\\[mew-summary-uudecode]', '\\[mew-summary-burst-multi]'. 
-	It can overlay the '*' mark.  The cursor stays always.
+\\[mew-summary-escape]	Put the escape mark '$'.
 
-\\[mew-summary-unshar]	Apply 'unshar' on messages marked with '@'.
-\\[mew-summary-uudecode]	Apply 'uudecode' on messages marked with '@'.
+\\[mew-summary-unshar]	Apply 'unshar' on messages marked with '*'.
+\\[mew-summary-uudecode]	Apply 'uudecode' on messages marked with '*'.
 
-\\[mew-summary-burst-multi]	De-capsulate messages embedded in the
-	messages marked with '@'.
-\\[mew-summary-join]	Concat Message/Partial fragments marked with '@'
+\\[mew-summary-burst-multi]	De-capsulate messages embedded in the messages marked with '*'.
+\\[mew-summary-join]	Concat Message/Partial fragments marked with '*'
 	to an original message.
 
 \\[mew-summary-undo]	Cancel the mark on this message.
 \\[mew-summary-undo-all]	Cancel all marks according to what you input.
 
-\\[mew-summary-mark-regexp]	Put the '*' mark onto Mall messages
+\\[mew-summary-mark-regexp]	Put the '*' mark onto all messages
 	matched to a regular expression.
 
-\\[mew-summary-mark-all]	Put the '*' mark onto all messages which are
-	not marked.
-\\[mew-summary-mark-review]	Change the '@' mark into the '*' mark.
-\\[mew-summary-mark-multi]	Change the '*' mark into the '@' mark.
+\\[mew-summary-mark-all]	Put the '*' mark onto all messages which are not marked.
+\\[mew-summary-mark-review]	Change the '$' mark into the '*' mark.
+\\[mew-summary-mark-escape]	Change the '*' mark into the '$' mark.
 \\[mew-summary-mark-undo-all]	Unmark all message marked with 'o'/'D'/'X'.
-\\[mew-summary-mark-swap]	Swap the '@' mark and the '*' mark.
+\\[mew-summary-mark-swap]	Swap the '$' mark and the '*' mark.
+\\[mew-summary-exchange-marks]	Exchange the first input mark to the second one.
 
 \\[mew-summary-mark-retrieve-message]	Retrieve the rest of a truncated('T') message marked with '*'.
-	In a LOCAL folder: 'mew-input-case' is used for a case.  If
-	'mew-pop-delete'/'mew-imap-delete' is non-nil, delete the message from
-	the mailbox.  When executed with '\\[universal-argument]',
+	In a LOCAL folder: 'mew-input-case' is used for a case.  
+	If 'mew-pop-delete'/'mew-imap-delete' is non-nil, 
+	delete the message from	the mailbox.
+	When executed with '\\[universal-argument]',
 	'mew-pop-delete'/'mew-imap-delete' is considered reversed.
-	In a REMOTE folder: case is determined by its folder.  The message in
-	the server side is always retained.
+	In a REMOTE folder: case is determined by its folder.  
+	The message in the server side is always retained.
 
-\\[mew-summary-delete]	Put the delete mark(default is 'D') on this
-	message.  This can overlay other marks. When it overlays, the
-	cursor stays on the message. If it marks newly, displays the
-	next message. 
+\\[mew-summary-delete]	Put the delete mark 'D' on this	message.  
+	This can overlay other marks. When it overlays, the cursor stays
+	on the message. If it marks newly, displays the next message. 
 
 \\[mew-summary-clean-trash]	Remove all messages in +trash.
-	When called with '\\[universal-argument]', you can specify 
-	a target folder	to clean up.
+	When called with '\\[universal-argument]', you can specify a target folder to clean up.
 
 \\[mew-summary-save]	Save any parts. If the target is a message, you
 	are asked which you want to save, the entire message or its
 	body. If the target is a non-message part, the part is saved
 	(with line delimiter conversion if it is a text object).
 
-\\[mew-summary-toggle-disp-msg]	Toggle 'Summary mode only' and
-	'Summary & Message mode'. If you choose 'Summary mode only',
-	you can quickly put the delete marks since the next message is
-	not displayed.
+\\[mew-summary-toggle-disp-msg]	Toggle 'Summary mode only' and	'Summary & Message mode'.
+	If you choose 'Summary mode only', you can quickly put 
+	the delete marks since the next message is not displayed.
+
 \\[mew-summary-toggle-header-veil]	If 'mew-use-header-veil' is non-nil, field lines of To: and Cc:
 	over 'mew-header-veil-count' are covered with invisible veils.
 	This commands toggles visibility of these lines.
@@ -523,14 +537,9 @@ The keys that are defined for both Summary mode and Virtual mode are:
 
 \\[mew-summary-cite]	Cite this message to one of drafts.
 
-\\[mew-summary-execute-external] Execute an external command according
-	to Content-Type:.  If called with '\\[universal-argument]',
-	Content-Type: is asked.
+\\[mew-summary-execute-external] Execute an external command according to Content-Type:.
 
-\\[mew-summary-execute-command] Execute an inputed command.
-
-\\[mew-summary-recenter]	Make the current line to the center of
-	Summary mode.
+\\[mew-summary-recenter]	Make the current line to the center of Summary mode.
 
 \\[mew-summary-burst]	De-capsulate embedded messages in MIME format.
 
@@ -544,44 +553,37 @@ The keys that are defined for both Summary mode and Virtual mode are:
 \\[mew-summary-set-case]	Set cases.
 
 \\[mew-summary-suspend]	Suspend Mew then switch to another buffer. All
-	buffers of Mew retain, so you can resume with buffer
-	operations.
+	buffers of Mew retain, so you can resume with buffer operations.
 \\[mew-summary-quit]	Quit Mew. All buffers of Mew are erased.
 \\[mew-summary-kill]	Kill this Summary mode.
 
-\\[mew-summary-convert-local-cs]	Convert the character set of body to
-	the one used locally.
+\\[mew-summary-convert-local-cs]	Convert the character set of body to the one used locally.
 
-\\[mew-summary-decode-old-pgp]	Decrypting/verifying old-fashioned PGP
-	messages.
+\\[mew-summary-decode-old-pgp]	Decrypting/verifying old-fashioned PGP messages.
 
 \\[mew-summary-x-face]	Display xface.
 
-\\[mew-pgp-fetch-key]	Fetch the PGP public key whose key ID appears in
-	the X-Mew: field.
+\\[mew-pgp-fetch-key]	Fetch the PGP public key whose key ID appears in the X-Mew: field.
 
 \\[mew-pgp-select]	Select PGP version.
 
-\\[mew-summary-addrbook-add]	Adding the value of From: in Message mode
-	to Addrbook.  When executed with '\\[universal-argument], it
-	will add personal information.  Otherwise, it will add an
-	alias.
+\\[mew-summary-addrbook-add]	Adding the value of From: in Message mode to Addrbook.
+	When executed with '\\[universal-argument], it will add personal information.
+	Otherwise, it will add an alias.
 
-\\[mew-summary-alias-edit]	Editing the auto alias file which contains a list of alias-address
-	pairs. Remove unnecessary entries and save the file by
-	'\\[save-buffer]'. After saving, the modification is automatically
-	reflected.
+\\[mew-summary-alias-edit]	Editing the auto alias file which contains
+	a list of alias-address pairs. 
+	Remove unnecessary entries and save the file by	'\\[save-buffer]'. 
+	After saving, the modification is automatically	reflected.
 
 \\[mew-summary-kill-subprocess]	Kill a process in Summary mode.
-	Sometime a process accidentally remains in Summary mode.  In
-	this situation, you cannot execute '\\[mew-summary-retrieve]',
-	'\\[mew-summary-ls]', nor '\\[mew-summary-exec]'.  Use this
-	command to solve this problem.
+	Sometime a process accidentally remains in Summary mode.
+	In this situation, you cannot execute '\\[mew-summary-retrieve]', '\\[mew-summary-ls]', nor '\\[mew-summary-exec]'.
+	Use this command to solve this problem.
 
 \\[mew-summary-isearch-forward]	Incremental search forward in Message mode.
 
-\\[mew-summary-isearch-backward]	Incremental search backward in
-	Message mode.
+\\[mew-summary-isearch-backward]	Incremental search backward in Message mode.
 
 \\[mew-summary-print]	Print the content of Message mode according to 'mew-print-function'.
 	If called with '\\[universal-argument]', you can specify a printer name.
@@ -593,7 +595,7 @@ The keys that are defined for both Summary mode and Virtual mode are:
 \\[mew-summary-cmd-msg]	Executing an external command specifying this message as an
 	argument.
 \\[mew-summary-cmd-msgs]	Executing an external command specifying messages
-	marked with '@' as arguments.
+	marked with '*' as arguments.
 
 \\[mew-summary-info]	Display the folder name of this message and the message number.
 	If this is a cache message, both the unique id and the size are 
@@ -605,10 +607,9 @@ The keys that are defined for both Summary mode and Virtual mode are:
 	Otherwise, flush the default queue.
 	If executed with '\\[universal-argument]', you can set the sending case.
 
-
 \\[mew-summary-toggle-8bit]	Toggle 8bit mode(i.e. 'mew-use-8bit').
 \\[mew-summary-cache-clean-up]	Clean-up caches of analyzed messages.
-\\[mew-summary-toggle-warning]	Toggle waring level.
+\\[mew-summary-toggle-warning]	Toggle warning level.
 	If 'mew-warning-field-level' is 2, set it to 1.
 	If 'mew-warning-field-level' is 1, set it to 2.
 \\[mew-summary-toggle-policy]	Toggle decode policy(i.e. 'mew-decode-broken').
@@ -618,22 +619,21 @@ The keys that are defined for both Summary mode and Virtual mode are:
 The following commands are provided for Summary mode only, not for
 Virtual mode.
 
-\\[mew-summary-refile]	Put the refile mark(default is 'o') on this
-	message.  If already marked with 'o', it prints where this
-	message will be refiled. This can overlay other marks. When it
-	overlays, the cursor stays on the message. If it marks newly,
-	displays the next message. If executed with
-	'\\[universal-argument]', it displays how the refile rules
-	work in Message mode.
+\\[mew-summary-refile]	Put the refile mark 'o' on this message. 
+	If already marked with 'o', it prints where this message
+	will be refiled. This can overlay other marks.
+	When it	overlays, the cursor stays on the message.
+	If it marks newly, displays the next message.
+	If executed with'\\[universal-argument]', it displays how the refile rules work
+	in Message mode.
 
-\\[mew-summary-copy]	Put the refile mark(default is 'o') on this message with
+\\[mew-summary-copy]	Put the refile mark 'o' on this message with
 	the current folder as a candidate in addition to guessed folders.
 
-\\[mew-summary-refile-again]	Put a refile mark on this message
-	according to the previous refile folder.
+\\[mew-summary-refile-again]	Put a refile mark on this message according to
+	the previous refile folder.
 
-\\[mew-summary-exec]	Process marked messages. To cancel the '*' mark,
-	use '\\[mew-summary-undo]' or '\\[mew-summary-undo-all]'.
+\\[mew-summary-exec]	Process marked messages. To cancel the '*' mark, use '\\[mew-summary-undo]' or '\\[mew-summary-undo-all]'.
 
 \\[mew-summary-exec-one]	Process the current marked messages.
 
@@ -642,41 +642,37 @@ Virtual mode.
 	If called with '\\[universal-argument]', only messages whose destination is 
 	the same as that of the current message are processed.
 
-\\[mew-summary-mark-refile]	Put the refile mark onto all messages
-	marked with '*'.  This is very convenient to refile all
-	messages picked by '\\[mew-summary-search-mark]'.
+\\[mew-summary-mark-refile]	Put the 'o' mark onto all messages marked with '*'. 
+	This is very convenient to refile all messages picked by '\\[mew-summary-pick-mark]' and \"\\[mew-summary-grep-mark]\".
 
-\\[mew-summary-mark-copy]	Put the refile mark onto all messages
-	marked with '*', with the current folder as a candidate in
-	addition to guessed folders. This is very convenient to refile 
-	all messages picked by '\\[mew-summary-search-mark]'.
+\\[mew-summary-mark-copy]	Put the refile mark onto all messages marked with '*', 
+	with the current folder as a candidate in addition to guessed folders.
+	This is very convenient to refile all messages picked by '\\[mew-summary-pick-mark]' and \"\\[mew-summary-grep-mark]\".
 
-\\[mew-summary-mark-delete]	Put the 'D' mark onto all messages
-	marked with '*'.
-\\[mew-summary-mark-unlink]	Put the 'X' mark onto all messages
-	marked with '*'.
+\\[mew-summary-mark-delete]	Put the 'D' mark onto all messages marked with '*'.
+\\[mew-summary-mark-unlink]	Put the 'X' mark onto all messages marked with '*'.
 
 \\[mew-summary-exec-offline]	Process marked messages *offline*. Messages to be refiled in 
 	a remote folder are moved to the corresponding folder but they are
-	marked invalid. A number of an invalid message starts with '0'.
+	marked invalid. Invalid messages are marked with '#'.
 	Invalid messages will be overridden when the remote folder
 	is scanned online. 
 	A job to delete or refile messages, which is created by this command, 
 	is queued in a queue folder (%queue for IMAP). To flush jobs in 
 	the queue, type '\\[mew-summary-send-message]' in the queue online.
-\\[mew-summary-local-copy]	Copy a cached message in a remote folder to a local folder.
-\\[mew-summary-mark-local-copy]	Copy cached message marked with '*' in a remote folder to a local
-folder.
+
+\\[mew-summary-local-copy]	Copy a message to a local folder.
+\\[mew-summary-mark-local-copy]	Copy messages marked with '*' to a local folder. 
+\\[mew-summary-imap-copy]	Copy a message to the server side of IMAP.
+\\[mew-summary-mark-imap-copy]	Copy messages marked with '*' to the server side of IMAP.
 
 \\[mew-summary-pick-mark]	Pick messages according to a pick pattern which you input,
 	then put the '*' mark onto them. 'mewl' is called as a picking
-	command. If called with '\\[universal-argument]', the target is
-	the region.
+	command. If called with '\\[universal-argument]', the target is	the region.
 
 \\[mew-summary-grep-mark]	Grep messages according to a grep pattern which you input,
 	then put the '*' mark onto them. 'mew-prog-grep' is called as a
-	grep command. If called with '\\[universal-argument]', the target
-	is the region.
+	grep command. If called with '\\[universal-argument]', the target is the region.
 
 \\[mew-summary-cmd-mark]	Grep messages according to a grep pattern which you input,
 	then put the '*' mark onto them. You can asked to specify a grep
@@ -684,12 +680,11 @@ folder.
 	the region.
 
 \\[mew-summary-find-keyword-down]	Display a message marked with '*'
-	and find a keyword and highlight it in the forward
-	direction. The keyword is stored in a buffer local variable in
-	Summary mode. If no key word is set to the variable, this
-	command first asks you a keyword. If you want to change the
-	stored keyword, execute this command with
-	'\\[universal-argument]'.
+	and find a keyword and highlight it in the forward direction.
+	The keyword is stored in a buffer local variable in Summary mode.
+	If no key word is set to the variable, this command first asks you
+	a keyword. If you want to change the stored keyword, 
+	execute this command with '\\[universal-argument]'.
 \\[mew-summary-find-keyword-up]	Display a message marked with '*' and
 	find a keyword and highlight it in the backward direction. The
 	keyword is stored in a buffer local variable in Summary
@@ -699,9 +694,10 @@ folder.
 
 \\[mew-summary-virtual]	Go to Virtual mode which gives a single view
 	to picked messages from multiple folders. Enter a virtual
-	folder name, comma-separated folders, and pick pattern.
+	folder name, comma-separated folders, and a pick pattern.
 
 \\[mew-summary-mark-virtual]	Making Virtual mode for messages marked with '*'.
+	If called with '\\[universal-argument]', you can specify a target mark.
 
 \\[mew-summary-sort]	Sort messages and list them up again.
 	If called with '\\[universal-argument]', sort the region.
@@ -712,10 +708,10 @@ folder.
 \\[mew-summary-delete-folder]	Delete this folder.
 \\[mew-summary-rename-folder]	Rename this folder.
 
-\\[mew-summary-make-thread]	This command makes threads for the
-	Summary mode as Virtual mode, then the cursor jump onto the current
-	message in the Virtual mode. If a corresponding Virtual mode exists,
-	this command just visits the Virtual mode.
+\\[mew-summary-make-thread]	This command makes threads for the Summary mode as Virtual mode,
+	then the cursor jump onto the current message in the Virtual mode. 
+	If a corresponding Virtual mode exists,	this command just visits
+	the Virtual mode.
 \\[mew-summary-regexp-make-thread]	Make threads for messages matched to a specified regular expression.
 
 \\[mew-summary-learn-spam]	Learn that this message is a spam.
@@ -743,7 +739,7 @@ Candidates of RANGE for remote folders are as follows:
 
 - 'sync' means deleting messages which were removed in a remote folder.
 
-PICK <pattern> is as follows (in strong order):
+<pick pattern> is as follows (in the strong order):
 - key=value
 	Match if the 'key' field contains the 'value' string
 	(case-insensitive).
@@ -781,12 +777,16 @@ is equivalent to
   (make-local-variable 'tab-width)
   (make-local-variable 'search-invisible)
   (setq search-invisible nil)
+  (make-local-variable 'line-move-ignore-invisible)
+  (setq line-move-ignore-invisible t)
+  (make-local-variable 'search-invisible)
   (cond
    (mew-gemacs-p
     (jit-lock-register 'mew-summary-cook-region))
    (t
     (make-local-hook 'window-scroll-functions)
     (add-hook 'window-scroll-functions 'mew-summary-cook-window nil 'local)))
+  (add-hook 'kill-buffer-hook 'mew-sinfo-save nil 'local)
   (mew-sinfo-set-disp-msg t)
   ;;
   (mew-summary-mode-name mew-mode-name-summary)
@@ -806,6 +806,7 @@ is equivalent to
 (defun mew-summary-reset ()
   (mew-unhighlight-cursor-line)
   (mew-window-configure 'summary)
+  (mew-remove-buffer (mew-buffer-message))
   (mew-current-set nil nil nil)
   (mew-summary-reset-mode-line)
   (mew-decode-syntax-delete))
@@ -814,7 +815,7 @@ is equivalent to
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
