@@ -12,7 +12,6 @@
 ;;; Variables
 ;;;
 
-(defvar mew-summary-inbox-position (make-marker))
 (defvar mew-last-shell-command "")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -20,13 +19,15 @@
 ;;; Switching to folder
 ;;;
 
-(defun mew-summary-switch-to-folder (case:folder &optional thread func)
+(defun mew-summary-switch-to-folder (case:folder &optional p-folder func)
   (let ((ofolder (mew-summary-folder-name 'ext))
 	new-folder)
     (cond
      ((get-buffer case:folder)
       (switch-to-buffer case:folder)
       (mew-set-buffer-cs mew-cs-m17n)
+      (when (and p-folder (mew-folder-virtualp case:folder))
+	(mew-vinfo-set-parent-folder p-folder))
       (unless (mew-folder-virtualp case:folder) (mew-summary-folder-cache-load))
       (unless (string= ofolder case:folder) (mew-window-configure 'summary)))
      (t
@@ -39,7 +40,7 @@
       (mew-sinfo-set-folder (mew-case:folder-folder case:folder))
       (if (mew-folder-virtualp case:folder)
 	  (progn
-	    (if thread (mew-vinfo-set-thread-p t))
+	    (if p-folder (mew-vinfo-set-parent-folder p-folder))
 	    (mew-virtual-mode))
 	(mew-summary-mode)
 	(if mew-summary-trace-directory (cd (mew-expand-folder case:folder)))
@@ -59,38 +60,44 @@ Summary mode."
   (interactive "P")
   (let* ((proto (mew-proto-to-go (mew-summary-folder-name 'ext)))
 	 (case1 (if (mew-folder-remotep proto)
-		   (mew-sinfo-get-case) ;; a remote folder
-		 mew-case-input)) ;; a local/virtual folder
+		    (mew-sinfo-get-case) ;; a remote folder
+		  mew-case)) ;; a local/virtual folder
 	 (inbox (mew-proto-inbox-folder	proto case1))
-
 	 (case:folder (mew-input-folder case1 inbox))
-	 (case (mew-case:folder-case case:folder))
-	 (folder (mew-case:folder-folder case:folder))
-	 buf win frame goend folder-alist sep)
-    (when (mew-folder-imapp folder)
-      (setq folder-alist (mew-imap-folder-alist case))
-      (setq sep (mew-imap-separator case))
-      (unless (or (assoc folder folder-alist)
-		  (assoc (concat folder sep) folder-alist))
-	(message "%s does not exist" case:folder)
-	(setq case:folder nil)))
-    (when case:folder
-      (when mew-use-other-frame-for-summary
-	(if (setq buf (get-buffer case:folder))
-	    (if (setq win (get-buffer-window buf 0))
-		(progn
-		  (setq frame (window-frame win))
-		  (raise-frame frame)
-		  (select-frame frame)
-		  ;; Ensure, if possible, that frame gets input focus.
-		  (if (eq window-system 'w32)
-		      (w32-focus-frame frame)
-		    (when focus-follows-mouse
-		      (set-mouse-position
-		       (selected-frame) (1- (frame-width)) 0))))
-	      (select-frame (make-frame)))
-	  (select-frame (make-frame))))
-      (mew-summary-visit-folder case:folder goend))))
+	 case folder buf win frame folder-alist sep)
+    (if (null case:folder)
+	(message "The folder does not exist")
+      (setq case (mew-case:folder-case case:folder))
+      (setq folder (mew-case:folder-folder case:folder))
+      (cond
+       ((mew-folder-imapp folder)
+	(setq folder-alist (mew-imap-folder-alist case))
+	(setq sep (mew-imap-separator case))
+	(unless (or (assoc folder folder-alist)
+		    (assoc (concat folder sep) folder-alist))
+	  (message "%s does not exist" case:folder)
+	  (setq case:folder nil)))
+       ((mew-folder-nntpp folder)
+	(unless (assoc folder (mew-nntp-folder-alist case))
+	  (message "%s does not exist" case:folder)
+	  (setq case:folder nil))))
+      (when case:folder
+	(when mew-use-other-frame-for-summary
+	  (if (setq buf (get-buffer case:folder))
+	      (if (setq win (get-buffer-window buf 0))
+		  (progn
+		    (setq frame (window-frame win))
+		    (raise-frame frame)
+		    (select-frame frame)
+		    ;; Ensure, if possible, that frame gets input focus.
+		    (if (eq window-system 'w32)
+			(w32-focus-frame frame)
+		      (when focus-follows-mouse
+			(set-mouse-position
+			 (selected-frame) (1- (frame-width)) 0))))
+		(select-frame (make-frame)))
+	    (select-frame (make-frame))))
+	(mew-summary-visit-folder case:folder goend)))))
 
 (defun mew-summary-visit-folder (folder &optional goend no-ls)
   (let ((dir (mew-expand-folder folder))
@@ -133,16 +140,26 @@ Summary mode."
 ;;; Going to a line
 ;;;
 
-(defun mew-summary-goto-line ()
+(defun mew-summary-goto-line (&optional arg)
   "Jump to a line according to the number which you input.
 If 'mew-summary-goto-line-then-display' is non-nil,
 the message is then displayed."
-  (interactive)
-  (let ((line (string-to-int (read-string "Line No.: " ""))))
-    (when (/= line 0)
-      (goto-line line)
-      (if mew-summary-goto-line-then-display
-	  (mew-summary-display nil)))))
+  (interactive "P")
+  (if arg
+      (let ((msg (read-string "Message No.: " "")) regex)
+	(while (not (string-match mew-regex-message-files3 msg))
+	  (setq msg (read-string "Message No.: " "")))
+	(setq regex (mew-regex-sumsyn-msg msg))
+	(when (or (re-search-forward regex nil t)
+		  (re-search-backward regex nil t))
+	  (beginning-of-line)
+	  (if mew-summary-goto-line-then-display
+	      (mew-summary-display))))
+    (let ((line (string-to-number (read-string "Line No.: " ""))))
+      (when (/= line 0)
+	(goto-line line)
+	(if mew-summary-goto-line-then-display
+	    (mew-summary-display))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -200,7 +217,7 @@ the top message is then displayed."
   (if mew-use-push-mark (mew-push-mark))
   (goto-char (point-min))
   (if mew-summary-jump-top-then-display
-      (mew-summary-display nil)))
+      (mew-summary-display)))
 
 (defun mew-summary-jump-bottom ()
   "Go to the end of this Summary mode.
@@ -211,7 +228,7 @@ the top message is then displayed."
   (goto-char (point-max))
   (unless (bobp) (forward-line -1))
   (if mew-summary-jump-bottom-then-display
-      (mew-summary-display nil)))
+      (mew-summary-display)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -219,35 +236,11 @@ the top message is then displayed."
 ;;;
 
 (defun mew-summary-exchange-point ()
-  "Get back to the position before typing '\\<mew-summary-mode-map>\\[mew-summary-retrieve]'."
+  "\\<mew-summary-mode-map>Get back to the position before typing '\\[mew-summary-retrieve]' or '\\[mew-summary-ls]'."
   (interactive)
   (mew-summary-only
-   (cond
-    ((not (mew-folder-inboxp (mew-summary-folder-name 'ext)))
-     (message "Cannot be used in this folder"))
-    ((not (marker-position mew-summary-inbox-position))
-     (message "No previous folder"))
-    (t
-     (let* ((buf (marker-buffer mew-summary-inbox-position))
-	    (pos (marker-position mew-summary-inbox-position))
-	    (folder (mew-summary-folder-name 'ext))
-	    getback)
-       (cond
-	((equal (current-buffer) buf)
-	 (goto-char pos))
-	((not (get-buffer buf))
-	 (message "Previous folder not exist"))
-	((mew-folder-virtualp folder)
-	 (if (mew-virtual-thread-p folder)
-	     (if (mew-thread-cache-valid-p folder)
-		 (setq getback t)
-	       (message "%s is old" folder))
-	   (setq getback t)))
-	(t
-	 (setq getback t)))
-       (when getback
-	 (switch-to-buffer buf)
-	 (goto-char pos)))))))
+   (let ((pos (mew-sinfo-get-ret-pos)))
+     (if pos (goto-char pos)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -320,17 +313,17 @@ If called with '\\[universal-argument]', the body of the message
 If called with '\\[universal-argument]', you can specify a printer name."
   (interactive "P")
   (mew-summary-display 'redisplay)
-  (when (y-or-n-p "Print this message? ")
-    (let ((printer-name (if arg
-			    (read-string "Printer name: ")
-			  printer-name))
-	  have-header str)
-      (save-excursion
-	(set-buffer (mew-buffer-message))
-	(save-restriction
-	  (widen)
-	  (setq have-header (mew-msghder-p))
-	  (setq str (mew-buffer-substring (point-min) (point-max)))))
+  (let ((printer-name (if arg
+			  (read-string "Printer name: ")
+			printer-name))
+	have-header str)
+    (save-excursion
+      (set-buffer (mew-buffer-message))
+      (save-restriction
+	(widen)
+	(setq have-header (mew-msghdr-p))
+	(setq str (mew-buffer-substring (point-min) (point-max)))))
+    (when (y-or-n-p "Print this message? ")
       (with-temp-buffer
 	(insert str)
 	(if have-header
@@ -357,8 +350,7 @@ If called with '\\[universal-argument]', you can specify a printer name."
 	 (mew-frwlet
 	  mew-cs-dummy mew-cs-text-for-write
 	  (write-region (point-min) (point-max) file nil 'no-msg)))
-       (mew-mime-start-process
-	mew-prog-image/*-ext mew-prog-image/*-ext-arg file)))))
+       (mew-mime-image-ext file)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -373,24 +365,24 @@ argument."
    (let* ((fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number))
 	  (file (mew-expand-folder fld msg))
-	  (command (read-string "Command: ")))
-     (while (not (mew-which-exec command))
-       (setq command (read-string "Command: ")))
-     (message "Executing %s for %s..." command msg)
-     (call-process command nil nil nil file)
-     (message "Executing %s for %s...done" command msg))))
+	  (cmd-args (mew-input-command))
+	  (cmd (car cmd-args)) 
+	  (args (nconc (cdr cmd-args) (list file))))
+     (message "Executing %s for %s..." cmd msg)
+     (apply 'call-process cmd nil nil nil args)
+     (message "Executing %s for %s...done" cmd msg))))
 
 (defun mew-summary-cmd-msgs ()
   "Executing an external command specifying messages
-marked with '@' as arguments."
+marked with '*' as arguments."
   (interactive)
   (mew-summary-multi-msgs
-   (let ((command (read-string "Command: ")))
-     (while (not (mew-which-exec command))
-       (setq command (read-string "Command: ")))
-     (message "Executing %s ..." command)
-     (apply 'call-process command nil nil nil FILES)
-     (message "Executing %s ...done" command))))
+   (let* ((cmd-args (mew-input-command))
+	  (cmd (car cmd-args)) 
+	  (args (nconc (cdr cmd-args) FILES)))
+     (message "Executing %s ..." cmd)
+     (apply 'call-process cmd nil nil nil args)
+     (message "Executing %s ...done" cmd))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -415,129 +407,185 @@ marked with '@' as arguments."
 ;;; Executing external commands
 ;;;
 
-(defun mew-summary-execute-external (&optional arg)
-  "Execute an external command according to Content-Type:. 
-If called with '\\[universal-argument]', Content-Type: is asked."
+(defun mew-summary-ask-ct (ct fname)
+  (let (pt fl)
+    (cond
+     (fname
+      (setq ct (or (mew-ctdb-ct (mew-ctdb-by-file fname))
+		   (mew-content-type (mew-sinfo-get-case))))
+      (setq pt "Type for %s (%s): ")
+      (setq fl fname))
+     (t
+      (setq pt "Type %s(%s): ")
+      (setq fl "")))
+    (mew-input-type pt fl ct mew-mime-content-type-list)))
+
+(defun mew-summary-execute-external (&optional ask-command)
+  "Execute an external command according to Content-Type:.
+If this command is executed on the entire message, the first part
+is chosen as a target. 
+
+If Content-Type of the target part is Application/Octet-Stream,
+Content-Type is automatically asked. And if it has both 
+internal and external visualization mechanisms, you are asked
+which you want to use.
+
+If executed with'\\[universal-argument]', you can specify a command
+to be executed.
+
+See 'mew-mime-content-type' to know how actions can be defined."
   (interactive "P")
   (mew-summary-msg-or-part
-   (mew-summary-execute-base arg nil nil)))
+   (let* ((fld (mew-summary-folder-name))
+	  (msg (mew-summary-message-number2))
+	  (nums (mew-syntax-nums))
+	  (cache (mew-cache-hit fld msg 'must-hit))
+	  (syntax (mew-cache-decode-syntax cache))
+	  (stx (mew-syntax-get-entry syntax nums))
+	  (ctl (mew-syntax-get-ct stx))
+	  (ct (mew-syntax-get-value ctl 'cap))
+	  (win (selected-window))
+	  begin end params cdpl fname program options async was-apo erase-p
+	  pro-opt ent1 ent2)
+     (when (and (string= ct mew-ct-msg)
+		(or (not ask-command)
+		    (not (y-or-n-p "Save the entire message (y) or the first part (n)? "))))
+       (setq stx (mew-syntax-get-part stx))
+       (if (mew-syntax-multipart-p stx)
+	   (setq stx (mew-syntax-get-part stx)))
+       (setq ctl (mew-syntax-get-ct stx))
+       (setq ct (mew-syntax-get-value ctl 'cap)))
+     (setq begin (mew-syntax-get-begin stx))
+     (setq end (mew-syntax-get-end stx))
+     (setq params (mew-syntax-get-params ctl))
+     (setq cdpl (mew-syntax-get-cdp stx))
+     (setq fname (mew-syntax-get-filename cdpl ctl))
+     (when (or (string= ct mew-ct-apo)
+	       (eq (mew-ctdb-prog (mew-ctdb-by-ct ct))
+		   'mew-mime-application/octet-stream))
+       (setq ct (mew-summary-ask-ct ct fname))
+       (setq was-apo t))
+     (if (not ask-command)
+	 (setq program (mew-ctdb-prog (mew-ctdb-by-ct ct)))
+       (setq pro-opt (mew-input-command mew-default-external-program))
+       (setq program (car pro-opt))
+       (setq options (cdr pro-opt)))
+     ;; prog
+     ;; func => called for both
+     ;; (prog ...)
+     ;; (nil prog) == prog
+     ;; (nil (prog...)) == (prog...)
+     ;; (nil func) != func
+     ;; (func func) => need to select
+     ;; (func prog) => need to select
+     ;; (func (prog ...)) => need to select
+     (cond
+      ((stringp program)
+       (setq async t))
+      ((symbolp program)
+       (setq erase-p t))
+      ((listp program)
+       (setq ent1 (nth 0 program))
+       (setq ent2 (nth 1 program))
+       (cond
+	((stringp ent1)
+	 (mew-set '(program options async) program))
+	((null ent1)
+	 (cond
+	  ((stringp ent2)
+	   (setq program ent2)
+	   (setq async t))
+	  ((listp ent2)
+	   (mew-set '(program options async) ent2))
+	  (t ;; symbol
+	   (setq program ent2))))
+	(t ;; symbol
+	 (if (and was-apo
+		  (y-or-n-p "Internal (y) or External (n)? "))
+	     (progn
+	       (setq erase-p t)
+	       (setq program ent1))
+	   (setq program ent2))
+	 (cond
+	  ((stringp program)
+	   (setq async t))
+	  ((listp program)
+	   (mew-set '(program options async) program)))))))
+     (mew-summary-toggle-disp-msg 'on)
+     (mew-window-configure 'message)
+     (if erase-p (mew-erase-buffer))
+     ;; message buffer
+     (unwind-protect
+	 (mew-elet
+	  (cond
+	   ((stringp program)
+	    (mew-summary-execute-program
+	     program ct ctl cache begin end params fname options async))
+	   ((symbolp program)
+	    (mew-summary-execute-symbol
+	     program ct ctl cache begin end params fname was-apo)))
+	  (mew-summary-display-postscript 'no-hook))
+       (select-window win)))))
 
-(defun mew-summary-execute-command (&optional arg)
-  "Execute an inputed command.
-If called with '\\[universal-argument]', options are asked."
-  (interactive "P")
-  (mew-summary-msg-or-part
-   (mew-summary-execute-base
-    nil
-    (read-string "Command: ")
-    (if arg (delete "" (mew-split-quoted
-			(read-string "Options: ") ?\  ?\" ?\"))))))
+(defun mew-summary-execute-symbol (program ct ctl cache begin end params fname was-apo)
+  (cond
+   ((not (fboundp program))
+    (message "%s is not implemented" (symbol-name program)))
+   ((eq program mew-prog-rfc822)
+    (message "%s cannot be executed" mew-prog-rfc822))
+   ((or (mew-ct-imagep ct) (mew-ct-modelp ct))
+    (funcall program cache begin end params fname ct))
+   (t
+    (let ((mew-use-text/html t)
+	  (mbuf (current-buffer))
+	  tbuf)
+      (if (and was-apo (mew-ct-textp ct)) ;; decode-broken?
+	  (with-temp-buffer
+	    (insert-buffer-substring cache begin end)
+	    (mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv)
+	    (setq tbuf (current-buffer))
+	    (setq begin (point-min))
+	    (setq end (point-max))
+	    (set-buffer mbuf)
+	    (funcall program tbuf begin end params))
+	(funcall program cache begin end params))))))
 
-(defun mew-summary-execute-base (ask-type com args)
-  (let* ((fld (mew-summary-folder-name))
-	 (msg (mew-summary-message-number2))
-	 (nums (mew-syntax-nums))
-	 (cache (mew-cache-hit fld msg 'must-hit))
-	 (syntax (mew-cache-decode-syntax cache))
-	 (stx (mew-syntax-get-entry syntax nums))
-	 (ctl (mew-syntax-get-ct stx))
-	 (ct (mew-syntax-get-value ctl 'cap))
-	 begin end params program fname cdpl options async doit pt fl
-	 mew-inherit-ct)
-    (if (not (string= ct mew-ct-msg))
-	(setq doit t)
-      ;; Message/Rfc822
-      (setq stx (mew-syntax-get-part stx))
-      (if (mew-syntax-multipart-p stx)
-	  (progn
-	    (setq stx (mew-syntax-get-entry stx '(1)))
-	    (setq ctl (mew-syntax-get-ct stx))
-	    (setq ct (mew-syntax-get-value ctl 'cap))
-	    (if (mew-ct-textp ct)
-		(setq doit t)))
-	;; singlepart
-	(setq ctl (mew-syntax-get-ct stx))
-	(setq ct (mew-syntax-get-value ctl 'cap))
-	(setq doit t)))
-    (if (not doit)
-	(message "No body")
-      (setq begin   (mew-syntax-get-begin stx))
-      (setq end     (mew-syntax-get-end stx))
-      (setq params  (mew-syntax-get-params ctl))
-      (setq cdpl    (mew-syntax-get-cdp stx))
-      (setq fname   (mew-syntax-get-filename cdpl ctl))
-      (when ask-type
-	(setq mew-inherit-ct ct)
+;;; external
+(defun mew-summary-execute-program (program ct ctl cache begin end params fname options async)
+  (if (not (mew-which-exec program))
+      (message "%s does not exist" program)
+    (let ((file (mew-make-temp-name fname))
+	  wcs)
+      (save-excursion
+	(set-buffer cache)
+	;; NEVER use call-process-region for privacy reasons
 	(cond
-	 (fname
-	  (setq ct (or (mew-ctdb-ct (mew-ctdb-by-file fname))
-		       (mew-content-type (mew-sinfo-get-case))))
-	  (setq pt "Type for %s (%s): ")
-	  (setq fl fname))
+	 ((not (mew-ct-linebasep ct))
+	  (setq wcs mew-cs-binary))
+	 ((not (mew-ct-textp ct))
+	  (setq wcs mew-cs-text-for-write))
 	 (t
-	  (setq pt "Type %s(%s): ")
-	  (setq fl "")))
-	(setq ct (mew-input-type pt fl ct mew-mime-content-type-list)))
-      (setq program (or com (mew-ctdb-prog (mew-ctdb-by-ct ct))))
-      (cond
-       ((and (listp program) (nth 1 program) (symbolp (nth 1 program)))
-	(if (fboundp (nth 1 program))
-	    (save-excursion
-	      (if (or (mew-ct-imagep ct) (mew-ct-modelp ct))
-		  (funcall (nth 1 program) cache begin end params fname)
-		(funcall (nth 1 program) cache begin end params)))
-	  (mew-message-for-summary "Use '\\[mew-summary-execute-command]' or \\[universal-argument] \\[mew-summary-execute-external]'")))
-       (t
-	(cond
-	 ((listp program)
-	  (setq options (nth 1 program))
-	  (setq async (nth 2 program))
-	  (setq program (nth 0 program)))
-	 (t
-	  (setq options args)
-	  (setq async t)))
-	(if (symbolp program)
-	    (message "Function %s is not executable here" program)
-	  (if (not (mew-which-exec program))
-	      (message "Program %s is not found" program)
-	    (let ((file (mew-make-temp-name fname))
-		  wcs)
-	      (save-excursion
-		(set-buffer cache)
-		;; NEVER use call-process-region for privacy reasons
-		(cond
-		 ((not (mew-ct-linebasep ct))
-		  (setq wcs mew-cs-binary))
-		 ((not (mew-ct-textp ct))
-		  (setq wcs mew-cs-text-for-write))
-		 (t
-		  (cond
-		   ((string= mew-ct-htm ct)
-		    (setq wcs (mew-text/html-detect-cs begin end ctl))
-		    (unless (mew-coding-system-p wcs)
-		      (setq wcs (mew-charset-to-cs
-				 (mew-syntax-get-param ctl "charset")))))
-		   ((string= mew-ct-xml ct)
-		    (setq wcs (mew-text/xml-detect-cs begin end ctl))
-		    (unless (mew-coding-system-p wcs)
-		      (setq wcs (mew-charset-to-cs
-				 (mew-syntax-get-param ctl "charset")))))
-
-		   (t
-		    (setq wcs (mew-charset-to-cs
-			       (mew-syntax-get-param ctl "charset")))))
-		  (unless (mew-coding-system-p wcs)
-		    (setq wcs (if mew-decode-broken
-				  (mew-charset-to-cs
-				   (mew-charset-guess-region
-				    begin end))
-				mew-cs-text-for-write)))))
-		(mew-frwlet
-		 mew-cs-dummy wcs
-		 (write-region begin end file nil 'no-msg))
-		(if async
-		    (mew-mime-start-process program options file)
-		  (mew-mime-call-process program options file)))))))))))
+	  (cond
+	   ((or (string= mew-ct-htm ct) (string= mew-ct-xml ct))
+	    (setq wcs (mew-text/html-detect-cs begin end))
+	    (unless (mew-coding-system-p wcs)
+	      (setq wcs (mew-charset-to-cs
+			 (mew-syntax-get-param ctl "charset")))))
+	   (t
+	    (setq wcs (mew-charset-to-cs
+		       (mew-syntax-get-param ctl "charset")))))
+	  (unless (mew-coding-system-p wcs)
+	    (setq wcs (if mew-decode-broken
+			  (mew-charset-to-cs
+			   (mew-charset-guess-region
+			    begin end))
+			mew-cs-text-for-write)))))
+	(mew-frwlet
+	 mew-cs-dummy wcs
+	 (write-region begin end file nil 'no-msg))
+	(if async
+	    (mew-mime-start-process program options file)
+	  (mew-mime-call-process program options file))))))
   
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -559,8 +607,8 @@ If executed with '\\[universal-argument]', you can set the sending case."
 	   (proto (mew-proto-to-flush fld))
 	   qfld case)
       (if (or arg mew-ask-flush-case)
-	  (setq case (mew-input-case mew-case-output "Queue"))
-	(setq case mew-case-output))
+	  (setq case (mew-input-case mew-case"Queue"))
+	(setq case mew-case))
       (if (or (mew-folder-queuep fld) (mew-folder-postqp fld))
 	  (progn
 	    (setq qfld fld)
@@ -600,9 +648,13 @@ any, are killed."
 	    (if (string-match "mew" (process-name (car pros)))
 		(delete-process (car pros)))
 	    (setq pros (cdr pros)))
-	  (if disp-msg (message "All SSH/SSL processes were killed")))
+	  (if disp-msg (message "All SSH/SSL/TLS processes were killed")))
       (cond
        ((not (processp process))
+	;; If you type "x" over SSH and type "C-g" to quit,
+	;; refile information remains invisible.
+	;; This command turns it visible.
+	(mew-summary-visible-buffer (current-buffer))
 	(if disp-msg (message "No process to kill. This buffer is unlocked anyway")))
        ((eq process t)
 	;; Emacs is locking
@@ -615,10 +667,10 @@ any, are killed."
 	(delete-process process)
 	(if disp-msg (message "The process was killed")))
        (t
-	;; Local process or SSH/SSL
+	;; Local process or SSH/SSL/TLS
 	(delete-process process)
-	(if disp-msg (message "The process was killed"))))
-      (mew-summary-unlock))))
+	(if disp-msg (message "The process was killed")))))
+    (mew-summary-unlock)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -626,7 +678,7 @@ any, are killed."
 ;;;
 
 (defun mew-summary-unshar ()
-  "Apply 'unshar' on messages marked with '@'."
+  "Apply 'unshar' on messages marked with '*'."
   (interactive)
   (mew-summary-multi-msgs
    (if (y-or-n-p (format "Execute %s for these messages? " mew-prog-unshar))
@@ -636,7 +688,7 @@ any, are killed."
 	 (message "Executing %s...done" mew-prog-unshar)))))
 
 (defun mew-summary-uudecode ()
-  "Apply 'uudecode' on messages marked with '@'."
+  "Apply 'uudecode' on messages marked with '*'."
   (interactive)
   (mew-summary-multi-msgs
    (cond
@@ -667,7 +719,7 @@ any, are killed."
 	 (mew-erase-buffer)
 	 (cd dir)
 	 ;; mew-folder-local is dummy to let mewencode use
-	 ;; the embedded filiname
+	 ;; the embedded filename
 	 (call-process mew-prog-mime-decode tmp t nil
 		       "-d" "-u" "-" mew-folder-local)
 	 (mew-delete-file tmp)
@@ -700,8 +752,9 @@ any, are killed."
 ;;;
 
 (defun mew-summary-join ()
-  "Concat Message/Partial fragments marked with '@' to an original
-message."   (interactive)
+  "Concat Message/Partial fragments marked with '*' to an original
+message."
+  (interactive)
   (mew-summary-only
    (mew-summary-with-mewl
     (mew-summary-multi-msgs
@@ -756,7 +809,7 @@ message."   (interactive)
 		   (error "fragment id mismatch"))
 	     (setq FID fid))
 	   (unless idx (error "no number"))
-	   (setq idx (1- (string-to-int idx)))
+	   (setq idx (1- (string-to-number idx)))
 	   (if (and (>= idx 0) (< idx len))
 	       (aset vec idx num)
 	     (error "invalid number")))
@@ -850,6 +903,7 @@ message."   (interactive)
 	  (message "No messages to be removed in %s" trash)
 	(if (not (y-or-n-p (format "Remove all messages in %s? " case:trash)))
 	    (message "No messages removed in %s" trash)
+	  (if (string= folder trash) (mew-summary-reset))
 	  (message "Removing all messages in %s..." case:trash)
 	  (mew-summary-unlink-msgs case:trash msgs)
 	  (mew-summary-folder-cache-clean case:trash)
@@ -866,7 +920,7 @@ message."   (interactive)
 ;;;
 
 (defun mew-summary-local-copy ()
-  "Copy a cached message in a remote folder to a local folder."
+  "Copy a message to a local folder."
   (interactive)
   (mew-summary-msg-or-part
    (let ((fld (mew-summary-folder-name))
@@ -894,8 +948,9 @@ message."   (interactive)
 	 (message "Copied to %s/%s" dstfld dstmsg)))))))
 
 (defun mew-summary-mark-local-copy (&optional arg)
-  "Copy cached message marked with '*' in a remote folder to a local
-folder."
+  "Copy messages marked with '*' to a local folder.  If called
+with '\\[universal-argument]', only messages marked with '*' in
+the region are handled."
   (interactive "P")
   (mew-summary-not-in-draft
    (let ((mew-use-highlight-x-face nil)
@@ -947,7 +1002,8 @@ folder."
      (mew-insert-file-contents srcfile)
      (goto-char (point-min))
      (mew-header-delete-lines (list mew-x-mew-uidl:))
-     (write-region (point-min) (point-max) dstfile nil 'no-msg))))
+     (write-region (point-min) (point-max) dstfile nil 'no-msg)
+     (mew-set-file-modes dstfile))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -971,7 +1027,8 @@ folder."
 	 (when (y-or-n-p "Delete this folder? ")
 	   (mew-delete-directory-recursively dir)
 	   (mew-local-folder-delete folder)
-	   (mew-summary-goto-folder mew-inbox-folder)
+	   (mew-nmz-folder-index-delete folder)
+	   (mew-summary-visit-folder mew-inbox-folder)
 	   (mew-kill-buffer buf))))
       ((mew-folder-imapp folder)
        (if (mew-folder-node-p folder case)
@@ -987,7 +1044,8 @@ folder."
 		 (setq mew-summary-buffer-process-error nil))
 	     (mew-imap-folder-delete case folder)
 	     (mew-delete-directory-recursively dir) ;; cache
-	     (mew-summary-goto-folder mew-imap-inbox-folder)
+	     (mew-nmz-folder-index-delete case:folder)
+	     (mew-summary-visit-folder mew-imap-inbox-folder)
 	     (mew-kill-buffer buf)))))))))
 
 (defun mew-summary-rename-folder ()
@@ -1004,12 +1062,13 @@ folder."
      (cond
       ((mew-folder-localp folder)
        (setq new-folder (mew-input-local-folder folder))
+       (setq new-dir (mew-expand-folder new-folder))
        (cond
 	((string= folder new-folder)
 	 (message "%s was not moved" folder))
+	((file-exists-p new-dir)
+	 (message "%s was not moved so %s is existed" folder new-folder))
 	(t
-	 (setq new-dir (mew-expand-folder new-folder))
-	 ;; xxx if new-dir exists?
 	 (rename-file dir new-dir)
 	 (mew-summary-switch-to-folder new-folder)
 	 (mew-kill-buffer buf)
@@ -1017,15 +1076,18 @@ folder."
 	     ;; xxx remove all related buffer?
 	     (mew-local-update 'force)
 	   (mew-local-folder-delete folder)
-	   (mew-local-folder-insert new-folder)))))
+	   (mew-local-folder-insert new-folder))
+	 (mew-nmz-folder-index-rename folder new-folder))))
       ((mew-folder-imapp folder)
        (setq new-folder (car (mew-input-refile-folders (list folder) t case mew-folder-imap)))
        (setq case:new-folder (mew-case-folder case new-folder))
+       (setq new-dir (mew-expand-folder case:new-folder))
        (cond
 	((string= folder new-folder)
 	 (message "%s was not moved" folder))
+	((file-exists-p new-dir)
+	 (message "%s was not moved so %s is existed" folder new-folder))
 	(t
-	 (setq new-dir (mew-expand-folder new-folder))
 	 (mew-imap-retrieve case 'rename case:folder case:new-folder)
 	 (mew-timing)
 	 (mew-rendezvous mew-summary-buffer-process)
@@ -1039,9 +1101,10 @@ folder."
 	   (if (mew-folder-node-p folder case)
 	       (if (y-or-n-p "Collect a folder list now? ")
 		   (mew-imap-update case)
-		 (mew-message-for-summary "Type '3\\[mew-status-update]' to collect a folder list."))
+		 (mew-message-for-summary "Type '3\\[mew-status-update]' to collect a folder list"))
 	     (mew-imap-folder-delete case folder)
-	     (mew-imap-folder-insert case new-folder))))))))))
+	     (mew-imap-folder-insert case new-folder))
+	   (mew-nmz-folder-index-rename case:folder case:new-folder)))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1103,7 +1166,7 @@ you choose 'Summary mode only', you can quickly put the delete
 ;;;
 
 (defun mew-summary-toggle-warning ()
-  "Toggle waring level.
+  "Toggle warning level.
 If 'mew-warning-field-level' is 2, set it to 1.
 If 'mew-warning-field-level' is 1, set it to 2."
   (interactive)
@@ -1210,11 +1273,23 @@ See also 'mew-ham-prog' and 'mew-ham-prog-args."
 	   (setq end (1- (point)))
 	   (message (concat "Learned as ham: " (mew-buffer-substring beg end)))))))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Search invisible
+;;;
+
+(defun mew-summary-toggle-invisible ()
+  (interactive)
+  (if (eq search-invisible t)
+      (setq search-invisible 'open)
+    (setq search-invisible t))
+  (message "'search-invisible' is set to '%s'" search-invisible))
+
 (provide 'mew-summary4)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
