@@ -160,23 +160,29 @@ All members must have the same length."
   (mew-summary-thread-region (point-min) (point-max) 'mark))
 
 (defsubst mew-thread-cache-valid-p (vfolder)
-  (let ((ofld (mew-thread-to-folder vfolder)))
-    (when (get-buffer ofld)
-      (save-excursion
-	(equal (progn (set-buffer ofld) (mew-sinfo-get-cache-time))
-	       (progn (set-buffer vfolder) (mew-sinfo-get-cache-time)))))))
+  (let ((cfolder (mew-summary-folder-name 'ext))
+	ofld)
+    (save-excursion
+      (when (get-buffer vfolder)
+	(set-buffer vfolder)
+	(setq ofld (mew-vinfo-get-parent-folder))
+	(and (equal ofld cfolder)
+	     (get-buffer ofld)
+	     (equal (mew-sinfo-get-cache-time)
+		    (progn (set-buffer ofld) (mew-sinfo-get-cache-time))))))))
 
 (defun mew-summary-make-thread (&optional arg)
-  "If called in Summary mode, this command makes threads for the
-Summary mode as Virtual mode, then the cursor jump onto the current
-message in the Virtual mode. If a corresponding Virtual mode exists,
-this command just visits the Virtual mode.
+  "If called in Summary mode or in Virtual mode made of a single
+physical folder, this command makes threads for the Summary mode
+as Virtual mode, then the cursor jump onto the current message in
+the Virtual mode. If a corresponding Virtual mode exists, this
+command just visits the Virtual mode.
 
 If called with '\\[universal-argument]' in Summary mode, make
 threads for messages in the region.
 
-If called in Virtual mode, switch back to the corresponding Summary
-mode and move to the current message."
+If called in Thread (Virtual) mode, switch back to the
+corresponding Summary mode and move to the current message."
   (interactive "P")
   (if (mew-mark-active-p) (setq arg t))
   (if arg
@@ -190,19 +196,21 @@ mode and move to the current message."
 	   fld vfolder)
       (cond
        ((mew-virtual-p)
-	(if (not (mew-thread-p))
-	    (message "No physical folder")
-	  (setq fld (mew-thread-to-folder folder))
+	(cond
+	 ((mew-thread-p)
+	  (setq fld (mew-vinfo-get-parent-folder))
 	  (if (not (and fld (get-buffer fld)))
-	      (message "No physical folder")
+	      (message "No original folder")
 	    (mew-summary-visit-folder fld nil 'no-ls)
 	    (mew-summary-toggle-disp-msg (if disp 'on 'off))
 	    (if (not msg)
 		(goto-char (point-max))
-	      (mew-summary-move-and-display msg)))))
+	      (mew-summary-move-and-display msg))))
+	 ((mew-pickable-p)
+	  (mew-summary-thread-region (point-min) (point-max) nil msg))
+	 (t
+	  (message "No physical folder"))))
        ((and (setq vfolder (mew-folder-to-thread folder))
-	     (get-buffer vfolder)
-	     (mew-virtual-thread-p vfolder)
 	     (mew-thread-cache-valid-p vfolder))
 	(mew-summary-visit-folder vfolder)
 	(mew-summary-toggle-disp-msg (if disp 'on 'off))
@@ -228,10 +236,11 @@ mode and move to the current message."
   "Make threads for messages in a region.  If you want to know how
 threads are created, see 'mew-use-complete-thread'."
   (interactive "r")
-  (mew-summary-only
+  (mew-pickable
    (when (mew-summary-exclusive-p)
-     (let* ((folder (mew-summary-folder-name 'ext))
-	    (vfolder (mew-folder-to-thread folder))
+     (let* ((ofolder (mew-summary-folder-name 'ext))
+	    (vfolder (mew-folder-to-thread (mew-physical-folder ofolder)))
+	    (folder (mew-summary-physical-folder))
 	    (disp (mew-sinfo-get-disp-msg))
 	    (ctime (mew-sinfo-get-cache-time))
 	    (case (mew-sinfo-get-case))
@@ -294,7 +303,7 @@ threads are created, see 'mew-use-complete-thread'."
        (if (null me)
  	   (message "No target messages")
 	 (setq tm2 (current-time))
-	 (mew-summary-switch-to-folder vfolder 'thread)
+	 (mew-summary-switch-to-folder vfolder ofolder)
 	 (mew-erase-buffer)
 	 (mew-hscroll)
 	 (mew-summary-toggle-disp-msg (if disp 'on 'off))
@@ -304,7 +313,6 @@ threads are created, see 'mew-use-complete-thread'."
 	 (mew-vinfo-set-top nil)
 	 (mew-vinfo-set-db db)
  	 (mew-vinfo-set-column column)
-	 (mew-vinfo-set-flds (list folder))
 	 ;;
 	 (message "Making thread (second pass)...")
 	 (setq tm3 (current-time))
@@ -382,7 +390,7 @@ threads are created, see 'mew-use-complete-thread'."
 	 (forward-char -1))
        (delete-region (point) end)
        (when (/= sum width)
-	 (insert (make-string (- sum width) 32)))))))
+	 (insert (make-string (- sum width) mew-sp)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -462,16 +470,17 @@ threads are created, see 'mew-use-complete-thread'."
   (interactive)
   (mew-thread-mark mew-mark-unlink 'valid-only))
 
-(defun mew-thread-mark-multi ()
-  "Put the '@' mark on all messages of the current sub-thread."
+(defun mew-thread-mark-escape ()
+  "Put the '$' mark on all messages of the current sub-thread."
   (interactive)
-  (mew-thread-mark mew-mark-multi))
+  (mew-thread-mark mew-mark-escape))
 
 (defun mew-thread-mark-refile ()
   "Put the 'o' mark on all messages of the current sub-thread."
   (interactive)
   (mew-thread-only
-   (let* ((fld (mew-thread-to-folder (mew-summary-folder-name 'ext)))
+   (let* ((fld (mew-physical-folder (mew-summary-folder-name 'ext)))
+	  ;; mew-vinfo-get-parent-folder cannot be used here
 	  (folders (mew-summary-refile-body nil nil nil 'no-mark))
 	  (folders-str (mew-join "," folders))
 	  (func (lambda ()
@@ -505,7 +514,7 @@ with the current folder as a candidate in addition to guessed folders."
       (setq fld (car ent))
       (setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
       (while msgs
-	(setq msg (int-to-string (car msgs)))
+	(setq msg (number-to-string (car msgs)))
 	(setq msgs (cdr msgs))
 	(when (get-buffer fld)
 	  (save-excursion
@@ -618,7 +627,7 @@ with the current folder as a candidate in addition to guessed folders."
 	(save-excursion
 	  (goto-char (point-min))
 	  (while msgs
-	    (setq msg (int-to-string (car msgs)))
+	    (setq msg (number-to-string (car msgs)))
 	    (setq msgs (cdr msgs))
 	    (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
 	      (mew-thread-undo fld msg)
@@ -649,7 +658,7 @@ a top node, move onto the top of the previous thread."
 	 (message "No more threads")
        (goto-char here)
        (mew-thread-move-cursor)
-       (mew-summary-display nil)))))
+       (mew-summary-display)))))
 
 (defun mew-thread-down ()
   "Move onto the top of the next thread."
@@ -666,7 +675,7 @@ a top node, move onto the top of the previous thread."
        (goto-char here)
        (unless (mew-summary-message-number) (forward-line))
        (mew-thread-move-cursor)
-       (mew-summary-display nil)))))
+       (mew-summary-display)))))
 
 (defun mew-summary-thread-parent ()
   "Move onto the parent message of the current message."
@@ -685,7 +694,7 @@ a top node, move onto the top of the previous thread."
 		(re-search-backward key nil t)))
 	  (progn
 	    (mew-thread-move-cursor)
-	    (mew-summary-display nil)
+	    (mew-summary-display)
 	    (message "Parent found"))
 	(goto-char pos)
 	(message "No parent")))))
@@ -707,7 +716,7 @@ a top node, move onto the top of the previous thread."
 		(re-search-forward key nil t)))
 	  (progn
 	    (mew-thread-move-cursor)
-	    (mew-summary-display nil)
+	    (mew-summary-display)
 	    (message "Child found"))
 	(goto-char pos)
 	(message "No child")))))
@@ -724,7 +733,7 @@ a top node, move onto the top of the previous thread."
       (if (re-search-backward key nil t)
 	  (progn
 	    (mew-thread-move-cursor)
-	    (mew-summary-display nil)
+	    (mew-summary-display)
 	    (message "Sibling found"))
 	(goto-char pos)
 	(message "No sibling")))))
@@ -742,7 +751,7 @@ a top node, move onto the top of the previous thread."
       (if (re-search-forward key nil t)
 	  (progn
 	    (mew-thread-move-cursor)
-	    (mew-summary-display nil)
+	    (mew-summary-display)
 	    (message "Sibling found"))
 	(goto-char pos)
 	(message "No sibling")))))
@@ -904,7 +913,8 @@ mail whose children are hidden, they appear."
 The thread/message is specified with the mark(\\[set-mark-command])."
   (interactive)
   (mew-summary-msg
-   (let ((pos (marker-position (mark-marker))))
+   (let* ((zmacs-regions nil)
+	  (pos (marker-position (mark-marker))))
      (cond
       ((null pos)
        (message "No marker"))
@@ -1050,7 +1060,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2003 Mew developing team.
+;; Copyright (C) 2000-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
