@@ -193,20 +193,37 @@ really deleted."
   "*The field name of anti spam information to be used for IMAP spam filter. 
 If both 'mew-imap-spam-field' and 'mew-imap-spam-word' are defined, 
 messages whose the field contains the word are automatically
-removed or refiled to 'mew-imap-trash-folder' (if defined) 
-while scanning %inbox."
+removed or refiled to 'mew-imap-spam-folder' or 'mew-imap-trash-folder'
+(if defined) while scanning %inbox."
   :group 'mew-imap
-  :type 'string)
+  :type '(choice (const :tag "Not use" nil)
+		 (string :tag "Spam field")))
 
 (defcustom mew-imap-spam-word nil
   "*The word of anti spam information to be used for IMAP spam filter. 
 See 'mew-imap-spam-field'."
   :group 'mew-imap
-  :type 'string)
+  :type '(choice (const :tag "Not use" nil)
+		 (string :tag "Spam word")))
+
+(defcustom mew-imap-spam-pattern nil
+  "*IMAP SEARCH pattern to filter spams.
+If non-nil, 'mew-imap-spam-field' and 'mew-imap-spam-field'
+treated as 'nil'."
+  :group 'mew-imap
+  :type '(choice (const :tag "Not use" nil)
+		 (string :tag "Spam word")))
+
+(defcustom mew-imap-spam-folder nil
+  "*A folder to be used for IMAP spam filter. 
+See 'mew-imap-spam-field'."
+  :group 'mew-imap
+  :type '(choice (const :tag "Not use" nil)
+		 (string :tag "Spam folder")))
 
 ;; must be here
 (defvar mew-basic-folders
-  `(,mew-inbox-folder ,mew-draft-folder ,mew-queue-folder ,mew-postq-folder))
+  (list mew-inbox-folder mew-draft-folder mew-queue-folder mew-postq-folder))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -270,7 +287,7 @@ is re-edited."
   :type 'hook)
 
 (defcustom mew-draft-mode-edit-again-hook nil
-  "*Hook called in Draft mode when a message returned with the old sytale
+  "*Hook called in Draft mode when a message returned with the old style
 is edited again."
   :group 'mew-draft
   :type 'hook)
@@ -299,7 +316,7 @@ Note that this hook is called before composition of the message."
   :type 'hook)
 
 (defcustom mew-post-hook nil
-  "*Hook called before posting/queuing  a NetNews message in Draft mode.
+  "*Hook called before posting/queuing a NetNews message in Draft mode.
 Note that this hook is called before composition of the message."
   :group 'mew-draft
   :type 'hook)
@@ -311,7 +328,7 @@ Note that this hook is called after composition of the message."
   :type 'hook)
 
 (defcustom mew-real-post-hook nil
-  "*Hook called before sending/queuing a NewNews message in Draft mode.
+  "*Hook called before sending/queuing a NetNews message in Draft mode.
 Note that this hook is called after composition of the message."
   :group 'mew-draft
   :type 'hook)
@@ -494,7 +511,7 @@ For more infomation, see the document of '\\<mew-summary-mode-map>\\[mew-summary
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; SSH/SSL
+;;; SSH/SSL/TLS
 ;;;
 
 (defcustom mew-ssh-keep-connection t
@@ -504,7 +521,7 @@ is kept for further connections. This must be 't' for IMAP and NNTP."
   :type 'boolean)
 
 (defcustom mew-ssl-keep-connection t
-  "*If non-nil, an SSL process for each service (POP, IMAP, NNTP, SMTP)
+  "*If non-nil, an SSL/TLS process for each service (POP, IMAP, NNTP, SMTP)
 is kept for further connections. This must be 't' for IMAP and NNTP."
   :group 'mew-net
   :type 'boolean)
@@ -529,15 +546,16 @@ use a string. (e.g. \"25\")"
 (defcustom mew-smtp-ssh-server nil
   "*The name of SSH server which forwards the SMTP port."
   :group 'mew-smtp
-  :type '(choice string (const nil)))
+  :type '(choice (const :tag "Not use" nil)
+		 (string :tag "Ssh server")))
 
 (defcustom mew-smtp-ssl nil
-  "*If non-nil, SMTP connections are made over SSL."
+  "*If non-nil, SMTP connections are made over SSL/TLS."
   :group 'mew-smtp
   :type 'boolean)
 
 (defcustom mew-smtp-ssl-port "465" ;; no universal consensus
-  "*The port for SMTP over SSL."
+  "*The port for SMTP over SSL/TLS."
   :group 'mew-smtp
   :type 'string)
 
@@ -599,7 +617,8 @@ value of 'mew-mail-domain' is used."
   :type '(choice string (const nil)))
 
 (defcustom mew-mbox-command-arg "-u"
-  "*Arguments for 'mew-mbox-command'."
+  "*Arguments for 'mew-mbox-command'.
+For maildir, set this to '-u -d /path/to/mbox'."
   :group 'mew-pop
   :type '(choice string (const nil)))
 
@@ -642,12 +661,12 @@ use a string. (e.g. \"110\")"
   :type '(choice string (const nil)))
 
 (defcustom mew-pop-ssl nil
-  "*If non-nil, POP connections are made over SSL."
+  "*If non-nil, POP connections are made over SSL/TLS."
   :group 'mew-pop
   :type 'boolean)
 
 (defcustom mew-pop-ssl-port "pop3s"
-  "*The port for POP over SSL."
+  "*The port for POP over SSL/TLS."
   :group 'mew-pop
   :type 'string)
 
@@ -664,19 +683,22 @@ t means SASL according to 'mew-pop-auth-list'."
   :group 'mew-pop
   :type '(choice (const apop) (const pass) (const t)))
 
-(defcustom mew-pop-auth-list '("CRAM-MD5")
+(defcustom mew-pop-auth-list '("CRAM-MD5" "PLAIN")
   "*A list of SASL methods in the preferred order.
 Currently, \"CRAM-MD5\" can be used."
   :group 'mew-pop
-  :type '(repeat (choice (const "CRAM-MD5"))))
+  :type '(repeat (choice (const "CRAM-MD5") (const "PLAIN"))))
 
 (defcustom mew-pop-delete t
-  "*Wether or not delete messages on a POP server after retrieval by
+  "*Whether or not delete messages on a POP server after retrieval by
 POP. If t, delete the messages. If nil, retain the messages. If
 number N, delete the messages N days after the first access.
 Otherwise they are not deleted."
   :group 'mew-pop
-  :type 'boolean)
+  :type '(choice
+	  (const :tag "Delete immediately" t)
+	  (const :tag "Retain the messages" nil)
+	  (integer :tag "Delete N days after")))
 
 (defcustom mew-pop-size (* 54 1024) ;; 4K hdr + 50K bdy
   "*The limit size of messages to be retrieved by POP. The default is
@@ -692,7 +714,7 @@ exceeds 'mew-pop-size'."
   :type 'integer)
 
 (defcustom mew-pop-header-only nil
-  "*Wether or not the body of a message is retrieved by POP.
+  "*Whether or not the body of a message is retrieved by POP.
 If non-nil, only header is retrieved."
   :group 'mew-pop
   :type 'boolean)
@@ -700,7 +722,8 @@ If non-nil, only header is retrieved."
 (defcustom mew-pop-proxy-server nil
   "*The name of PROXY server connecting to POP server."
   :group 'mew-pop
-  :type 'string)
+  :type '(choice (const :tag "Not use" nil)
+		 (string :tag "Pop proxy server")))
 
 (defcustom mew-pop-proxy-port nil
   "*The port of PROXY proxy."
@@ -732,12 +755,12 @@ use a string. (e.g. \"143\")"
   :type '(choice string (const nil)))
 
 (defcustom mew-imap-ssl nil
-  "*If non-nil, IMAP connections are made over SSL."
+  "*If non-nil, IMAP connections are made over SSL/TLS."
   :group 'mew-imap
   :type 'boolean)
 
 (defcustom mew-imap-ssl-port "imaps"
-  "*The port for IMAP over SSL."
+  "*The port for IMAP over SSL/TLS."
   :group 'mew-imap
   :type 'string)
 
@@ -760,7 +783,7 @@ Currently, \"CRAM-MD5\" and \"LOGIN\" can be used."
   :type '(repeat (choice (const "CRAM-MD5") (const "LOGIN"))))
 
 (defcustom mew-imap-delete t
-  "*Wether or not delete messages on an IMAP server after retrieval by
+  "*Whether or not delete messages on an IMAP server after retrieval by
 IMAP. If t, delete the messages. If nil, retain the messages. If
 number N, delete the messages N days after the first access.
 Otherwise they are not deleted."
@@ -775,7 +798,7 @@ IMAP server."
   :type 'integer)
 
 (defcustom mew-imap-header-only nil
-  "*Wether or not the body of a message is retrieved by IMAP.
+  "*Whether or not the body of a message is retrieved by IMAP.
 If non-nil, only header is retrieved."
   :group 'mew-imap
   :type 'boolean)
@@ -839,12 +862,12 @@ use a string. (e.g. \"119\")"
   :type '(choice string (const nil)))
 
 (defcustom mew-nntp-ssl nil
-  "*If non-nil, NNTP connections are made over SSL."
+  "*If non-nil, NNTP connections are made over SSL/TLS."
   :group 'mew-nntp
   :type 'boolean)
 
 (defcustom mew-nntp-ssl-port "nntps"
-  "*The port for NNTP over SSL."
+  "*The port for NNTP over SSL/TLS."
   :group 'mew-nntp
   :type 'string)
 
@@ -852,7 +875,8 @@ use a string. (e.g. \"119\")"
   "*The user name on the NNTP server. If non-nil, authentication is
 used."
   :group 'mew-nntp
-  :type 'string)
+  :type '(choice (const :tag "Not use" nil)
+		 (string :tag "Nntp user")))
 
 (defcustom mew-nntp-newsgroup "-fj.mail.reader.mew"
   "*The name of default Newsgroup."
@@ -867,7 +891,7 @@ NNTP server."
   :type 'integer)
 
 (defcustom mew-nntp-header-only nil
-  "*Wether or not the body of a message is retrieved by NNTP.
+  "*Whether or not the body of a message is retrieved by NNTP.
 If non-nil, only header is retrieved."
   :group 'mew-nntp
   :type 'boolean)
@@ -942,6 +966,71 @@ If this value is nil, an appropriate value is set when Mew is booting.
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; unread-mark
+;;;
+
+(defcustom mew-unread-mark-list
+  '((("+inbox" "$inbox" "%inbox" "-") t)
+    (t nil))
+  "*A list to define the read/unread marks for each folder when
+scanning. If VALUE is non-nil, messages are marked as
+unread. Otherwise, message are not marked, that is, marked as
+read.
+
+If KEY is t, all folders matches it. The corresponding VALUE is
+returned always.
+
+If KEY is a string, match-function is applied according to
+'mew-unread-mark-list-string-type'. If matched, the corresponding
+VALUE is returned.
+
+If KEY is a list of strings, match-function is applied according to
+'mew-unread-mark-list-list-type'. If a match found out of the strings,
+the corresponding VALUE is returned.
+
+Candidate values for 'mew-rage-list-string-type' and
+'mew-unread-mark-list-list-type' are as follows:
+
+	'regex		Regular expression.
+	'recursive	Initial substring match. That is, 
+			not only a folder specified but all so
+			all sub-folders are matched.
+	'string		String comparison.
+
+The default value is:
+	'(((\"+inbox\" \"$inbox\" \"%inbox\" \"-\") t)
+	  (t nil))
+
+An example for the case where messages are marked as unread in any
+folders:
+	'((t t))
+
+Note that this variable is effective only when 'mew-use-unread-mark'
+is non-nil.
+"
+  :group 'mew-summary
+  :type '(choice
+	  (const nil)
+	  (repeat
+	   (list (choice (string :tag "Folder")
+			 (repeat :tag "Folders" (string :tag "Folder"))
+			 (const :tag "Other" t))
+		 string))))
+
+(defcustom mew-unread-mark-list-string-type 'regex
+  "*A value to specify an action if the key is a string in 
+'mew-unread-mark-list'. Candidates are 'regex, 'recursive, and 'string."
+  :group 'mew-summary
+  :type '(choice (const regex) (const recursive) (const string)))
+
+(defcustom mew-unread-mark-list-list-type 'recursive
+  "*A value to specify an action if the key is a list of strings in 
+'mew-unread-mark-list'. Candidates are 'regex, 'recursive, and 'string."
+  :group 'mew-summary
+  :type '(choice (const regex) (const recursive) (const string)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Scan
@@ -1146,14 +1235,18 @@ of the From: field is used."
 duplicated messages. If a character, use it as a mark
 instead of the 'D' mark."
   :group 'mew-summary
-  :type 'boolean)
+  :type '(choice (const :tag "Use mew-mark-delete" t)
+		 (const :tag "Not use" nil)
+		 (character :tag "Use another character")))
 
 (defcustom mew-summary-form-mark-review nil
   "*If non-nil, the '*' mark automatically is put onto
 messages destined to me. If a character, use it as a mark
 instead of the '*' mark."
   :group 'mew-summary
-  :type 'boolean)
+  :type '(choice (const :tag "Use mew-mark-review" t)
+		 (const :tag "Not use" nil)
+		 (character :tag "Use another character")))
 
 (defcustom mew-summary-form-size-0k nil
   "*If non-nil, the size of message is displayed as '0k'
@@ -1164,7 +1257,7 @@ if the size is less than 1k byte."
 (defcustom mew-summary-form-size-huge t
   "*If non-nil, the size of message is displayed as 'HUGE'
 when the size is greater than or equal to 1000 after maximum
-quantization by `mew-summary-form-size-unit'."
+quantization by 'mew-summary-form-size-unit'."
   :group 'mew-summary
   :type 'boolean)
 
@@ -1178,11 +1271,11 @@ E.g.
   :type 'string)
 
 (defcustom mew-scan-fields
-  `("Folder:" "Filename:" 
-    ,mew-subj: ,mew-date: ,mew-from: ,mew-to: ,mew-cc:
-    ,mew-ct: ,mew-cte: ,mew-x-mew-uidl:
-    ,mew-message-id: ,mew-in-reply-to: ,mew-references: ,mew-x-mew-ref:
-    ,mew-spam: "Body")
+  (list "Folder:" "Filename:" 
+	mew-subj: mew-date: mew-from: mew-to: mew-cc:
+	mew-ct: mew-cte: mew-x-mew-uidl:
+	mew-message-id: mew-in-reply-to: mew-references: mew-x-mew-ref:
+	mew-spam: "Body")
   "*A list which specifies mewl's output.
 The first element MUST be \"Folder:\".
 The second element MUST be \"Filename:\".
@@ -1204,7 +1297,7 @@ Functions called MEW-FOO will be defined according to this variable.")
 (defvar mew-summary-form-func-prefix "mew-summary-form-"
   "The prefix for symbol used in 'mew-summary-form'.")
 
-(defvar mew-scan-decode-fields `(,mew-subj: ,mew-from: ,mew-to:))
+(defvar mew-scan-decode-fields (list mew-subj: mew-from: mew-to:))
 
 (defcustom mew-thread-column 28
   "A position where thread indentation is inserted
@@ -1268,7 +1361,8 @@ See also 'mew-header-max-length'."
   "*A value for the max number of message IDs in References: when reply.
 Non-integer means no limit."
   :group 'mew-reply
-  :type 'integer)
+  :type '(choice (integer :tag "Max count")
+		 (const :tag "No limit" nil)))
 
 (defcustom mew-lisp-max-length 2000
   "*Mew saves an internal lisp structure to a file truncating
@@ -1369,11 +1463,6 @@ invalid. Otherwise, 'update is used for range in the case."
   :group 'mew-summary
   :type 'boolean)
 
-(defcustom mew-ask-mark-process nil
-  "*If *non-nil*, mark processing when exiting Emacs."
-  :group 'mew-summary
-  :type 'boolean)
-
 (defcustom mew-ask-virtual-folder-name nil
   "*If *non-nil*, ask a virtual folder name."
   :group 'mew-summary
@@ -1395,13 +1484,13 @@ message is stored."
   :type 'boolean)
 
 (defcustom mew-ask-folder-after-burst nil
-  "*If *non-nil*, Mew asks to move to the folder where the bursted
+  "*If *non-nil*, Mew asks to move to the folder where the burst
 messages are stored."
   :group 'mew-summary
   :type 'boolean)
 
 (defcustom mew-ask-pipe nil
-  "*If *non-nil*, Mew confirms when \"\\<mew-summary-mode-map>\\[mew-summary-pipe-message]\" is used."
+  "*If *non-nil*, Mew confirms when '\\<mew-summary-mode-map>\\[mew-summary-pipe-message]' is used."
   :group 'mew-summary
   :type 'boolean)
 
@@ -1597,11 +1686,11 @@ are selected out of leaf folders only."
   :group 'mew-refile
   :type 'boolean)
 
-(defcustom mew-use-case-input-completion t
-  "*If non-nil, 'mew-case-input' is automatically completed when you
+(defcustom mew-use-case-completion t
+  "*If non-nil, 'mew-case' is automatically completed when you
 input a folder.  For example, consider that you input a folder in a
 local folder after typing '\\<mew-summary-mode-map>\\[mew-summary-goto-folder]'.  
-Typing '%' automatically inserts 'mew-case-input' before '%'."
+Typing '%' automatically inserts 'mew-case' before '%'."
   :group 'mew-summary
   :type 'boolean)
 
@@ -1612,19 +1701,14 @@ to the right side."
   :type 'boolean)
 
 (defcustom mew-use-push-mark nil
-  "*If non-nil, \\<mew-summary-mode-map>\\[mew-summary-jump-top] and \\<mew-summary-mode-map>\\[mew-summary-jump-bottom] push the mark."
-  :group 'mew-summary
-  :type 'boolean)
-
-(defcustom mew-use-retrieve t
-  "*If non-nil, '\\<mew-summary-mode-map>\\[mew-summary-retrieve]' can be used."
+  "*If non-nil, '\\<mew-summary-mode-map>\\[mew-summary-jump-top]' and '\\<mew-summary-mode-map>\\[mew-summary-jump-bottom]' push the mark."
   :group 'mew-summary
   :type 'boolean)
 
 (defcustom mew-use-old-pgp nil
-  "*If non-nil, the old fashioned PGP message are used when composing.
+  "*If non-nil, the old fashioned PGP message are used when signing/encrypting.
 Otherwise PGP/MIME is used."
-  :group 'mew-summary
+  :group 'mew-privacy
   :type 'boolean)
 
 (defcustom mew-use-full-alias nil
@@ -1634,6 +1718,19 @@ learned address are used."
   :group 'mew-summary
   :type 'boolean)
 
+(defcustom mew-use-unread-mark nil
+  "*If non-nil, the unread mark is used in Summary mode."
+  :group 'mew-summary
+  :type 'boolean)
+
+(defcustom mew-delete-unread-mark-by-mark t
+  "*If nil, the unread mark remains when the cursor is moved
+to the current message from the previous message by
+putting any mark on the previous one. If non-nil,
+the unread mark is deleted."
+  :group 'mew-summary
+  :type 'boolean)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Appearance
@@ -1740,7 +1837,7 @@ where the cursor jumped to the bottom."
   :type 'boolean)
 
 (defcustom mew-summary-region-include-cursor-line nil
-  "*This value decides wether or not the cursor line is included
+  "*This value decides whether or not the cursor line is included
 to a region."
   :group 'mew-summary
   :type '(choice (const :tag "Include if the cursor is not on the beginning of the line" nil)
@@ -1777,7 +1874,7 @@ to a region."
 
 (defcustom mew-insert-final-newline t
   "*If *non-nil*, and if text/plain does not have final newline,
-insert a newline at the end of buffer. Due to Emacs 19 and XEmacs bug,
+insert a newline at the end of buffer. Due to XEmacs bug,
 mew-break-pages cannot work without this."
   :group 'mew-message
   :type 'boolean)
@@ -1831,6 +1928,16 @@ line."
   :group 'mew-draft
   :type 'boolean)
 
+(defcustom mew-comment-start ">"
+  "*The value to be set to comment-start in Draft mode."
+  :group 'mew-draft
+  :type 'string)
+
+(defcustom mew-comment-start-skip "^> *"
+  "*The value to be set to comment-start-skip in Draft mode."
+  :group 'mew-draft
+  :type 'string)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Draft mode
@@ -1907,12 +2014,12 @@ according to 'mew-cite-format' and 'mew-cite-fields'."
   :type 'function)
 
 (defcustom mew-cite-prefix-confirmp nil
-  "*If *non-nil*, ciattion prefix (such as 'kazu> ') is
+  "*If *non-nil*, citation prefix (such as 'kazu> ') is
 confirmed to be used."
   :group 'mew-cite
   :type 'boolean)
 
-(defcustom mew-cite-fields `(,mew-from: ,mew-subj: ,mew-date:)
+(defcustom mew-cite-fields (list mew-from: mew-subj: mew-date:)
   "*The fields that you want to extract as citation label. 
 If you change this valuable, you must change mew-cite-format.
 The value of the first field becomes the first argument for mew-cite-format.
@@ -2078,7 +2185,7 @@ as the final part."
 (defcustom mew-cs-samba 'shift_jis
   "*A character set before SAMBA encoding."
   :group 'mew-summary
-  :type 'symbol)
+  :type 'coding-system)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -2102,10 +2209,10 @@ in Attachments."
 ;;;
 
 (defcustom mew-fields
-  `(,mew-from: ,mew-to: ,mew-cc: ,mew-dcc: ,mew-fcc: ,mew-bcc: ,mew-subj:
-    ,mew-reply-to: ,mew-followup-to: ,mew-newsgroups: ,mew-distribution:
-    ,mew-resent-from: ,mew-resent-to: ,mew-resent-cc:
-    ,mew-resent-dcc: ,mew-resent-fcc: ,mew-resent-bcc:)
+  (list mew-from: mew-to: mew-cc: mew-dcc: mew-fcc: mew-bcc: mew-subj:
+	mew-reply-to: mew-followup-to: mew-newsgroups: mew-distribution:
+	mew-resent-from: mew-resent-to: mew-resent-cc:
+	mew-resent-dcc: mew-resent-fcc: mew-resent-bcc:)
   "*Completion field list in Draft mode."
   :group 'mew-complete
   :type '(repeat string))
@@ -2140,7 +2247,7 @@ circular domain completion."
 ;;;
 
 (defcustom mew-field-order-for-reediting
-  `(,mew-to: ,mew-newsgroups: ,mew-cc: ,mew-subj: ,mew-from:)
+  (list mew-to: mew-newsgroups: mew-cc: mew-subj: mew-from:)
   "*A field order list for reediting."
   :group 'mew-draft
   :type '(repeat string))
@@ -2150,61 +2257,70 @@ circular domain completion."
 ;;; Fields to be deleted
 ;;;
 
-(defcustom mew-field-delete-common '("From " ">From" "X-Mew")
+(defcustom mew-field-delete-common
+  '("From " ">From" "X-Mew")
   "*A field list to be deleted for reediting/resending/forwarding/saving."
   :group 'mew-draft
-  :type '(repeat string))
+  :type '(choice (const nil)
+		 (repeat :tag "Field list" (string :tag "Field"))))
 
 (defvar mew-field-delete nil) ;; backward compatibility
 
 (defcustom mew-field-delete-for-reediting
   (or mew-field-delete
-      `(,mew-received: "Return-Path:" "Delivery-Date:" "Delivered-To:"
-	,mew-date: ,mew-message-id: ,mew-mv: "Content-"
-	"Precedence:" ,mew-organization: "X-"
-	"Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:"))
+      (list mew-received: "Return-Path:" "Delivery-Date:" "Delivered-To:"
+	    mew-date: mew-message-id: mew-mv: "Content-"
+	    "Precedence:" mew-organization: "X-"
+	    "Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:"
+	    "DomainKey-Signature:"))
   "*A field list to be deleted when edit again."
   :group 'mew-draft
-  :type '(repeat string))
+  :type '(choice (const nil)
+		 (repeat :tag "Field list" (string :tag "Field"))))
 
 (defcustom mew-field-delete-for-others
-  `(,mew-received: "Return-Path:" "Delivery-Date:" "Delivered-To:"
-    ,mew-to: ,mew-cc: ,mew-from: ,mew-organization:
-    "Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:")
+  (list mew-received: "Return-Path:" "Delivery-Date:" "Delivered-To:"
+	mew-to: mew-cc: mew-from: mew-organization:
+	"Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:"
+	"DomainKey-Signature:")
   "*A field list to be deleted when send with different To: and Cc:."
   :group 'mew-draft
-  :type '(repeat string))
+  :type '(choice (const nil)
+		 (repeat :tag "Field list" (string :tag "Field"))))
 
 (defcustom mew-field-delete-for-resending
-  `(,mew-received: "Return-Path:" "Delivery-Date:" "Delivered-To:"
-    "Resent-" "X-Resent-"
-    "Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:")
+  (list mew-received: "Return-Path:" "Delivery-Date:" "Delivered-To:"
+	"Resent-" "X-Resent-"
+	"Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:")
   "*A field list to be deleted when resending."
   :group 'mew-draft
-  :type '(repeat string))
+  :type '(choice (const nil)
+		 (repeat :tag "Field list" (string :tag "Field"))))
 
 (defcustom mew-field-delete-for-forwarding
   '("Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:")
   "*A field list to be deleted when forwarding."
   :group 'mew-draft
-  :type '(repeat string))
+  :type '(choice (const nil)
+		 (repeat :tag "Field list" (string :tag "Field"))))
 
 (defcustom mew-field-delete-for-saving ()
   "*A field list to be deleted when saving a message to a file."
   :group 'mew-summary
-  :type '(repeat string))
+  :type '(choice (const nil)
+		 (repeat :tag "Field list" (string :tag "Field"))))
 
 (defcustom mew-field-delete-for-joining
-  `(,mew-mv: ,mew-subj: ,mew-message-id: "Encrypted" "Content-")
+  (list mew-mv: mew-subj: mew-message-id: "Encrypted" "Content-")
   "*A field list to be deleted when joining."
   :group 'mew-summary
-  :type '(repeat string))
+  :type '(repeat :tag "Field list" (string :tag "Field")))
 
 (defcustom mew-field-for-printing
   '("Subject:" "From:" "To:" "Cc:" "Date:")
   "*A field list to print"
   :group 'mew-summary
-  :type '(repeat string))
+  :type '(repeat :tag "Field list" (string :tag "Field")))
   
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -2378,13 +2494,6 @@ Otherwise, the old cases are overridden."
   :group 'mew-draft
   :type 'boolean)
 
-(defcustom mew-case-synchronize t
-  "'\\<mew-summary-mode-map>If non-nil, \\[mew-summary-set-case] sets both
-'mew-case-input' and 'mew-case-output'. If nil, \\[mew-summary-set-case] sets 'mew-case-input' only. 
-Use \\[universal-argument]\\[mew-summary-set-case] to set 'mew-case-output'."
-  :group 'mew-summary
-  :type 'boolean)
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Temporary files
@@ -2421,7 +2530,7 @@ guys can wiretap the temporary file."
 
 (defcustom mew-auto-get t
   "If *non-nil*, Mew gets messages from your mailbox automatically. 
-\"\\[universal-argument] \\[mew]\" equals to \"\\[mew]\" with 'mew-auto-get'
+'\\[universal-argument] \\[mew]' equals to '\\[mew]' with 'mew-auto-get'
 revered."
   :group 'mew-summary
   :type 'boolean)
@@ -2451,7 +2560,7 @@ The first member is displayed as a default value."
   :type '(choice (const nil) (repeat string)))
 
 (defcustom mew-sort-default-key "date"
-  "*Default sort key when inputing a sort key. 
+  "*Default sort key when inputting a sort key. 
 Its format is key:mode where more is found in 'mew-sort-mode'."
   :group 'mew-summary
   :type 'string)
@@ -2496,13 +2605,13 @@ MODE is one of \"date\" (by chronological order) or
 (defvar mew-range-str-sync   "sync")
 
 (defcustom mew-input-range-list
-  `(,mew-range-str-update ,mew-range-str-all ,mew-range-str-last)
+  (list mew-range-str-update mew-range-str-all mew-range-str-last)
   "*A list used by range completion."
   :group 'mew-complete
   :type '(repeat string))
 
 (defcustom mew-input-range-remote-list
-  `(,mew-range-str-update ,mew-range-str-all ,mew-range-str-last ,mew-range-str-sync)
+  (list mew-range-str-update mew-range-str-all mew-range-str-last mew-range-str-sync)
   "*A list used by range completion."
   :group 'mew-complete
   :type '(repeat string))
@@ -2693,10 +2802,7 @@ appropriate for this value."
                  (const pgp-encryption)
                  (const pgp-signature-encryption)
 		 (const pgp-encryption-signature)
-		 (const smime-signature)
-		 (const smime-encryption)
-		 (const smime-signature-encryption)
-		 (const smime-encryption-signature)
+		 (const smime-signature) ;; xxx
                  (other :tag "nil" nil)))
 
 (defcustom mew-protect-privacy-encrypted nil
@@ -2721,6 +2827,12 @@ and nil are available."
 		 (const smime-encryption-signature)
                  (other :tag "nil" nil)))
 
+(defcustom mew-protect-privacy-with-old-pgp-signature nil
+  "*If non-nil and 'mew-use-old-pgp' is non-nil, 
+the old fashioned PGP message are used when composing."
+  :group 'mew-privacy
+  :type 'boolean)
+
 (defcustom mew-passwd-timer-unit 10
   "*Minutes of timer unit to cancel the cached passwords."
   :group 'mew-privacy
@@ -2754,7 +2866,7 @@ your public key as well as receivers' one."
     "x-pgp5-public-key-url:" "x-public-key:"))
    
 (defvar mew-pgp-pks-port "11371")
-(defvar mew-pgp-pks-lang "/pks/lookup?op=get&exact=on&search=")
+(defvar mew-pgp-pks-lang "/pks/lookup?op=get&search=")
 
 ;; Tue Dec 16 16:28:11 JST 2003
 ;; http://pgp.zdv.uni-mainz.de/keyserver/bigbrother/
@@ -2983,7 +3095,7 @@ Level 2: syntax error."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2003 Mew developing team.
+;; Copyright (C) 1996-2005 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
